package il.co.cortext.rui.examples.phonebook;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;

public class PhoneBookExample {

	private Shell            shell;
	private Button           newButton, deleteButton, exitButton;
	private ContactNavigator navigator;
	private ContactEditor    editor;
	private ContactModel     selected;
	private PhoneBookModel   phoneBook;

	public static void main(String [] args) {
		Display display = new Display();
		Shell shell = new PhoneBookExample().open(display);
		while (!shell.isDisposed())
			if (!display.readAndDispatch()) display.sleep();
		display.dispose();
	}

	public Shell open(Display display) {
		shell = new Shell(display);
		shell.setLayout(new GridLayout());

		phoneBook = PhoneBookModel.createExample();
		selected = (ContactModel) phoneBook.getContacts().get(0);

		Composite toolbar = new Composite(shell, SWT.NONE);
		toolbar.setLayout(new RowLayout());

		newButton = new Button(toolbar, SWT.NONE);
		newButton.setText("New");
		newButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ContactModel newContact = new ContactModel();
				newContact.setName("new contact");
				phoneBook.getContacts().add(newContact);
				ListViewer navigatorViewer = navigator.getViewer();
				navigatorViewer.add(newContact);
				navigatorViewer.setSelection
					(new StructuredSelection(newContact), true);
					
				deleteButton.setEnabled(true);
			}
		});

		deleteButton = new Button(toolbar, SWT.NONE);
		deleteButton.setText("Delete");
		deleteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				List contacts = phoneBook.getContacts();
				int index = contacts.indexOf(selected);
				if (index != 0) index--;
				contacts.remove(selected);
				ListViewer navigatorViewer = navigator.getViewer();
				ContactModel oldSelected = selected;
				selected = contacts.isEmpty()?
					null: (ContactModel) contacts.get(index);
				if (selected != null) {
					navigatorViewer.setSelection
						(new StructuredSelection(selected), true);
				} else {
					deleteButton.setEnabled(false);
				}
				navigatorViewer.remove(oldSelected);
			}
		});

		exitButton = new Button(toolbar, SWT.NONE);
		exitButton.setText("Exit");
		exitButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e)
				{ shell.close(); }
		});

		SashForm panel = new SashForm(shell, SWT.HORIZONTAL);
		panel.setLayoutData(new GridData(GridData.FILL_BOTH));

		navigator = new ContactNavigator(panel, phoneBook);
		editor    = new ContactEditor(panel, selected);

		navigator.getViewer().addSelectionChangedListener(
			new ISelectionChangedListener() {
				public void selectionChanged(SelectionChangedEvent event) {
					ContactModel newSelected = (ContactModel)
						((IStructuredSelection) event.getSelection()).
							getFirstElement();
					editor.setInput(newSelected);
					if (selected != null)
						navigator.getViewer().update(selected, null);
					selected = newSelected;
				}
			}
		);
		
		shell.pack();
		shell.open();
		return shell;
	}


}
