package il.co.cortext.rui.remote;

import java.util.Date;

import org.eclipse.swt.widgets.Display;

public class RuiSession {

	public static final String BOOT_REQUEST  = "boot";
	public static final String EVENT_REQUEST = "event";

	private String                 id;
	private Display                display;
	private RuiApplicationManager  applicationManager;
	private long                   lastActivityTime;

	private int                    nextWindowId = 0;
	private int                    nextWidgetId = 0;
	private RuiClientEvenManager   eventManager = new RuiClientEvenManager();
	private RuiServerCommandBuffer buffer       = new RuiServerCommandBuffer();

	public RuiSession(
		RuiApplicationManager applicationManager,
		String sessionId
	) {
		if (applicationManager == null)
			throw new IllegalArgumentException("No Application Manager given");
		this.id                 = sessionId;
		this.applicationManager = applicationManager;
		restartTimer();
	}

	public String getId() { return id; }

// session manager interface --------------------------------------------------

	public void handleRequest(RuiClientEvent event) throws Exception {
		restartTimer();
		if (!event.containsKey("requestType"))
			throw new IllegalArgumentException("Event with no requestType");
		String requestType = event.getValue("requestType");
		if (requestType.equals(BOOT_REQUEST))
			handleBootRequest(event);
		else if(requestType.equals(EVENT_REQUEST))
			handleEventRequest(event);
		else throw new
			IllegalArgumentException("Unknown request type: " + requestType);
	}

	public String flush() { return buffer.flush(); }

// UI proxy interface ---------------------------------------------------------

	public void addCommand(RuiServerCommand command)
		{ buffer.addCommand(command); }

	public void setListener(String sourceId, IRuiClientListener listener)
		{ eventManager.setListener(sourceId, listener); }

	public void removeListener(String sourceId)
		{ eventManager.removeListener(sourceId); }

	public String makeWindowId() { return "W" + ++nextWindowId; }
	public String makeWidgetId() { return "E" + ++nextWidgetId; }

	public Display getDisplay() { return display; }

	public void setDisplay(Display display) { this.display = display; }

// request handling -----------------------------------------------------------

	private void handleBootRequest(RuiClientEvent event) throws Exception {
		assert applicationManager != null;
		String applicationName = event.getValue("applicationName");
		if (applicationName == null || applicationName.equals(""))
			applicationName = "DEFAULT";
		applicationManager.start(applicationName);
	}

	private void handleEventRequest(RuiClientEvent event) throws Exception {
		eventManager.fireEvent(event);
	}

// session timeout ------------------------------------------------------------

	public boolean showsActivity(long timeoutMiliSeconds) {
		return (new Date().getTime() - lastActivityTime) < timeoutMiliSeconds;
	}

	private void restartTimer() { lastActivityTime = new Date().getTime(); }

}
