package il.co.cortext.rui.remote;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class RuiSessionManager {

	private RuiApplicationManager applicationManager;
	private Map sessions = Collections.synchronizedMap(new HashMap());
	private int nextSessionId = 0;
	private static ThreadLocal threadSession = new ThreadLocal();
	private static RuiSessionManager instance = new RuiSessionManager();

	public static RuiSessionManager getInstance () { return instance; }

	private RuiSessionManager() {}

// server interface -----------------------------------------------------------

	public void setApplicationManager(
		RuiApplicationManager applicationManager
	) { this.applicationManager = applicationManager; }

	public String handleRequest(RuiClientEvent event) throws Exception {
		assert applicationManager != null;
		String sessionId   = event.getValue("sessionId");
		RuiSession session = sessionId == null?
			createSession(): getSessionById(sessionId);
		attachSessionToThread(session);

//System.out.print("event recieved: " + event);

		session.handleRequest(event);
		detachSessionFromThread();
		String message = session.flush();

//System.out.print(RuiServerCommand.translateToText(message));

		return session.getId() + "\n" + message;
	}

// UI proxy interface ---------------------------------------------------------

	public RuiSession getSessionByThread() {
		assert threadSession.get() != null;
		return (RuiSession) threadSession.get();
	}

// session management ---------------------------------------------------------

	private synchronized RuiSession createSession() throws Exception {
		assert applicationManager != null;
		String sessionId = makeSessionId();
		RuiSession session = new RuiSession(applicationManager, sessionId);
		sessions.put(sessionId, session);
		return session;
	}

	private synchronized void removeSession(RuiSession session)
		{ sessions.remove(session.getId()); }

	public RuiSession getSessionById(String sessionId) throws Exception {
		if (!sessions.containsKey(sessionId))
			throw new Exception("Session not found: " + sessionId);
		RuiSession session = (RuiSession) sessions.get(sessionId);
		return session;
	}

	public void attachSessionToThread(RuiSession session)
		{ threadSession.set(session); }

	public void detachSessionFromThread() { threadSession.set(null); }

	private synchronized String makeSessionId()
		{ return "S" + ++nextSessionId; }

// testing --------------------------------------------------------------------

	public void reset() {
		detachSessionFromThread();
		sessions = Collections.synchronizedMap(new HashMap());
		nextSessionId = 0;
	}
}
