package org.eclipse.swt.layout;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;

public final class GridData {

	public int     verticalAlignment         = CENTER;
	public int     horizontalAlignment       = BEGINNING;
	public int     widthHint                 = SWT.DEFAULT;
	public int     heightHint                = SWT.DEFAULT;
	public int     horizontalIndent          = 0;
	public int     horizontalSpan            = 1;
	public int     verticalSpan              = 1;
	public boolean grabExcessHorizontalSpace = false;
	public boolean grabExcessVerticalSpace   = false;

	// Alignment constants.
	public static final int BEGINNING = 1;
	public static final int CENTER    = 2;
	public static final int END       = 3;
	public static final int FILL      = 4;

	// Style constants
	public static final int VERTICAL_ALIGN_BEGINNING   = 1 << 1;
	public static final int VERTICAL_ALIGN_CENTER      = 1 << 2;
	public static final int VERTICAL_ALIGN_END         = 1 << 3;
	public static final int VERTICAL_ALIGN_FILL        = 1 << 4;
	public static final int HORIZONTAL_ALIGN_BEGINNING = 1 << 5;
	public static final int HORIZONTAL_ALIGN_CENTER    = 1 << 6;
	public static final int HORIZONTAL_ALIGN_END       = 1 << 7;
	public static final int HORIZONTAL_ALIGN_FILL      = 1 << 8;
	public static final int GRAB_HORIZONTAL            = 1 << 9;
	public static final int GRAB_VERTICAL              = 1 << 10;

	// Style convenience constants
	public static final int FILL_VERTICAL   = VERTICAL_ALIGN_FILL   | GRAB_VERTICAL;
	public static final int FILL_HORIZONTAL = HORIZONTAL_ALIGN_FILL | GRAB_HORIZONTAL;
	public static final int FILL_BOTH       = FILL_VERTICAL         | FILL_HORIZONTAL;

	public GridData(int style) {
		if ((style & VERTICAL_ALIGN_BEGINNING) != 0)
			verticalAlignment = BEGINNING;
		if ((style & VERTICAL_ALIGN_CENTER) != 0)
			verticalAlignment = CENTER;
		if ((style & VERTICAL_ALIGN_FILL) != 0)
			verticalAlignment = FILL;
		if ((style & VERTICAL_ALIGN_END) != 0)
			verticalAlignment = END;

		if ((style & HORIZONTAL_ALIGN_BEGINNING) != 0)
			horizontalAlignment = BEGINNING;
		if ((style & HORIZONTAL_ALIGN_CENTER) != 0)
			horizontalAlignment = CENTER;
		if ((style & HORIZONTAL_ALIGN_FILL) != 0)
			horizontalAlignment = FILL;
		if ((style & HORIZONTAL_ALIGN_END) != 0)
			horizontalAlignment = END;

		if ((style & GRAB_HORIZONTAL) != 0)
			grabExcessHorizontalSpace = true;
		else
			grabExcessHorizontalSpace = false;
		if ((style & GRAB_VERTICAL) != 0)
			grabExcessVerticalSpace = true;
		else
			grabExcessVerticalSpace = false;
	}

	public Map getState() {
		Map state = new HashMap();
		state.put("indent", "" + horizontalIndent);
		state.put("hAlign", translateAlignment(horizontalAlignment));
		state.put("vAlign", translateAlignment(verticalAlignment));
		state.put("hSpan",  "" + horizontalSpan);
		state.put("vSpan",  "" + verticalSpan);
		state.put("hFlex",  grabExcessHorizontalSpace? "1": "0");
		state.put("vFlex",  grabExcessVerticalSpace?   "1": "0");
		if (widthHint  != SWT.DEFAULT) state.put("width",  "" + widthHint);
		if (heightHint != SWT.DEFAULT) state.put("height", "" + heightHint);
		return state;
	}

	private String translateAlignment(int align) {
		return
			align == BEGINNING? "begin":
			align == CENTER?    "center":
			align == END?       "end":
			"fill"; 
	}

}
