package org.eclipse.swt.widgets;

import il.co.cortext.rui.remote.RuiClientEvent;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

public class Button extends Label {

	private boolean isSelected = false;

	public Button(Composite parent, int style) { super(parent, style); }

// selection management -------------------------------------------------------

	public boolean getSelection() {
		if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return false;
		return isSelected;
	}

	public void setSelection(boolean isSelected) {
		if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
		this.isSelected = isSelected;
		if ((style & (SWT.RADIO)) != 0) unselectRadioSiblings();
		sendToClient(METHOD_VALUE, isSelected? "1": "0");
	}

// radio support --------------------------------------------------------------

	private void unselectRadio() { isSelected = false; }

	private void unselectRadioSiblings() {
		Control[] siblings = parent.getChildren();
		for (int i = 0; i < siblings.length; i++) {
			Control child = siblings[i];
			if (
				this != child &&
				child instanceof Button &&
				(child.getStyle() & SWT.RADIO) != 0
			) ((Button) child).unselectRadio();
		}
	}

// event handling -------------------------------------------------------------

	public void addSelectionListener(SelectionListener listener) {
		if (listener == null) error(SWT.ERROR_NULL_ARGUMENT);
		TypedListener typedListener = new TypedListener(listener);
		addListener(SWT.Selection, typedListener);
	}

	public void removeSelectionListener(SelectionListener listener) {
		if (listener == null) error(SWT.ERROR_NULL_ARGUMENT);
		if (eventTable == null) return;
		eventTable.unhook(SWT.Selection, listener);
	}

// template methods -----------------------------------------------------------

	public Event translateEvent(RuiClientEvent clientEvent) throws Exception {
		Event event = new Event();
		event.type = SWT.Selection;
		if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
			String value = clientEvent.getValue("value");
			assert value != null;
			isSelected = value.equals("1")? true: false;
			if ((style & (SWT.RADIO)) != 0) unselectRadioSiblings();
		}
		return event;
	}

	protected String getDefaultProxyClassName() {
		return
			((style & SWT.CHECK) != 0)? CLASS_CHECKBOX:
			((style & SWT.RADIO) != 0)? CLASS_RADIOBOX:
			CLASS_BUTTON;
	}

	protected String getSetTextProxyMethodName() {
		return
			((style & SWT.CHECK) != 0)? METHOD_LABEL:
			((style & SWT.RADIO) != 0)? METHOD_LABEL:
			METHOD_VALUE;
	}

}
