package org.eclipse.swt.widgets;

import java.util.ArrayList;

import il.co.cortext.rui.remote.RuiClientEvent;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

public class Combo extends Control {

	private final int NO_SELECTION = -1;

	private ArrayList items   = new ArrayList();
	private int selectedIndex = NO_SELECTION;

	public Combo(Composite parent, int style) { super(parent, style); }

// item management ------------------------------------------------------------

	public String getItem(int index)   { return (String) items.get(index); }
	public int    getItemCount ()      { return items.size(); }
	public int    indexOf(String item) { return indexOf(item, 0); }
	public void   add(String item)     { add(item, items.size()); }

	public int indexOf(String item, int start) {
		if (item == null) error(SWT.ERROR_NULL_ARGUMENT);
		return items.indexOf(item);
	}

	public String[] getItems() {
		int count = getItemCount();
		String[] result = new String[count];
		for (int i = 0; i < count; i++) result[i] = getItem(i);
		return result;
	}

	public void add(String item, int index) {
		if (item == null) error(SWT.ERROR_NULL_ARGUMENT);
		items.add(index, item);
		sendToClient(METHOD_INSERT, "" + index, item);
	}

	public void setItem(int index, String item) {
		if (item == null) error(SWT.ERROR_NULL_ARGUMENT);
		remove(index);
		add(item, index);
		if (index == selectedIndex) select(index);
	}

	public void setItems(String[] items) {
		if (items == null) error(SWT.ERROR_NULL_ARGUMENT);
		removeAll();
		for (int i = 0; i < items.length; i++) add(items[i], i);
	}

	public void remove(int index) {
		items.remove(index);
		sendToClient(METHOD_REMOVEBYINDEX, "" + index);
		if (selectedIndex == index) selectedIndex = NO_SELECTION;
	}

	public void removeAll() {
		items = new ArrayList();
		selectedIndex = NO_SELECTION;
		sendToClient(METHOD_CLEAR);
	}

// selection management -------------------------------------------------------

	public int     getSelectionIndex() { return selectedIndex; }
	public void    deselectAll()       { select(NO_SELECTION); }

	public void select(int index) {
		selectedIndex = index;
		sendToClient(METHOD_SELECTEDINDEX, "" + index);
	}

	public void deselect(int index) {
		if (index == -1) return;
		if (selectedIndex == index) select(NO_SELECTION);
	}

// event handling -------------------------------------------------------------

	public void addSelectionListener(SelectionListener listener) {
		if (listener == null) error(SWT.ERROR_NULL_ARGUMENT);
		TypedListener typedListener = new TypedListener(listener);
		addListener(SWT.Selection, typedListener);
	}

	public void removeSelectionListener(SelectionListener listener) {
		if (listener == null) error(SWT.ERROR_NULL_ARGUMENT);
		if (eventTable == null) return;
		eventTable.unhook(SWT.Selection, listener);
	}

// template methods -----------------------------------------------------------

	protected Event translateEvent(RuiClientEvent clientEvent)
		throws Exception {
		selectedIndex = Integer.parseInt(clientEvent.getValue("selectedIndex"));
		Event event   = new Event();
		event.type    = SWT.Selection;
		return event;
	}

	protected String getDefaultProxyClassName() { return CLASS_COMBO; }

}
