package org.eclipse.swt.widgets;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

public class Composite extends Control {

	private Layout    layout;
	private ArrayList children = new ArrayList();

	Composite() {}

	public Composite(Composite parent, int style) { super(parent, style); }

	Composite(Composite parent, int style, String proxyClassName)
		{ super(parent, style, proxyClassName); }

	protected void releaseChildrenFromSession() {
		for (Iterator iter = children.iterator(); iter.hasNext();)
			((Control) iter.next()).releaseFromSession();
	}

// child management -----------------------------------------------------------

	protected String addChild(Widget child) {
		children.add(child);
		return getWidgetId();
	}

	protected void removeChild(Widget child) { children.remove(child); }

	public Control[] getChildren() {
		Object[] oldChildren  = children.toArray();
		Control[] newChildren = new Control[oldChildren.length];
		for (int i = 0; i < oldChildren.length; i++)
			newChildren[i] = (Control) oldChildren[i];
		return newChildren;
	}

// layout ---------------------------------------------------------------------

	public Object getLayout()              { return layout; }
	public void   setLayout(Layout layout) { this.layout = layout; }
	public void   pack(boolean changed)    { layout(); }
	public void   layout()                 { layout(true); }

	public void layout(boolean changed) {
		if (layout != null) {
			sendToClient(METHOD_LAYOUTMANAGER, "name", layout.getName());
			Map state = layout.getState();
			for (Iterator i = state.keySet().iterator(); i.hasNext();) {
				String key = (String) i.next();
				sendToClient(METHOD_LAYOUTMANAGER, key, (String) state.get(key));
			}
		}
		layoutChildren();
	}

	private void layoutChildren() {
		for (Iterator iter = children.iterator(); iter.hasNext();) {
			Control child = (Control) iter.next();
			Object layoutData = child.getLayoutData();
			if (layoutData != null && layout!= null)
				child.sendLayoutDataToClient
					(layout.mapChildLayoutData(layoutData));
			if (child instanceof Composite) ((Composite) child).layout();
		}
	}

// template methods -----------------------------------------------------------

	protected String getDefaultProxyClassName() { return CLASS_PANEL; }

}
