package org.eclipse.swt.widgets;

import il.co.cortext.rui.remote.RuiServerCommand;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;

public class Shell extends Composite {

	private Composite mainPanel;
	private Display   display;
	private String    titleText;

	Shell() {}

	Shell(Display display, String proxyClassName) {
		this.display = display;
		this.proxyClassName = (proxyClassName == null)?
			getDefaultProxyClassName(): proxyClassName;
		String id = getSession().makeWindowId();
		init(id, WINDOW_WIDGETID, id);
		sendToClient(METHOD_CREATE);
		mainPanel = new Composite(this, SWT.NONE);
	}

	public Shell(Display display) { this(display, null); }

	public  void      open()         {}
	public  void      close()        { super.dispose(); }
	public  Shell     getShell()     { return this; }
	public  Display   getDisplay()   { return display; }
	public  Composite getMainPanel() { return mainPanel; }

	protected void releaseChildrenFromSession()
		{ mainPanel.releaseFromSession(); }

	// hack! we need event loop to short circuits until we do jface window
	public boolean isDisposed() { return true; }

// accessing ------------------------------------------------------------------

	public String getText() { return titleText; }

	public void setText(String titleText) {
		this.titleText = titleText;
		sendToClient(METHOD_TITLE, titleText);
	}

// child management -----------------------------------------------------------

	protected String addChild(Widget child) {
		return (mainPanel == null)?
			super.addChild(child):
			mainPanel.addChild(child);
	}

	protected void removeChild(Widget child)          { mainPanel.removeChild(child); }
	public Control[] getChildren()                     { return mainPanel.getChildren(); }

// layout ---------------------------------------------------------------------

	public Object getLayoutData()                     { return mainPanel.getLayoutData(); }
	public Object getLayout()                         { return mainPanel.getLayout(); }
	public void   setLayoutData(Object layoutData)    { mainPanel.setLayoutData(layoutData); }
	public void   setLayout(Layout layout)            { mainPanel.setLayout(layout); }
	public void   pack()                              { mainPanel.pack(); }
	public void   pack(boolean changed)               { mainPanel.pack(changed); }
	public void   layout()                            { mainPanel.layout(); }
	public void   layout(boolean changed)             { mainPanel.layout(changed); }

// style ----------------------------------------------------------------------

	public Color getBackground()                      { return mainPanel.getBackground(); }
	public Color getForeground()                      { return mainPanel.getForeground(); }
	public void  setBackground(Color backgroundColor) { mainPanel.setBackground(backgroundColor); }
	public void  setForeground(Color foregroundColor) { mainPanel.setForeground(foregroundColor); }

// misc control features ------------------------------------------------------

	public String  getTooltipText()                   { return mainPanel.getTooltipText(); }
	public boolean getEnabled()                       { return mainPanel.getEnabled(); }
	public boolean getVisible()                       { return mainPanel.getVisible(); }
	public void    setToolTipText(String tooltipText) { mainPanel.setToolTipText(tooltipText); }
	public void    setEnabled(boolean isEnabled)      { mainPanel.setEnabled(isEnabled); }
	public void    setVisible(boolean isVisible)      { mainPanel.setVisible(isVisible); }

// UI proxy interface ---------------------------------------------------------

	protected void sendToClient(String methodName, String[] params) {
		String windowId = getWindowId();
		assert windowId != null;
		RuiServerCommand command = new RuiServerCommand
			(getDefaultProxyClassName(), methodName, windowId, params);
		sendToClient(command);
	}

// state management -----------------------------------------------------------

	public String getWidgetId() {
		return mainPanel == null? super.getWidgetId(): mainPanel.getWidgetId();
	}

// template methods -----------------------------------------------------------

	protected String getDefaultProxyClassName() { return CLASS_WINDOW; }

}
