package org.eclipse.swt.widgets;


import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import il.co.cortext.rui.remote.RuiClientEvent;
import java.util.StringTokenizer;
import java.util.ArrayList;
import org.eclipse.swt.SWT;

public class Tree extends Composite {
	TreeItem [] items;
	TreeItem [] selectedItems;
	private final int NO_SELECTION = -1;
	private ArrayList roots  = new ArrayList();

	public Tree (Composite parent, int style) {
		super (parent, style);
		createWidget();
	}

// item creators and destructors-----------------------------------------------------------

	void createItem (TreeItem item) {
		item.foreground = item.background = -1;
		int id = 0;
		while (id < items.length && items [id] != null) id++;
		if (id == items.length) {
			TreeItem [] newItems = new TreeItem [items.length + 4];
			System.arraycopy (items, 0, newItems, 0, items.length);
			items = newItems;
		}
		sendToClient(METHOD_INSERT,pathToString(getPath(item)));
		items [id] = item;
		item.id = id;
	}
	
	void destroyItem (TreeItem item) {
		sendToClient(METHOD_REMOVEBYPATH ,pathToString(getPath(item)));
		items [item.id] = null;
	}

	
// roots---------------------------------------------

	void addRoot(TreeItem item ,int index) {
		roots.add(index,item);
	}
	
	void addRoot(TreeItem item) {
			roots.add(item);
	}
	
	void removeRoot(TreeItem item) {
			roots.remove(item);
	}
	
// helpers---------------------------------------------

	void createWidget () {
		items = new TreeItem [4];
	}
	
	public String pathToString(int[] path) {
		String sPath =String.valueOf(path[0]);
		for (int i=1;i<path.length;i++ ) {
			sPath = sPath+ "_"+ String.valueOf(path[i]) ;
		};
		return sPath ;
	}
	
	public int[] getPath (TreeItem item) {
		if (item == null) error (SWT.ERROR_NULL_ARGUMENT);

		TreeItem temp = item;
		TreeItem parentItem = temp.getParentItem();
		int pathDepth = 0;
		while (parentItem  != null) {
			pathDepth++;
			temp       = parentItem;
			parentItem = temp.getParentItem();
		}

		int[] path = new int[pathDepth +1];
		int i;
		for (i=pathDepth;i>0;i-- ) {
			parentItem = item.getParentItem();
			path[i] = parentItem.indexOf(item);
			item = parentItem;
		};
		path[i++] = roots.indexOf(item);
		return path;
	}
	
// node accessing---------------------------------------------

	void setNodeValue(TreeItem item,String text) {
		sendToClient(METHOD_NODEVALUE,pathToString(getPath(item)),text);
	}

	void setNodeImage(TreeItem item,Image image) {
		String[] args = {pathToString(getPath(item)), image.getFileName(),"" + image.getWidth(),"" + image.getHeight()};
		sendToClient(METHOD_NODEIMAGE, args);
	}

// event handling -------------------------------------------------------------

	public void addSelectionListener(SelectionListener listener) {
		if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
		TypedListener typedListener = new TypedListener (listener);
		addListener (SWT.Selection, typedListener);
		addListener (SWT.DefaultSelection, typedListener);
	}

	public void removeSelectionListener (SelectionListener listener) {
		if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
		eventTable.unhook (SWT.Selection, listener);
		eventTable.unhook (SWT.DefaultSelection, listener);
	}
	
// selection -----------------------------------------------------------

	public TreeItem [] getSelection () {
		return selectedItems;
	}
	
	private void setSelection (TreeItem [] items) {
		if (items == null) error (SWT.ERROR_NULL_ARGUMENT);
		selectedItems =  items;
	}
	
	public void deselectAll () {
			selectedItems = null;
			sendToClient(METHOD_SELECTEDPATH, "" + NO_SELECTION);
	}
	
	private void setSelection (int[] itemPath) {
		selectedItems =  new TreeItem [1];
		selectedItems[0] = findItemBypath(itemPath);
		sendToClient(METHOD_SELECTEDPATH, pathToString(itemPath));
	}
	
	private TreeItem findItemBypath(int[] path) {
		int i = 0;
		TreeItem item = (TreeItem) roots.get(path[i]);
		for (i=1;i<path.length;i++)
			item = (item.getItems())[path[i]];
		return item;
	}
	
// template methods -----------------------------------------------------------
	
	protected Event translateEvent(RuiClientEvent clientEvent)
		throws Exception {
		StringTokenizer tokenizer = new StringTokenizer(clientEvent.getValue("selectedPath"), "_");
		int pathDepth =tokenizer.countTokens();
		int[] path = new int[pathDepth];
		for (int i=0;i<pathDepth;i++ ) {
			path[i] = Integer.parseInt(tokenizer.nextToken());
		};
		setSelection(path);
		Event event   = new Event();
		event.type    = SWT.Selection;
		return event;
	}

	protected String getDefaultProxyClassName() { return CLASS_TREE; }

}
