package org.eclipse.swt.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import java.util.ArrayList;

public class TreeItem extends Item {

	Tree parent;
	int background, foreground, id;
	TreeItem parentItem;
	private ArrayList items = new ArrayList();
	
	public TreeItem (Tree parent, int style) {
		super(parent, style);
		this.parent     = parent;
		this.parentItem = null;
		parent.addRoot(this);
		parent.createItem(this);
	}

	public TreeItem (Tree parent, int style, int index) {
		super(parent, style);
		if (index < 0) error(SWT.ERROR_INVALID_RANGE);
		this.parent     = parent;
		this.parentItem = null;
		parent.addRoot(this, index);
		parent.createItem(this);
	}


	public TreeItem (TreeItem parentItem, int style)
		{ this(parentItem, style, parentItem.items.size()); }

	public TreeItem (TreeItem pItem, int style, int index) {
		super(pItem.parent, style);
		if (index < 0) error(SWT.ERROR_INVALID_RANGE);
		parent          = pItem.parent;
		this.parentItem = pItem;
		parentItem.items.add(index, this);
		parent.createItem(this);
	}
// tree methods----------------------------------------------------

	public Tree getParent () { return parent; }
	
	public TreeItem getParentItem () { return parentItem; }
	
	public int indexOf(TreeItem item) {
		if (item == null) error(SWT.ERROR_NULL_ARGUMENT);
		return items.indexOf(item);
	}

	public TreeItem [] getItems () {
		TreeItem [] tIetems =  new TreeItem [items.size()];
		for ( int i = 0; i < items.size(); i++)
			tIetems[i] = (TreeItem)items.get(i);
		return tIetems;
	}
//	public TreeItem getChild(int index)
//	{ return (TreeItem) items.get(index);	}
	
// accessing ------------------------------------------------------------------

	public void setImage (Image image) {
		super.setImage(image);
		parent.setNodeImage(this, image);		
	}

	public void setText (String string) {
		if (string == null) error(SWT.ERROR_NULL_ARGUMENT);
		super.setText(string);
		parent.setNodeValue(this, string);
	}
	
// destruction----------------------------------------------

// move to widget
	public void dispose () {
		if (isDisposed()) return;
		releaseChild();
		releaseWidget();
		super.dispose();
	}
	
	void releaseWidget () {
		super.releaseWidget();
		parent = null;
	}
	
	void releaseChild () {
//		super.releaseChild();
		parent.destroyItem(this);
		if (parentItem != null) 
			parentItem.items.remove(this);
		else
			parent.removeRoot(this);
	}

// widget/item implementation----------------------------------------------

	public Display getDisplay () {
		Tree parent = this.parent;
		if (parent == null) error(SWT.ERROR_WIDGET_DISPOSED);
		return parent.getDisplay();
	}
	
}
