package org.eclipse.swt.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeListener;

public abstract class Widget {

	private Object   data;
	private String[] keys;
	private Object[] values;
	private boolean  isDisposed = false;

	protected EventTable eventTable;
	protected int        style;

	protected Widget() {}

	public Widget (Widget parent, int style) { this.style = style; }

	public void dispose() { isDisposed = true; }

	public boolean isDisposed() { return isDisposed; }

	public int getStyle() { return style; }

	public Object getData() { return data; }

	public void setData(Object data) { this.data = data; }

	public Object getData(String key) {
		if (key == null) error(SWT.ERROR_NULL_ARGUMENT);
		if (keys == null) return null;
		for (int i = 0; i < keys.length; i++)
			if (keys [i].equals (key)) return values[i];
		return null;
	}

	public void setData(String key, Object value) {
		if (key == null) error(SWT.ERROR_NULL_ARGUMENT);

		/* Remove the key/value pair */
		if (value == null) {
			if (keys == null) return;
			int index = 0;
			while (index < keys.length && !keys[index].equals(key)) index++;
			if (index == keys.length) return;
			if (keys.length == 1) {
				keys   = null;
				values = null;
			} else {
				String [] newKeys   = new String[keys.length - 1];
				Object [] newValues = new Object[values.length - 1];
				System.arraycopy(keys, 0, newKeys, 0, index);
				System.arraycopy(keys, index + 1, newKeys, index, newKeys.length - index);
				System.arraycopy(values, 0, newValues, 0, index);
				System.arraycopy(values, index + 1, newValues, index, newValues.length - index);
				keys   = newKeys;
				values = newValues;
			}
			return;
		}

		/* Add the key/value pair */
		if (keys == null) {
			keys   = new String[] {key};
			values = new Object[] {value};
			return;
		}
		for (int i = 0; i < keys.length; i++) {
			if (keys[i].equals (key)) {
				values[i] = value;
				return;
			}
		}
		String[] newKeys   = new String[keys.length + 1];
		Object[] newValues = new Object[values.length + 1];
		System.arraycopy(keys, 0, newKeys, 0, keys.length);
		System.arraycopy(values, 0, newValues, 0, values.length);
		newKeys[keys.length]     = key;
		newValues[values.length] = value;
		keys   = newKeys;
		values = newValues;
	}

	protected void error(int code) { SWT.error(code); }

	public abstract Display getDisplay();

// event handling -------------------------------------------------------------

	public void addListener(int eventType, Listener listener) {
		if (listener == null) error(SWT.ERROR_NULL_ARGUMENT);
		if (eventTable == null) eventTable = new EventTable();
		eventTable.hook(eventType, listener);
	}

	public void removeListener(int eventType, Listener listener) {
		if (listener == null) error(SWT.ERROR_NULL_ARGUMENT);
		if (eventTable == null) return;
		eventTable.unhook(eventType, listener);
	}

	public void notifyListeners(int eventType, Event event) {
		if (eventTable == null) return;
		event.widget = this;
		eventTable.sendEvent(event);
	}

// mock methods for SWT compatibility -----------------------------------------

	public    void addDisposeListener    (DisposeListener listener) {}
	public    void removeDisposeListener (DisposeListener listener) {}
	protected void checkWidget           ()                         {}
	          void releaseWidget         ()                         {}

}
