package org.eclipse.jface.viewers.tests;

import java.util.HashMap;

import il.co.cortext.rui.examples.phonebook.ContactModel;
import il.co.cortext.rui.examples.phonebook.PhoneBookModel;

import junit.framework.TestCase;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.tests.ControlTestHelper;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;

public class ListViewerTest extends TestCase {

	private ListViewer            subject;
	private PhoneBookModel        phoneBook;
	private Shell                 shell;
	private Display               display;
	private ControlTestHelper     helper;
	private SelectionChangedEvent event;

	public ListViewerTest(String name) { super(name); }

	protected void setUp() throws Exception {
		helper    = new ControlTestHelper();
		display   = new Display();
		shell     = new Shell(display);
		phoneBook = PhoneBookModel.createExample();
		subject   = new ListViewer
			(shell, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);

		subject.setContentProvider(new IStructuredContentProvider() {
			public void inputChanged(Viewer viewer, Object oldObject, Object newObject) {}
			public void dispose() {}
			public Object[] getElements(Object inputElement)

				{ return ((PhoneBookModel) inputElement).getContacts().toArray(); }

		});

		subject.setLabelProvider(new LabelProvider() {
			public String getText(Object element)
				{ return ((ContactModel) element).getName(); }
		});
		subject.setInput(phoneBook);
	}

	protected void tearDown() { helper.tearDown(); }

	public void testControl() {
		List control = subject.getList();
		assertEquals(2, control.getItemCount());
	}

	public void testSelection() throws Exception {
		subject.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e)
				{ event = e; }
		});
		event = null;
		HashMap clientEvent = new HashMap();
		clientEvent.put("name", "SelectionChanged");
		clientEvent.put("selectedIndex", "1");
		clientEvent.put("sourceId", subject.getList().getWidgetId());
		helper.fireEvent(clientEvent);
		ContactModel contact = (ContactModel)
			((IStructuredSelection) event.getSelection()).getFirstElement();
		assertNotNull(event);
		assertNotNull(contact);
		assertEquals("Don Joe", contact.getName());
	}

}
