package org.eclipse.swt.widgets.tests;

import il.co.cortext.rui.remote.IRuiConstants;
import java.util.HashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public class ButtonTest extends LabelTest {

	private Button subject;
	private SelectionEvent event;

	public ButtonTest(String name) { super(name); }

	protected void setUp() throws Exception {
		super.setUp();
		subject = (Button) getSubject();
	}

	public void testSetText() throws Exception {
		subject = (Button) makeSubject(shell, SWT.CHECK);
		flush();
		subject.setText("foo");
		assertBuffer(subject, IRuiConstants.METHOD_LABEL, "foo");

		subject = (Button) makeSubject(shell, SWT.RADIO);
		flush();
		subject.setText("foo");
		assertBuffer(subject, IRuiConstants.METHOD_LABEL, "foo");
	}

	public void testEvents() throws Exception {
		event = null;
		SelectionListener listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent inEvent)
				{ event = inEvent; }
		};
		subject.addSelectionListener(listener);

		HashMap clientEvent = new HashMap();
		clientEvent.put("name", "Click");
		clientEvent.put("sourceId", subject.getWidgetId());
		fireEvent(clientEvent);
		assertNotNull(event);
		assertEquals(subject, event.getSource());

		event = null;
		fireEvent(clientEvent);
		assertNotNull(event);
		assertEquals(subject, event.getSource());

		event = null;
		subject.removeSelectionListener(listener);
		fireEvent(clientEvent);
		assertNull(event);

		subject.dispose();
		boolean flag = false;
		try { fireEvent(clientEvent); } catch(Exception e) { flag = true; }
		if (!flag) { fail("no posthomous event exception thrown"); } 
	}

	public void testValueEventsCheck() throws Exception
		{ _testValueEvents((Button) makeSubject(shell, SWT.CHECK)); }

	public void testValueEventsRadio() throws Exception
		{ _testValueEvents((Button) makeSubject(shell, SWT.RADIO)); }

	public void testRadio() throws Exception {
		Composite parent = new Composite(shell, SWT.NONE);
		Composite child1 = new Composite(parent, SWT.NONE);
		Button radio1 = new Button(parent, SWT.RADIO);
		Button radio2 = new Button(parent, SWT.RADIO);
		Button radio3 = new Button(parent, SWT.RADIO);
		Composite child2 = new Composite(parent, SWT.NONE);

		radio1.setSelection(true);
		assert(!radio2.getSelection());
		assert(!radio3.getSelection());
		radio2.setSelection(true);
		assert(!radio1.getSelection());
		assert(!radio3.getSelection());
		radio3.setSelection(true);
		assert(!radio1.getSelection());
		assert(!radio2.getSelection());

		HashMap clientEvent = new HashMap();
		clientEvent.put("name", "Click");
		clientEvent.put("sourceId", radio1.getWidgetId());
		clientEvent.put("value", "1");
		fireEvent(clientEvent);
		assert(!radio2.getSelection());
		assert(!radio3.getSelection());
		clientEvent.put("sourceId", radio2.getWidgetId());
		fireEvent(clientEvent);
		assert(!radio1.getSelection());
		assert(!radio3.getSelection());
		clientEvent.put("sourceId", radio3.getWidgetId());
		fireEvent(clientEvent);
		assert(!radio1.getSelection());
		assert(!radio2.getSelection());
	}

	public void _testValueEvents(Button subject) throws Exception {
		HashMap clientEvent = new HashMap();
		clientEvent.put("name", "Click");
		clientEvent.put("sourceId", subject.getWidgetId());
		clientEvent.put("value", "1");
		fireEvent(clientEvent);
		assertEquals(true, subject.getSelection());

		clientEvent.put("value", "0");
		fireEvent(clientEvent);
		assertEquals(false, subject.getSelection());
	}

// template methods -----------------------------------------------------------

	protected Class getSubjectClass() { return Button.class; }

}
