package org.eclipse.swt.widgets.tests;

import il.co.cortext.rui.remote.IRuiConstants;
import java.util.HashMap;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Combo;

public class ComboTest extends ControlTestCase {

	private Combo subject;
	private SelectionEvent event;

	public ComboTest(String name) { super(name); }

	protected void setUp() throws Exception {
		super.setUp();
		subject = (Combo) getSubject();
	}

	public void testItems() {
		String[] items = {"foo", "bar", "baz"};
		subject.setItems(items);
		assertEquals(3, subject.getItemCount());
		assertEquals("foo", subject.getItem(0));
		assertEquals("bar", subject.getItem(1));
		assertEquals("baz", subject.getItem(2));

		String[] newItems = subject.getItems();
		assertEquals(items[0], newItems[0]);
		assertEquals(items[1], newItems[1]);
		assertEquals(items[2], newItems[2]);
		assertEquals(0, subject.indexOf("foo"));
		assertEquals(1, subject.indexOf("bar"));
		assertEquals(2, subject.indexOf("baz"));
		assertEquals(-1, subject.indexOf("not in list"));

		subject.setItem(2, "new");
		assertEquals(3, subject.getItemCount());
		assertEquals("new", subject.getItem(2));
		assertEquals(2, subject.indexOf("new"));
		assertEquals(-1, subject.indexOf("baz"));

		subject.add("added");
		assertEquals(4, subject.getItemCount());
		assertEquals("added", subject.getItem(3));
		assertEquals(3, subject.indexOf("added"));

		subject.add("added by index", 0);
		assertEquals(5, subject.getItemCount());
		assertEquals("added by index", subject.getItem(0));
		assertEquals(0, subject.indexOf("added by index"));
		assertEquals("added", subject.getItem(4));
		assertEquals(4, subject.indexOf("added"));

		subject.remove(0);
		assertEquals(4, subject.getItemCount());
		assertEquals("added", subject.getItem(3));

		subject.removeAll();
		assertEquals(0, subject.getItemCount());
	}

	public void testSelection() {
		String[] items = {"foo", "bar", "baz"};
		subject.setItems(items);
		assertEquals(-1, subject.getSelectionIndex());

		subject.select(1);
		assertEquals(1, subject.getSelectionIndex());
		assertEquals("bar", subject.getItem(subject.getSelectionIndex()));

		subject.deselect(2);
		assertEquals(1, subject.getSelectionIndex());

		subject.deselect(1);
		assertEquals(-1, subject.getSelectionIndex());

		subject.select(2);
		assertEquals(2, subject.getSelectionIndex());
		subject.deselectAll();
		assertEquals(-1, subject.getSelectionIndex());
	}

	public void testEvents() throws Exception {
		String[] items = {"foo", "bar", "baz"};
		subject.setItems(items);

		event = null;
		SelectionListener listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent inEvent)
				{ event = inEvent; }
		};
		HashMap clientEvent = new HashMap();
		clientEvent.put("name", "SelectionChanged");
		clientEvent.put("selectedIndex", "1");
		clientEvent.put("sourceId", subject.getWidgetId());

		subject.addSelectionListener(listener);
		fireEvent(clientEvent);
		assertNotNull(event);
		assertEquals(subject, event.getSource());
		assertEquals("1", "" + subject.getSelectionIndex());

		event = null;
		clientEvent.put("selectedIndex", "0");
		fireEvent(clientEvent);
		assertNotNull(event);
		assertEquals(subject, event.getSource());
		assertEquals("0", "" + subject.getSelectionIndex());
	}

	public void testInsert() {
		String[] items = {"foo", "bar", "baz"};
		subject.setItems(items);
		Object[][] results = {
			{subject, IRuiConstants.METHOD_CLEAR,  null},
			{subject, IRuiConstants.METHOD_INSERT, "0.foo"},
			{subject, IRuiConstants.METHOD_INSERT, "1.bar"},
			{subject, IRuiConstants.METHOD_INSERT, "2.baz"},
		};
		assertBufferMatches(results);

		subject.add("added");
		assertBuffer(subject, IRuiConstants.METHOD_INSERT, "3.added");

		subject.add("added by index", 0);
		assertBuffer(subject, IRuiConstants.METHOD_INSERT, "0.added by index");
	}

	public void testRemove() {
		String[] items = {"foo", "bar", "baz"};
		subject.setItems(items);
		flush();

		subject.remove(1);
		assertBuffer(subject, IRuiConstants.METHOD_REMOVEBYINDEX, "1");
		
		subject.removeAll();
		assertBuffer(subject, IRuiConstants.METHOD_CLEAR, null);
	}

	public void testClientSelection() {
		String[] items = {"foo", "bar", "baz"};
		subject.setItems(items);
		flush();

		subject.select(1);
		assertBuffer(subject, IRuiConstants.METHOD_SELECTEDINDEX, "1");
		
		subject.deselect(1);
		assertBuffer(subject, IRuiConstants.METHOD_SELECTEDINDEX, "-1");
	}

// template methods -----------------------------------------------------------

	protected Class getSubjectClass() { return Combo.class; }

}
