package org.eclipse.swt.widgets.tests;

import il.co.cortext.rui.remote.IRuiConstants;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Widget;

public class CompositeTest extends ControlTestCase {

	private Composite subject;

	public CompositeTest(String name) { super(name); }

	protected void setUp() throws Exception {
		super.setUp();
		subject = (Composite) getSubject();
	}

// composition tests ----------------------------------------------------------

	public void testGetChildren() throws Exception {
		Composite child1 = new Composite(subject, SWT.NONE);
		Composite child2 = new Composite(subject, SWT.NONE);
		Composite child3 = new Composite(subject, SWT.NONE);
		Widget[] children = subject.getChildren();
		assertEquals(3, children.length);
		assertEquals(child1, children[0]);
		assertEquals(child2, children[1]);
		assertEquals(child3, children[2]);
	}

	public void testRemoveChild() throws Exception {
		Composite child1 = new Composite(subject, SWT.NONE);
		Composite child2 = new Composite(subject, SWT.NONE);
		child1.dispose();
		Widget[] children = subject.getChildren();
		assertEquals(child2, children[0]);
		assertEquals(1, children.length);
		child2.dispose();
		assertEquals(0, subject.getChildren().length);
	}

	public void testInsertion() throws Exception {
		Composite child1 = new Composite(subject, SWT.NONE);
		Composite child2 = new Composite(subject, SWT.NONE);
		Composite child3 = new Composite(subject, SWT.NONE);
		Object[][] results = {
			{child1,   IRuiConstants.METHOD_CREATE, "E2"},
			{child2,   IRuiConstants.METHOD_CREATE, "E2"},
			{child3,   IRuiConstants.METHOD_CREATE, "E2"},
		};
		assertBufferMatches(results);
	}

// layout tests ---------------------------------------------------------------

	public void testSetLayoutManagerName() throws Exception {
		Object[][] layouts = {
			{"fill", new FillLayout()},
			{"grid", new GridLayout()},
			{"row",  new RowLayout()},
		};
		for (int i = 0; i < layouts.length; i++) {
			String name   = (String) layouts[i][0];
			Layout layout = (Layout) layouts[i][1];
			subject.setLayout(layout);
			subject.layout();
			assertBufferMatches(
				getDelegate(subject),
				IRuiConstants.METHOD_LAYOUTMANAGER,
				"name." + name
			);
		}
	}

	public void testSetLayoutManagerState() throws Exception {
		FillLayout layout = new FillLayout();
		subject.setLayout(layout);
		layout.type = SWT.VERTICAL;
		subject.layout();
		assertBufferMatches(
			getDelegate(subject),
			IRuiConstants.METHOD_LAYOUTMANAGER,
			"orient.vertical"
		);
	}

	public void testSetLayoutManagerRecursive() throws Exception {
		Composite child1 = new Composite(subject, SWT.NONE);
		Composite child2 = new Composite(subject, SWT.NONE);
		flush();
		subject.setLayout(new FillLayout());
		child1. setLayout(new GridLayout());
		child2. setLayout(new RowLayout());
		subject.layout();
		Control delegate = getDelegate(subject);
		Object[][] results = {
			{delegate, IRuiConstants.METHOD_LAYOUTMANAGER, "name.fill"},
			{child1,   IRuiConstants.METHOD_LAYOUTMANAGER, "name.grid"},
			{child2,   IRuiConstants.METHOD_LAYOUTMANAGER, "name.row"},
		};
		assertBufferMatches(results);
	}

	public void testRowLayoutData() throws Exception {
		Composite child1 = new Composite(subject, SWT.NONE);
		Composite child2 = new Composite(subject, SWT.NONE);
		Composite child3 = new Composite(subject, SWT.NONE);
		Composite child4 = new Composite(subject, SWT.NONE);
		Composite child5 = new Composite(child4, SWT.NONE);
		Composite child6 = new Composite(child5, SWT.NONE);
		subject.setLayout(new RowLayout());
		child4.setLayout(new RowLayout());
		flush();
		child1. setLayoutData(new RowData(3, 4));
		child2. setLayoutData(new RowData(5, 6));
		child4. setLayoutData(new RowData(7, 8));
		child5. setLayoutData(new RowData(9, 10));
		child6. setLayoutData(new RowData(SWT.DEFAULT, SWT.DEFAULT));
		subject.layout();
		Object[][] results = {
			{child1, IRuiConstants.METHOD_LAYOUTHINT, "width.3"},
			{child1, IRuiConstants.METHOD_LAYOUTHINT, "height.4"},
			{child2, IRuiConstants.METHOD_LAYOUTHINT, "width.5"},
			{child2, IRuiConstants.METHOD_LAYOUTHINT, "height.6"},
			{child4, IRuiConstants.METHOD_LAYOUTHINT, "width.7"},
			{child4, IRuiConstants.METHOD_LAYOUTHINT, "height.8"},
			{child5, IRuiConstants.METHOD_LAYOUTHINT, "width.9"},
			{child5, IRuiConstants.METHOD_LAYOUTHINT, "height.10"},
		};
		assertBufferMatches(results);
	}

	public void testGridLayoutData() throws Exception {
		Composite child1 = new Composite(subject, SWT.NONE);
		subject.setLayout(new GridLayout(3, false));
		flush();
		GridData gridData1 = new GridData(GridData.FILL_BOTH);
		child1.setLayoutData(gridData1);
		shell.pack();
		Object[][] results = {
			{child1, IRuiConstants.METHOD_LAYOUTHINT, "hFlex.1"},
			{child1, IRuiConstants.METHOD_LAYOUTHINT, "vFlex.1"},
			{child1, IRuiConstants.METHOD_LAYOUTHINT, "hAlign.fill"},
			{child1, IRuiConstants.METHOD_LAYOUTHINT, "vAlign.fill"},
		};
		assertBufferMatches(results);
	}

// template methods -----------------------------------------------------------

	protected Class getSubjectClass() { return Composite.class; }

}
