package org.eclipse.swt.widgets.tests;

import il.co.cortext.rui.remote.IRuiConstants;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Label;

public class LabelTest extends ControlTestCase {

	private Label subject;

	public LabelTest(String name) { super(name); }

	protected void setUp() throws Exception {
		super.setUp();
		subject = (Label) getSubject();
	}

	public void testCreate() throws Exception {
		super.testCreate();

		subject = (Label) makeSubject(shell, SWT.LEFT);
		assertBufferMatches(subject, IRuiConstants.METHOD_HALIGN, "left");
		assertEquals(SWT.LEFT, subject.getAlignment());

		subject = (Label) makeSubject(shell, SWT.CENTER);
		assertBufferMatches(subject, IRuiConstants.METHOD_HALIGN, "center");
		assertEquals(SWT.CENTER, subject.getAlignment());

		subject = (Label) makeSubject(shell, SWT.RIGHT);
		assertBufferMatches(subject, IRuiConstants.METHOD_HALIGN, "right");
		assertEquals(SWT.RIGHT, subject.getAlignment());
	}

	public void testSetText() throws Exception {
		subject.setText("Hello World");
		assertBuffer(subject, IRuiConstants.METHOD_SETTEXT, "Hello World");
		assertEquals("Hello World", subject.getText());
	}

	public void testSetAlignment() throws Exception {
		subject.setAlignment(SWT.LEFT);
		assertBuffer(subject, IRuiConstants.METHOD_HALIGN, "left");
		assertEquals(SWT.LEFT, subject.getAlignment());

		subject.setAlignment(SWT.CENTER);
		assertBuffer(subject, IRuiConstants.METHOD_HALIGN, "center");
		assertEquals(SWT.CENTER, subject.getAlignment());

		subject.setAlignment(SWT.RIGHT);
		assertBuffer(subject, IRuiConstants.METHOD_HALIGN, "right");
		assertEquals(SWT.RIGHT, subject.getAlignment());
	}

	public void testSetImage() throws Exception {
		subject.setImage(new Image(null, "icons/Box.gif"));
		Object[][] results = {
			{subject, IRuiConstants.METHOD_IMAGEURL,    "images/icons/Box.gif"},
			{subject, IRuiConstants.METHOD_IMAGEWIDTH,  "16"},
			{subject, IRuiConstants.METHOD_IMAGEHEIGHT, "16"},
		};
		assertBufferMatches(results);
	}

// template methods -----------------------------------------------------------

	protected Class getSubjectClass() { return Label.class; }

}
