package org.eclipse.swt.widgets.tests;

import il.co.cortext.rui.remote.IRuiConstants;
import java.util.HashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Text;

public class TextTest extends ControlTestCase {

	private Text subject;
	private ModifyEvent event;

	public TextTest(String name) { super(name); }

	protected void setUp() throws Exception {
		super.setUp();
		subject = (Text) getSubject();
	}

	public void testCreate() throws Exception {
		super.testCreate();
		subject = (Text) makeSubject(shell, SWT.SINGLE);
		assertBuffer(
			subject,
			IRuiConstants.CLASS_TEXTFIELD,
			IRuiConstants.METHOD_CREATE,
			"E1"
		);
		subject = (Text) makeSubject(shell, SWT.MULTI);
		assertBuffer(
			subject,
			IRuiConstants.CLASS_TEXTAREA,
			IRuiConstants.METHOD_CREATE,
			"E1"
		);
	}

	public void testSetText() throws Exception {
		subject.setText("Hello World");
		assertBuffer(subject, IRuiConstants.METHOD_SETTEXT, "Hello World");
		assertEquals("Hello World", subject.getText());
	}

	public void testEvents() throws Exception {
		event = null;
		ModifyListener listener = new ModifyListener() {
			public void modifyText(ModifyEvent inEvent)
				{ event = inEvent; }
		};
		HashMap clientEvent = new HashMap();
		clientEvent.put("name", "Change");
		clientEvent.put("value", "foo");
		clientEvent.put("sourceId", subject.getWidgetId());

		subject.addModifyListener(listener);
		fireEvent(clientEvent);
		assertNotNull(event);
		assertEquals(subject, event.getSource());
		assertEquals("foo", subject.getText());

		event = null;
		clientEvent.put("value", "bar");
		fireEvent(clientEvent);
		assertNotNull(event);
		assertEquals(subject, event.getSource());
		assertEquals("bar", subject.getText());
	}

// template methods -----------------------------------------------------------

	protected Class getSubjectClass() { return Text.class; }

}
