#!perl -w

# Usage: $0 newProjectName

use FindBin;
use lib "$FindBin::Bin/../../../base/perllib";

use Rui;
use strict;

my $NEW_PROJECT_NAME = $ARGV[0] || die qq{Usage: "$0 newProjectName"\n};

use File::Path;
use File::Copy;

$| = 1;

my $OLD_PROJECT_HOME = "$Rui::PROJECT_HOME";
my $NEW_PROJECT_HOME = "$Rui::PROJECTS_HOME/$NEW_PROJECT_NAME";

die "Project directory already exists at $NEW_PROJECT_HOME\n"
	if -d $NEW_PROJECT_HOME;

# create directories -------------------------------------------------------

print "Creating project directories  ... ";

foreach my $dir ("", getDirectories()) {
	my $destPath = "$NEW_PROJECT_HOME/$dir";
	$destPath =~ s|/$||;
	mkpath($destPath) or die "Cannot create subdir $destPath: $!\n";
}
print "done\n";

# copy files ---------------------------------------------------------------

print "Copying project files ... ";
foreach my $file (getFilesToCopy()) {
	my $src = "$OLD_PROJECT_HOME/$file";
	my $dst = "$NEW_PROJECT_HOME/$file";
	copy($src, $dst) or die "Cannot copy $src $dst: $!\n";
}
print "done\n";

# touch files --------------------------------------------------------------

print "Touching project files ... ";
foreach my $file (getFilesToTouch()) {
	open FILE, ">$NEW_PROJECT_HOME/$file"
		or die "Cannot open file $file for touching: $!\n";
	close FILE;
}
print "done\n";

print "Project $NEW_PROJECT_NAME has been created at $NEW_PROJECT_HOME\n";

# configuration ------------------------------------------------------------

sub getDirectories
{qw(

	bin/servers
	conf
	log
	perllib

)}

sub getFilesToCopy
{qw(

	bin/servers/server-http.pl
	log/.cvsignore
	perllib/Rui.pm

)}

sub getFilesToTouch
{qw(

	conf/applications.cfg

)}
