#!/usr/bin/perl -w

use strict;

use FindBin;
use lib "$FindBin::Bin/../../../base/perllib";

use Rui;
use Rui::Remote::SessionManager;
use Rui::Remote::ClientMessage;
use Rui::Application::Map;

chdir($Rui::PROJECT_HOME);
$| = 1;

my @appNames = Rui::Application::Map->new->getApplicationNames;
my $sm = Rui::Remote::SessionManager->instance;
my $runningApps = [];

sub readPromptAnswer ($) {
	my $prompt = shift || "Your choice";
	print "\n$prompt: ";
	my $answer = <>;
	chomp($answer);
	$answer =~ s/^\s+//;
	$answer =~ s/\s+$//;
	print "-" x 78, "\n";
	return $answer;
}

sub showApplicationChoices () {
	print "\nAll known applications:\n";
	my $count = 0;
	foreach ("[Cancel]", @appNames) {
		printf "\t%2s) %s\n", $count, $_;
		$count++;
	};

	my $answer = readPromptAnswer("Enter application number to run");
	return undef if $answer !~ /^\d+$/ || $answer < 1 || $answer > @appNames;
	return $appNames[$answer - 1];
}

sub startNewApp ($) {
	my $appName = shift;

	my $event = {
		requestType => "boot",
		applicationName => $appName,
	};

	my $session = $sm->createSession;
	push @$runningApps, [ $appName, $session ];
	return ($session->getId, $event);
}

sub processEvent (@) {
	my $sessionId = shift || die "No sessionId";
	my $event = shift || die "No event";

	my $session = $sm->getSessionById($sessionId);
	$sm->setActiveSession($session);
	$session->postClientEvent(%$event);
	my $content = $session->flushBuffer;
	$sm->setActiveSession(undef);
	$content =~ s/\01/ /g;
	$content =~ s/\02/\n/g;
	return $content;
}

sub showActionChoices () {
	print "\nYou can now:\n";
	my $count = 0;
	foreach ("[Quit]",
		"Show running applications",
		"Start new application",
		"Fire event in running application",
	) {
		printf "\t%2s) %s\n", $count, $_;
		$count++;
	}

	my $answer = readPromptAnswer("Enter your choice number");
	exit(0) if $answer !~ /^\d+$/ || $answer < 1 || $answer > 3;

	if ($answer == 1) {
		my $count = 0;
		foreach (@$runningApps) {
			$count++;
			my ($appName, $session) = @$_;
			print "[$count] $appName, sessionId = ", $session->getId, "\n";
		}
	}

	if ($answer == 2) {
		my $appName = showApplicationChoices();
		return unless defined $appName;

		my $commands = processEvent(startNewApp($appName));
		my $appClass = Rui::Application::Map->new->getApplicationClassName($appName);
		print "Started new $appName ($appClass)\n";
		print $commands;
	}

	if ($answer == 3) {
		print "Unimplemented yet\n";
	}
}

while (1) {
	showActionChoices();
}
