#!perl -w

use FindBin;
use lib "$FindBin::Bin/../../../base/perllib";

use Rui;
use strict;

$| = 1;

my $rootDir = "$Rui::PROJECTS_HOME/../web/jslib";
my $outFile = "$rootDir/../jslib.js";

my $wordBreak     = '+#-/%*,?:=<>!&|~ .;()[]';
my $fullWordBreak = "{}$wordBreak";
my @files         = getClassNames();
my $startTime     = time();
my $out           = "";

foreach my $file (@files) {
	(my $fileName = "$rootDir/$file.js") =~ s|_|/|g;
	open F, $fileName or die "Can't open $fileName: $!";
	$out .= "\n". join('', <F>). "\n";
	close F;
}

$out = filter($out);

open G, ">$outFile" or die "Can't open $outFile: $!";
print G $out;
close G;

print "Done: ". (time() - $startTime). " seconds";

sub filter {
	my $in = shift;

	my %strings     = ();
	my $stringCount = 0;

	while ($in =~ s/([^\\])"(.*?[^\\])"/$1$stringCount/o) {
		$strings{"$stringCount"} = $2;
		$stringCount++;
	}

	for ($in) {
		# remove one line comments
		s|//.*?\n||g;
		# remove multi-line comments
		s|/\*(.*\n)*?\*/||g;
		# remove more than one consecutive newline, space
		s/\n+/\n/g;
		s/ +/ /g;
		# remove tabs
		s/\t//g;
		# remove useless newlines
		s/\n*([\Q$wordBreak\E])\n*/$1/g;
		s|\}\n\}|}}|g;
		s|\{\n\{|{{|g;
		s|\n*\{\n*|{|g;
		s|\n*\}|}|g;
		# remove useless spaces
		s| ?([\Q$fullWordBreak\E]) ?|$1|g;
		# remove useless semicolons
		s|;\}|}|g;
	}

	$in =~ s/$_/"$strings{$_}"/ foreach (keys %strings);

	return $in;
}

sub getClassNames {qw(

	Util_BootStrap

	Client_ServerResponse
	Client_ServerProxy

	Widget_IE6_WindowManager
	Widget_IE6_LayoutBuffer
	Widget_IE6_Window
	Widget_IE6_URLWindow
	Widget_IE6_Base

	Widget_IE6_Label
	Widget_IE6_Button
	Widget_IE6_CheckBox
	Widget_IE6_RadioBox
	Widget_IE6_TextField
	Widget_IE6_TextArea
	Widget_IE6_HTMLArea
	Widget_IE6_PasswordField

	Widget_IE6_Layout_Null
	Widget_IE6_Layout_Fill
	Widget_IE6_Layout_Row
	Widget_IE6_Layout_Grid
	Widget_IE6_Layout_Stack
	Widget_IE6_Layout_Box

	Widget_IE6_Panel
	Widget_IE6_Group

	Widget_IE6_Splitter
	Widget_IE6_SplitPanel

	Widget_IE6_NoteBook_Tab
	Widget_IE6_NoteBook_Header
	Widget_IE6_NoteBook

	Widget_IE6_List_Controller_Single
	Widget_IE6_List_Item
	Widget_IE6_List

	Widget_IE6_SelectBox_Controller_Single
	Widget_IE6_SelectBox_Item
	Widget_IE6_SelectBox

	Widget_IE6_Tree_Controller_Single
	Widget_IE6_Tree_NodeBody
	Widget_IE6_Tree_NodeHead
	Widget_IE6_Tree_Node
	Widget_IE6_Tree

	Widget_IE6_HTMLEditor

	Client_Runner
	Client_Application

)}
