package Rui::Application::Map;

use Carp;
use Attribute::Protected;

sub new
{
	my $thing      = shift;
	my $self       = bless ({map => {}}, ref($thing) || $thing);
	my $fileCfg    = 'conf/applications.cfg';
	my $baseCfg    = "$Rui::PROJECTS_HOME/$Rui::PROJECT_BASE_NAME/$fileCfg";
	my $projectCfg = "$Rui::PROJECT_HOME/$fileCfg";
	$self->addMappingFile($baseCfg);
	$self->addMappingFile($projectCfg)
		if $Rui::PROJECT_NAME ne $Rui::PROJECT_BASE_NAME; 
	return $self;
}

sub addMappingFile : Private
{
	my ($self, $fileName) = @_;
	open FILE, $fileName or
		croak "cannot open base applications config file [$fileName]: $!";
	while (<FILE>) {
		chomp;
		next unless /\w/;
		s/^[\s\t]*//;
		s/[\s\t]*$//;
		my ($name, $class) = split /[\s\t]+/;
		croak "badly formed mapping at $fileName: [ $name => $class ]" 
			unless $name && $class;
		$self->{map}->{$name} = $class
	}
	close FILE;
}

sub getApplicationClassName { shift->{map}->{shift()} }
sub getApplicationNames     { keys %{shift->{map}} }

1;

=head1 NAME

Rui::Application::Map - map from application names to application classes

=head1 SYNOPSYS

  use Rui::Application::Map;
  $applicationMap = Rui::Application::Map->new;
  $className1 = $applicationMap->getApplicationClassName('DEFAULT');
  $className2 = $applicationMap->getApplicationClassName('WidgetExample');

=head1 DESCRIPTION

The actual mapping is configured from a file at
C<$Rui::PROJECT_HOME/conf/applications.cfg>. If the current project is
not the base project, then the base mapping is added to the current
project mapping.

Sessions use this class to find the application class of an application
name, using the method C<getApplicationClassName()>.

=head1 CONFIGURATION

The configuration file defines a mapping per line. Each mapping consists
of an application name (users boot applications by name) and a
corresponding application class. Any number of space characters seperate
the two. C<DEFAULT> is a special application name used when none is
given by the client. For example here is an C<applications.cfg> file
that defines 2 applications and a default application:

  DEFAULT  MyProject::Application::Foo
  Foo      MyProject::Application::Foo
  Bar      MyProject::Application::Bar

=cut
