package Rui::Application::eg::Widget;

use Carp;
use Attribute::Protected;

use base 'Rui::Application::Base';

my @Demos = qw(

	Label
	Button
	TextField
	HTMLEditor
	NoteBook
	List
	Tree
	DoubleList
	ComboBox
	SplitPanel
	Window

);

sub start : Protected
{
	my $self = shift;
	$self->
		layoutManager(name => 'grid')->
		title('Rui Widget Demo');

	$self->{demos} = [];
	$self->{selectedIndex} = 0;

	my $noteBook = $self->NoteBook->
		layoutHint(flex => 1, fill => 1)->
		addListener(Selection_Change => sub {
			$self->{selectedIndex} = shift->value
		});
	foreach my $demo (@Demos) {

		my $demoClass = "Rui::Application::eg::Widget::$demo";
		eval "use $demoClass";
		croak "error using demo class $demoClass:\n$@\n" if $@;
		$noteBook->Tab(value => $demo);

		push @{$self->{demos}}, $noteBook->Page->
			layoutManager(name => 'fill')->
			createWidget(type => $demoClass);

	}

	my $toolbar = $self->Panel->
		layoutHint(hFlex => 1, hAlign => 'fill')->
		layoutManager(spacing => 0)->
		style(
			borderTop  => '2px groove threedhighlight',
			paddingTop => '3px',
		);

	$toolbar->Button(value => 'Source',
		tooltip => 'Show server widget source',
		events  => {Click => sub {
			($self->{sourceWindow}, $self->{sourceArea}) =
				$self->createWindow(PerlSource => $self->{sourceWindow});
			$self->{sourceArea}->model->
				value(ref $self->{demos}->[$self->{selectedIndex}]);
		}},
	);

	$toolbar->Button(value => 'Markup',
		tooltip => 'Show client widget markup',
		events  => {Click => sub {
			($self->{markupWindow}, $self->{markupArea}) =
				$self->createWindow(TextArea => $self->{markupWindow});
			$self->{demos}->[$self->{selectedIndex}]->
				showMarkup($self->{markupArea});
		}},
	);

	$toolbar->Button(value => 'Monitor',
		tooltip => 'Show incoming messages from server',
		events  => {Click => sub {
			($self->{monitorWindow}, $self->{monitorArea}) =
				$self->createWindow(TextArea => $self->{monitorWindow});
			$self->{monitorArea}->monitorServerMessages;
		}},
	);

	$toolbar->Button(value => 'Exit',
		tooltip => 'Exit the demo',
		events  => {Click => sub { $self->destroy }},
		confirm => 'Exit the demo application?',
	);

}

sub createWindow : Private
{
	my ($self, $widgetType, $currentWindow) = @_;
	if ($currentWindow && $currentWindow->isAlive) {
		($window, $widget) = ($currentWindow, $currentWindow->firstChild);
	} else {
		$window = Rui::Window->new(
			width         => 690,
			height        => 580,
			layoutManager => {name => 'fill'},
		);
		$widget = $window->createWidget(type => $widgetType);
	}
	$window->focus;
	return ($window, $widget);
}

1;
