package Rui::Application::eg::Widget::Button;

use Carp;
use Attribute::Protected;

use base 'Rui::Application::eg::Widget::Base';

sub populate : Protected
{
	my $self = shift;
	$self->layoutManager(name => 'grid');
	my $panel = $self->Panel->
		layoutManager(name => 'grid', numCols => 3)->
		layoutHint(hAlign => 'center', flex => 1);

	my ($mainButton, @colorButtons); # we have 4 buttons
	
	# create 3 color buttons
	foreach my $i (1..3) {
		$colorButtons[$i-1] = $panel->Button(
			underlineHover => 1,
			value          => $self->getDemoColorNames->[$i-1],
			image          => $self->getDemoImages->{
				$self->getDemoAntiColorNames->[$i-1]
			},
			events         => {Click => sub { $mainButton->style(
				backgroundColor => $self->getDemoColors->[$i-1]
			) }},
			style          => {
				color           => $self->getDemoAntiColors->[$i-1],
				backgroundColor => $self->getDemoColors->[$i-1],
				fontWeight      => 'bold',
				fontSize        => '20px',
			},
		);
	}

	# create the button that disables the color buttons
	my $tooltip = 'click to disable the 3 colored buttons messing around '.
		'with my backgroundColor';
	$mainButton = $panel->Button(
		value      => 'disable',
		flat       => 1,
		tooltip    => $tooltip,
		layoutHint => {hSpan => 3, hAlign => 'fill'},
		style      => {fontWeight => 'bold', fontSize => '20px'},
		events     => {Click => sub {
			if ($mainButton->value eq 'disable') {
				$colorButtons[$_]->disabled(1) foreach (0..2);
				$mainButton->value('enable')->
					tooltip('click to enable them again');
			} else {
				$colorButtons[$_]->disabled(0) foreach (0..2);
				$mainButton->value('disable')->
					tooltip('click to disable them again');
			}
		}},
	);

}

1;
