package Rui::Application::eg::Widget::ComboBox;

use Carp;
use Attribute::Protected;

use Rui::Model::List;

use base 'Rui::Application::eg::Widget::Base';

sub populate : Protected
{
	my $self = shift;
	$self->layoutManager(name => 'grid', numCols => 2);

	my $categoryItems = [
		{
			value => 'shapes',
			list  => Rui::Model::List->new(data => [ qw(line dot circle) ]),
		},
		{
			value => 'fruit',
			list  => Rui::Model::List->new(data => [ qw(orange apple cherry) ]),
		},
		{
			value => 'animals',
			list  => Rui::Model::List->new(data => [ qw(koala fish mouse) ]),
		},
	];

	# instructions
	my $instructions = $self->Panel(
		cssClass      => 'InfoLabel',
		layoutManager => {name   => 'grid'},
		layoutHint    => {vAlign => 'top'},
	);
	$instructions->Label(value => $_) foreach split /\$/,
		'Instructions:$1) Select category$2) Select item';

	my $panel = $self->Panel->
		layoutHint(hFlex => 1, fill => 1)->
		layoutManager(name => 'grid', numCols => 2)->
		style(
			border  => '2px groove threedhighlight',
			padding => '5px 0px 5px 5px',
		);

	$panel->Label(value => 'select category')->
		layoutHint(hAlign => 'end');
	my $categoryModel = $panel->ComboBox(
		renderer   => sub { shift },
		layoutHint => {hFlex => 1, hAlign => 'fill'},
	)->model;

	$panel->Label(value => 'select item')->
		layoutHint(hAlign => 'end');
	my $itemModel = $panel->ComboBox(
		renderer   => sub { { value => shift } },
		layoutHint => {hAlign => 'fill'},
	)->model;

	$panel->Label(value => 'selected item')->
		layoutHint(hAlign => 'end');
	my $resultField = $panel->TextField->
		disabled(1)->
		layoutHint(hAlign => 'fill');

	$categoryModel->list->add($categoryItems);
	$categoryModel->selectFirst;
	$itemModel->listHolder(Rui::Model::Value->new(
		adapt           => ['value.{list}' => $categoryModel->selection],
		attachToAdaptee => 1,
	))->selectFirst;
	$resultField->model(Rui::Model::Value->new(
		adapt           => [value => $itemModel->selection],
		attachToAdaptee => 1,
	));
}

1;

