package Rui::Application::eg::Widget::Label;

use Carp;
use Attribute::Protected;

use base 'Rui::Application::eg::Widget::Base';

sub populate : Protected
{
	my $self = shift;
	$self->layoutManager(name => 'grid');
	my %marginAndPadding = (  
		padding => '3px 4px 3px 4px',
		margin  => '5px 4px 5px 4px',
	);

	my $panel = $self->Panel->
		layoutHint(hAlign  => 'fill')->
		layoutManager(name => 'grid', numCols => 5);
	my $label = $panel->Label(
		value => 'A Bordered Icon Label',
		image => $self->getDemoImages->{Folder},
		style => {
			%marginAndPadding,
			border => '2px groove threedhighlight',
		},
	);

	my @colorNames = @{$self->getDemoColorNames};
	my $buttons = {};
	foreach my $color (@colorNames) {

		$buttons->{$color} = $panel->Button(

			value   => "Set $color",
			image   => $self->getDemoImages->{$color},
			tooltip => 'click to change label to '. lc($color),
			events  => {Click => sub {
				$buttons->{$_}->disabled(0) foreach @colorNames;
				$buttons->{$color}->disabled(1);
				$label->style(borderColor => $color);
				$label->image($self->getDemoImages->{$color});
			}},
		);

	}

	$self->Label(
		value       => 'A Styled Label',
		layoutHint  => {hFlex => 1, hAlign => 'fill'},
		style       => {
			backgroundColor => '#00FF00',
			fontFamily      => 'serif',
			fontWeight      => 'bold',
			fontSize        => '25px',
			border          => '2px inset #00FF00',
			%marginAndPadding,
		},
	);

	$self->Label(
		value           => 'A Tooltip Label',
		layoutHint      => {flex => 1, fill => 1},
		backgroundImage => $self->getDemoImages->{Background},
		tooltip         => 'The Tooltip Of a Label With a Tooltip',
		style => {
			color           => '#FFFFFF',
			fontSize        => '25px',
			fontWeight      => 'bold',
			%marginAndPadding,
		},
	);
}

1;

