package Rui::Application::eg::Widget::List;

use Carp;
use Attribute::Protected;

use Rui::Model::SelectionInList;

use base 'Rui::Application::eg::Widget::Base';

sub populate : Protected
{
	my $self = shift;
	my ($counter, $deleteButton) = 1;
	$self->layoutManager(name => 'grid', numCols => 2);

	# a panel for all the buttons
	my $buttonPanel = $self->Panel->
		layoutManager(name => 'grid')->
		layoutHint(vAlign => 'top');

	# instructions
	my $instructions = $buttonPanel->Panel(
		cssClass      => 'InfoLabel',
		layoutManager => {name   => 'grid'},
		layoutHint    => {hAlign => 'fill'},
	);
	$instructions->Label(value => $_) foreach split /\$/,
		'Instructions:$1) Add shape$2) Select shape$3) Delete shape';

	# create selection in list
	my $model = Rui::Model::SelectionInList->new;

	# create 3 buttons for adding shapes
	foreach my $shape (@{$self->getDemoShapes}) {
		$buttonPanel->Button(
			layoutHint => {hFlex => 1, hAlign => 'fill'},
			hAlign     => 'left',
			image      => $self->getDemoImages->{$shape},
			events     => {Click => sub {
				# when clicked, add to the list a rendering of the shape
				$model->addAfterSelection({
					value => "$shape #". $counter++,
					image => $self->getDemoImages->{$shape},
				});
				# and enable the delete button
				$deleteButton->disabled(0);
			}},
		)->value("Add $shape");
	}

	# a button to delete shapes
	$deleteButton = $buttonPanel->Button(
		layoutHint => {hAlign => 'fill'},
		image      => $self->getDemoImages->{Delete},
		hAlign     => 'left',
		events     => {Click => sub {
			# if there is a selection, remove it
			$model->removeSelection
				if defined $model->selectedItem;
			# if this is last element, disable myself
			$deleteButton->disabled(1)
				unless $model->list->getSize;
		}},
	)->value("delete shape")->disabled(1);

	# text field for showing the selected item
	my $resultField = $buttonPanel->TextField(
		disabled   => 1,
		layoutHint => {hAlign => 'fill'},
	)->model(Rui::Model::Value->new(
		adapt           => ['value.{value}' => $model->selection],
		attachToAdaptee => 1,
	));

	# create the list
	$self->List(
		renderer   => sub { shift },
		layoutHint => {flex => 1, fill => 1},
		model      => $model,
	);;
}

1;

