package Rui::Application::eg::Widget::Window;

use Carp;
use Attribute::Protected;
use Rui::Window::URL;

use base 'Rui::Application::eg::Widget::Base';

sub populate : Protected
{
	my $self = shift;
	$self->layoutManager(name => 'grid', numCols => 3);

	my @windows = ();
	my $counterModel = Rui::Model::Value->new(value => 1);
	my $counterUp    = sub { $counterModel->value($counterModel->value+1) };
	my $counterDown  = sub { $counterModel->value($counterModel->value-1) };

	$self->Button(value => 'URLWindow')->addListener(Click => sub {
		Rui::Window::URL->new(url => 'index.html');
	});
	$self->Button(
		value   => 'Counter View',
		events  => {Click => sub {
			my $window = 
				Rui::Window->new(width => 100, height => 50, isResizable => 0)->
				layoutManager(name => 'grid')->title("Rui Demo: Counter View");
			push @windows, $window;
			$window->TextField(model => $counterModel, disabled => 1)->
				layoutHint(hAlign => 'fill', hFlex => 1);
		}},
	);
	$self->Button(
		value   => 'Counter Editor',
		events  => {Click => sub {
			my $window =
				Rui::Window->new(width => 200, height => 100)->
				layoutManager(name => 'grid', numCols => 3)->
				title("Rui Demo: Counter Editor");
			push @windows, $window;
			$window->TextField(model => $counterModel)->
				layoutHint(hAlign => 'fill', hFlex => 1);
			$window->Button(value => 'up',   events => {Click => $counterUp});
			$window->Button(value => 'down', events => {Click => $counterDown});
		}},
	);
	$self->Button(
		value   => 'Close All Counter Windows',
		events  => {Click => sub {
			foreach my $window (@windows)
				{ $window->destroy if $window->isAlive }
			@windows = ();
		}},
	)->layoutHint(hSpan => 3, hAlign => 'center');

	my $panel = $self->Panel->
		layoutManager(name => 'fill')->
		layoutHint(hSpan => 3, hAlign => 'fill')->
		style(
			border  => '2px inset threedhighlight',
			padding => '5px 3px 5px 2px',
		);
	$panel->TextField(model => $counterModel)->layoutHint(hAlign => 'fill');
	$panel->Button(value => 'up',   events => {Click => $counterUp});
	$panel->Button(value => 'down', events => {Click => $counterDown});
}

1;

