
package Rui::Event;

use Carp;

sub new
{
	my ($thing, %params) = @_;
	my $self = bless ({}, ref($thing) || $thing);
	$self->init(%params);
	return $self;
}

sub init
{
	my ($self, %eventParams) = @_;
	$self->{name}   = $eventParams{name};
	$self->{source} = $eventParams{source};
}

sub name
{ 
	my $self = shift;
	return $self->{name} unless @_;
	$self->{name} = shift;
	return $self;
}

sub source
{ 
	my $self = shift;
	return $self->{source} unless @_;
	$self->{source} = shift;
	return $self;
}

sub clone
{ 
	my $self = shift;
	my $clone = $self->new;
	while (my ($key, $value) = each %$self) {
		$clone->{$key} = eval { $value };
	}
	return $clone;
}

1;

=head1 NAME

Rui::Event - event base class

=head1 SYNOPSIS

  my $event = Rui::Event->new(
     name   => 'someEventName',
     source => $self,
  );
  my ($name, $source) = ($event->name, $event->source);

=head1 DESCRIPTION

A descriptor of some named event fired by some source. All events have a
name and a source object. The name of the event is used by listeners and
listenables.

Before you fire an event, clone it so that each listener gets its own
event, so it can do anything it wants with it without disturbing other
listeners.

=head1 METHODS

=head2 CONSTRUCTING

=head3 new

=over 4

=item parameters

Hash of named parameters. Keys used here:

name - Scalar. Event name.

source - Any. Object that fired the event.

=back

=head2 ACCESSING

=head3 name

=over 4

=item description

Get/set event name. Scalar.

=back

=head3 source

=over 4

=item description

Get/set source. Scalar.

=back

