
package Rui::Event::List;
use Attribute::Protected;

use base qw(Rui::Event);

sub init : Protected
{
	my ($self, %params) = @_;
	$self->SUPER::init(%params);
	$self->{index} = $params{index};
	$self->{size}  = $params{size};
	$self->{items} = $params{items};
}

sub index
{ 
	my $self = shift;
	return $self->{index} unless @_;
	$self->{index} = shift;
	return $self;
}

sub size
{ 
	my $self = shift;
	return $self->{size} unless @_;
	$self->{size} = shift;
	return $self;
}

sub items
{ 
	my $self = shift;
	return $self->{items} unless @_;
	$self->{items} = shift;
	return $self;
}

1;

=head1 NAME

Rui::Event::List - a list model change event

=head1 SYNOPSIS

  # 5 items added at index 3
  my $event = Rui::Event::List->new(
     name     => 'RangeAdded',
     source   => $self,
     index    => 3,
     size     => 5,
     items    => [@newItems],
  );
  print $event->index, $event->size; # 35
  $event->index(3);
  $event->size(1);

=head1 SUPERCLASS

L<Rui::Event>

=head1 DESCRIPTION

A change in a list. The event is described by an index in the list, and
the number of list elements involved, starting from that index. Event
name will be one of:

=over 4

=item *

C<RangeAdded>

=item *

C<RangeRemoved>

=item *

C<RangeChanged>

=back

=head1 METHODS

=head2 CONSTRUCTING

=head3 new

=over 4

=item parameters

Hash of named parameters. Keys used here:

index - Integer. Where did the change happen.

size - Integer>0. Number of elements involved.

items - Array ref of elements removed, added, or changed.

=back

=head2 ACCESSING

=head3 index

Get/set index. Integer.

=head3 size

Get/set size. Integer.

=head3 items

Get/set items. Array ref of elements removed , added, or changed.


