
package Rui::Event::Value;
use Attribute::Protected;
use base qw(Rui::Event);

sub init
{
	my ($self, %params) = @_;
	$self->SUPER::init(%params);
	$self->{value} = $params{value};
}

sub value
{ 
	my $self = shift;
	return $self->{value} unless @_;
	$self->{value} = shift;
	return $self;
}

1;

=head1 NAME

Rui::Event::Value - value changed event

=head1 SYNOPSIS

  my $event = Rui::Event::Value->new(
     name   => 'someEventName',
     source => $self,
     value  => 'someValue',
  );
  print $event->name. ':', $event->source. ':', $event->value;

=head1 SUPERCLASS

L<Rui::Event>

=head1 DESCRIPTION

Value change event which carries the new value.

=head1 METHODS

=head2 CONSTRUCTING

=head3 new

=over 4

=item parameters

Hash of named parameters. Keys used here:

value - Optional. Any. The value that has changed.

=back

=head2 ACCESSING

=head3 value

=over 4

=item description

Get/set value. Any.

=back

