package Rui::Event::tests::Listenable;

use Rui::Event;
use Rui::Event::Listenable;

use base qw(Test::Unit::TestCase);

sub test_new
{
	my $self = shift;
	my $cb_called = 0;
	my $cb = sub {$cb_called = shift->name};
	my $list = Rui::Event::Listenable->new(events => {DoorOpened => $cb});
	$list->fireEvent(Rui::Event->new(name => 'DoorOpened', source => $self));
	$self->assert_equals('DoorOpened', $cb_called);
}

sub test_removeListener
{
	my $self = shift;
	my $list = Rui::Event::Listenable->new;
	$list->addListener(DoorOpened => [someMethod => $self]);
	$list->removeListener(DoorOpened => $self);
	my $eventNames = join( '|', $list->getEventNamesOfListeners);
	$self->assert_equals('', $eventNames);
}

sub test_hasListenerForEvent
{
	my $self = shift;
	my $list = Rui::Event::Listenable->new;
	$list->addListener(DoorOpened => $self);
	$self->assert($list->hasListenerForEvent('DoorOpened'), "no listener");
	$list->removeListener(DoorOpened => $self);
	$self->assert(!$list->hasListenerForEvent('DoorOpened'), "extra listener");
}

sub test_getEventNamesOfListeners
{
	my $self = shift;
	my $cb = {};
	my $list = Rui::Event::Listenable->new(events => {DoorOpened => $cb});
	$list->addListener(DoorClosed => $cb);
	my $eventNames = join('|', $list->getEventNamesOfListeners);
	$self->assert_equals('DoorClosed|DoorOpened', $eventNames);
}

sub test_weakRefs1
{
	my $self = shift;
	my $list = Rui::Event::Listenable->new;
	my $listener1 = Rui::Event::Listenable->new;
	my $listener2 = Rui::Event::Listenable->new;
	$list->addListener(DoorOpened => $listener1);
	$list->addListener(DoorClosed => $listener2);
	$listener1 = undef;
	my $eventNames = join('|', $list->getEventNamesOfListeners);
	$self->assert_equals('DoorClosed', $eventNames);
}

sub test_weakRefs2
{
	my $self = shift;
	my $list = Rui::Event::Listenable->new;
	my $listener1 = Rui::Event::Listenable->new;
	my $listener2 = Rui::Event::Listenable->new;
	$list->addListener(DoorOpened => [someMethod => $listener1]);
	$list->addListener(DoorClosed => [someMethod => $listener2]);
	$listener1 = undef;
	my $eventNames = join('|', $list->getEventNamesOfListeners);
	$self->assert_equals('DoorClosed', $eventNames);
}

1;
