
package Rui::Image;

use Carp;
use Attribute::Protected;

sub new
{
	my ($thing, %params) = @_;
	my $self = bless ({}, ref($thing) || $thing);
	$self->init(%params);
	return $self;
}

sub init : Protected
{
	my ($self, %params) = @_;
	$self->{$_} = ($params{$_} || croak "image with no $_")
		foreach qw(file size);
}

sub file
{ 
	my $self = shift;
	return $self->{file} unless @_;
	$self->{file} = shift;
	return $self;
}

sub size
{ 
	my $self = shift;
	return $self->{size} unless @_;
	$self->{size} = shift;
	return $self;
}

sub setOn
{ 
	my $self   = shift;
	my $widget = shift;
	return $self->setOnLocator($widget, @_) if @_;
	$widget->sendToClient(image       => $self->getURL);
	$widget->sendToClient(imageWidth  => $self->size->[0]);
	$widget->sendToClient(imageHeight => $self->size->[1]);
	return $self;
}

sub setOnLocator : Private
{
	my ($self, $widget, $messageName, $locator) = @_;
	$widget->sendToClient($messageName => 
		$locator, $self->getURL, $self->size->[0], $self->size->[1]
	);
	return $self;
}

sub removeFrom
{ 
	my ($self, $widget) = @_;
	$widget->sendToClient(image => '');
	return $self;
}

sub addToBackground
{ 
	my ($self, $widget) = @_;
	$widget->style(backgroundImage => 'url('. $self->getURL. ')');
	return $self;
}

sub removeFromBackground
{ 
	my ($self, $widget) = @_;
	$widget->style(backgroundImage => 'url(none)');
	return $self;
}

sub getURL : Private
{ 
	my $self = shift;
	return "images/$self->{file}";
}

1;

=head1 NAME

Rui::Image - image object

=head1 SYNOPSIS

  use Rui::Image;

  $image = Rui::Image->new(
     file => "eg/Circle.gif",
     size => [16, 16],
  );
  
  $parent->Label(image => $image);           # a label with an image
  $parent->Button(image => $image);          # a button with an image
  $parent->Panel(backgroundImage => $image); # a panel with a background image

  $label->image(undef);                      # image will be removed
  $panel->backgroundImage(undef);            # background image will be removed

=head1 DESCRIPTION

Rui supports PNG, GIF, and JPEG images. You place the image file under
C<$Rui::PROJECT_HOME/web/images>. Different widgets feature different
ways to add images to them.

=cut
