package Rui::Model::DoubleList;

use Carp;

use Rui::Model::Value;
use Rui::Model::SelectionInList;

use base 'Rui::Model::Base';

# creating -----------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	$self->{leftHolder} = Rui::Model::Value->new
		(value  => $params{left} || Rui::Model::SelectionInList->new);
	$self->{rightHolder} = Rui::Model::Value->new
		(value  => $params{right} || Rui::Model::SelectionInList->new);
	$self->SUPER::init(%params);
}

sub destroy
{
	my $self = shift;
	$self->SUPER::destroy;
	$self->{rightHolder}->destroy;
	$self->{leftHolder}->destroy;
}

# accessing ----------------------------------------------------------------

sub left
{
	my ($self, $left) = @_;
	return $self->{leftHolder}->value if @_ == 1;
	$self->{leftHolder}->value($left);
	return $self;
}

sub right
{
	my ($self, $right) = @_;
	return $self->{rightHolder}->value if @_ == 1;
	$self->{rightHolder}->value($right);
	return $self;
}

sub leftHolder  { shift->{leftHolder}  }
sub rightHolder { shift->{rightHolder} }

sub getChildModel_left  { shift->left }
sub getChildModel_right { shift->right }

# list management ----------------------------------------------------------

sub add
{
	my $self  = shift;
	my $left  = $self->left;
	my $right = $self->right;
	my $item  = $left->selectedItem;
	return unless $item;
	return if $right->list->itemExists($item);
	$self->right->addNearSelection($item, 0);
	return $self;
}

sub remove
{
	my $self  = shift;
	my $model = $self->right;
	my $item  = $model->selectedItem;
	return unless $item;
	$self->right->list->removeItem($item);
	return $self;
}

sub removeAll
{
	my $self = shift;
	$self->right->list->clear;
}

sub addAll
{
	my $self  = shift;
	my $left  = $self->left->list;
	my $right = $self->right->list;
	$right->clear;
	$right->add([$left->getAll]);
}

sub moveDown { shift->right->moveDown }
sub moveUp   { shift->right->moveUp   }

# event handling -----------------------------------------------------------

sub fireChange : Private
{
	my $self = shift;
	$self->fireEvent(Rui::Event->new(name => 'Change', source => $self));
}

sub handleEvent_leftHolder_Change  { shift->fireChange }
sub handleEvent_rightHolder_Change { shift->fireChange }

# template methods ---------------------------------------------------------

sub getAttachments : Protected
{{
	leftHolder  => {Change => 'Change'},
	rightHolder => {Change => 'Change'},
}}

# --------------------------------------------------------------------------

1;
