package Rui::Model::tests::SelectionInList;

use Rui::Model::Value;
use Rui::Model::List;
use Rui::Model::SelectionInList;

use base qw(Rui::Model::tests::ModelTestCase);

sub getModelClass { 'Rui::Model::SelectionInList' }

sub test_new1
{
	my $self = shift;
	$self->makeModel(
		data          => [qw(a b c)],
		selectedIndex => 1,
	);
	$self->assert($self->{model}->list->get(0) eq 'a');
	$self->assert($self->{model}->list->get(1) eq 'b');
	$self->assert($self->{model}->list->get(2) eq 'c');
	$self->assert($self->{model}->list->getSize == 3);
	$self->assert($self->{model}->selectedIndex == 1);
	$self->assert($self->{model}->selectedItem eq 'b');
}

sub test_new2
{
	my $self = shift;
	$self->makeModel(
		list         => Rui::Model::List ->new( data => [qw(a b c)] ),
		selectedItem => 'c',
	);
	$self->assert($self->{model}->list->get(0) eq 'a');
	$self->assert($self->{model}->list->get(1) eq 'b');
	$self->assert($self->{model}->list->get(2) eq 'c');
	$self->assert($self->{model}->list->getSize == 3);
	$self->assert($self->{model}->selectedIndex == 2);
}

sub test_events_List_RangeAdded
{
	my $self = shift;
	$self->makeModel;
	my $event = 0;
	$self->{model}->addListener(List_RangeAdded   => sub { $event = shift });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = 0 });
	$self->{model}->addListener(List_RangeChanged => sub { $event = 0 });
	$self->{model}->addListener(List_Change       => sub { $event = 0 });
	$self->{model}->addListener(Selection_Change  => sub { $event = 0 });
	$self->{model}->list->add( 'foo');
	$self->assert($event->index == 0);
	$self->assert($event->size == 1);
}

sub test_events_List_RangeRemoved
{
	my $self = shift;
	$self->makeModel( data => [qw(a b c)]);
	my $event = 0;
	$self->{model}->addListener(List_RangeAdded   => sub { $event = 0 });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = shift });
	$self->{model}->addListener(List_RangeChanged => sub { $event = 0 });
	$self->{model}->addListener(List_Change       => sub { $event = 0 });
	$self->{model}->addListener(Selection_Change  => sub { $event = 0 });
	$self->{model}->list->remove( 0);
	$self->assert($event->index == 0);
	$self->assert($event->size == 1);
}

sub test_events_List_RangeChanged
{
	my $self = shift;
	my $a = Rui::Model::Value->new(value => 'a');
	my $b = Rui::Model::Value->new(value => 'b');
	my $c = Rui::Model::Value->new(value => 'c');
	$self->makeModel(data => [$a, $b, $c], attach => 1);
	my $event = 0;
	$self->{model}->addListener(List_RangeAdded   => sub { $event = 0 });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = 0 });
	$self->{model}->addListener(List_RangeChanged => sub { $event = shift });
	$self->{model}->addListener(List_Change       => sub { $event = 0 });
	$self->{model}->addListener(Selection_Change  => sub { $event = 0 });
	$b->value('foo');
	$self->assert($event->index == 1);
	$self->assert($event->size == 1);
}

sub test_events_List_Change
{
	my $self = shift;
	$self->makeModel;
	my $event = 0;
	$self->{model}->addListener(List_RangeAdded   => sub { $event = 0 });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = 0 });
	$self->{model}->addListener(List_RangeChanged => sub { $event = 0 });
	$self->{model}->addListener(List_Change       => sub { $event = shift });
	$self->{model}->addListener(Selection_Change  => sub { $event = 0 });
	$self->{model}->list( my $list = Rui::Model::List->new);
	$self->assert($event->value eq $list );
}

sub test_events_Selection_Change1
{
	my $self = shift;
	$self->makeModel( data => [qw(a b c)]);
	my $event = 0;
	$self->{model}->addListener(List_RangeAdded   => sub { $event = 0 });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = 0 });
	$self->{model}->addListener(List_RangeChanged => sub { $event = 0 });
	$self->{model}->addListener(List_Change       => sub { $event = 0 });
	$self->{model}->addListener(Selection_Change  => sub { $event = shift });
	$self->{model}->selectedIndex(1);
	$self->assert($event->value eq 'b');
}

sub test_events_Selection_Change2
{
	my $self = shift;
	$self->makeModel( data => [qw(a b c)]);
	my $event = 0;
	$self->{model}->addListener(List_RangeAdded   => sub { $event = 0 });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = 0 });
	$self->{model}->addListener(List_RangeChanged => sub { $event = 0 });
	$self->{model}->addListener(List_Change       => sub { $event = 0 });
	$self->{model}->addListener(Selection_Change  => sub { $event = shift });
	$self->{model}->selection( Rui::Model::Value->new( value => 'a' ));
	$self->assert($event->value eq 'a');
}

sub test_attach1
{
	my $self = shift;
	my $event = 0;
	$self->makeModel(
		data   => [
			Rui::Model::Value->new(value => 'a'),
			Rui::Model::Value->new(value => 'b'),
			Rui::Model::Value->new(value => 'c'),
		],
		attach => 0,
	);
	$self->{model}->addListener(List_RangeAdded   => sub { $event = shift });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = shift });
	$self->{model}->addListener(List_RangeChanged => sub { $event = shift });
	$self->{model}->addListener(List_Change       => sub { $event = shift });
	$self->{model}->addListener(Selection_Change  => sub { $event = shift });
	$self->{model}->list->get(1)->value( 'foo');
	$self->assert($event == 0);
}

sub test_attach2
{
	my $self = shift;
	my $event = 0;
	$self->makeModel(
		data   => [
			Rui::Model::Value->new(value => 'a'),
			Rui::Model::Value->new(value => 'b'),
			Rui::Model::Value->new(value => 'c'),
		],
		attach => 1,
	);
	$self->{model}->addListener(List_RangeAdded   => sub { $event = 0 });
	$self->{model}->addListener(List_RangeRemoved => sub { $event = 0 });
	$self->{model}->addListener(List_RangeChanged => sub { $event = shift });
	$self->{model}->addListener(List_Change       => sub { $event = 0 });
	$self->{model}->addListener(Selection_Change  => sub { $event = 0 });
	$self->{model}->list->get(1)->value( 'foo');
	$self->assert_equals(1, $event->index);
	$self->assert_equals(1, $event->size);
}

sub test_selectionManagement1
{
	my $self = shift;
	$self->makeModel;
	$self->{model}->list->add('a');
	$self->{model}->selectedIndex(0);
	$self->{model}->list->remove(0);
	$self->assert(!defined $self->{model}->selectedItem);
}

sub test_selectionManagement2
{
	my $self = shift;
	$self->makeModel;
	$self->{model}->list->add([qw(a b c)]);
	$self->{model}->selectedIndex(0);
	$self->{model}->list->remove(0);
	$self->assert_equals('b', $self->{model}->selectedItem);
	$self->{model}->list->remove(0);
	$self->assert_equals('c', $self->{model}->selectedItem);
	$self->{model}->list->remove(0);
	$self->assert(!defined $self->{model}->selectedItem);
}

sub test_selectionManagement3
{
	my $self = shift;
	$self->makeModel(
		attach        => 1,
      selectedIndex => 0,
		isNavigating  => 0,
		data          => [
			Rui::Model::Value->new(value => 'a'),
			Rui::Model::Value->new(value => 'b'),
			Rui::Model::Value->new(value => 'c'),
		],
	);
	my $valueModel = Rui::Model::Value->new(value => 'foo');
	$self->{model}->list->set($valueModel, 0);
	$self->assert_null($self->{model}->selectedItem, 'selectedItem rest');
	$self->{model}->selectedIndex(0);
	$valueModel->value( 'bar');
	$self->assert($self->{model}->selectedItem->value eq 'bar');
}

sub test_SharedList {
	my $self = shift;
	my $model1 = Rui::Model::SelectionInList->new(data => [qw(a b c)]);
	my $model2 = Rui::Model::SelectionInList->new(list => $model1->list);
	my ($event1, $event2) = (0, 0);
	$model1->addListener(List_RangeAdded => sub { $event1 = shift });
	$model2->addListener(List_RangeAdded => sub { $event2 = shift });
	$model1->list->add('foo');
	$self->assert_equals(3, $event1->index, 'event1 index');
	$self->assert_equals(3, $event2->index, 'event2 index');
}

1;
