package Rui::Remote::ClientMessage;

use Carp;
use Attribute::Protected;
use XML::Parser;

# creating -----------------------------------------------------------------

sub new
{
	my $thing = shift;
	my $self  = bless {}, ref($thing) || $thing;
	$self->init(@_);
	return $self;
}

sub init
{
	my ($self, $body) = @_;
	$body =~ s/\r\n/\n/g; # newlines could come in wrong
	my $event = {_xml => $body};
	$parser = new XML::Parser(Style => 'Tree');
	my @parsed = @{$parser->parse($body)->[1]};
	shift @parsed;
	my %parsed = @parsed;
	while (my ($key, $value) = each %parsed) {
		next if $key eq '0';
		$event->{$key} = $value->[2];
	}
	$self->{event} = $event;
}

# accessing ----------------------------------------------------------------

sub getEvent { %{shift->{event}} }

# --------------------------------------------------------------------------

=head1 NAME

Rui::Remote::ClientMessage - a message from the client to the server

=head1 DESCRIPTION

The message is received as XML. You get the event described in it as a
hash. A client that wants to inform the server of a Click event would
send this message:

  <rui>
     <sessionId>1234</sessionId>
     <name>Click</name>
     <sourceId>E3</sourceId>
  </rui>

The event you would get though C<getEvent()> is:

  (sessionId => 1234, name => 'Click', sourceId => 'E3')

=cut

1;
