package Rui::Remote::SessionManager;

use Carp;
use Attribute::Protected;
use Rui::Remote::Session;

my $Instance;

sub instance
{
	$Instance = Rui::Remote::SessionManager->new unless $Instance;
	return $Instance;
}

sub new
{
	my $thing = shift;
	my $self  = bless {}, ref($thing) || $thing;
	$self->init;
	return $self;
}

sub init
{
	my $self = shift;
	$self->{sessions}       = {};
	$self->{nextSessionId}  = 0;
	$self->{activeSession}  = undef;
}

sub createSession
{
	my $self = shift;
	my $sessionId = $self->makeSessionId;
	my $session   = Rui::Remote::Session->new($sessionId);
	$self->{sessions}->{$sessionId} = $session;
	return $session;

}

sub destroySession
{
	my ($self, $session) = @_;
	delete $self->{sessions}->{$session->getId};
	$session->destroy;
}

sub makeSessionId : Private
{
	$self->{nextSessionId}++;
	return "S$self->{nextSessionId}";
}

sub getSessionById    { shift->{sessions}->{shift()} }
sub setActiveSession  { pop->{activeSession} = pop }
sub getActiveSession  { shift->{activeSession} }

1;


