package Rui::Util;

use Carp;
use Syntax::Highlight::Perl;

# navigating object graphs -------------------------------------------------

sub jumpByRoute
{
	my ($source, $route) = @_;
	croak "no route for jumping" unless defined $route;
	croak "no source for jumping" unless $source;
	my $stage  = $source;
	foreach my $token (@$route) {
		return undef if !defined $stage;
		croak "stage is not ref for jumping [$token]" unless ref $stage;
		$stage = $token =~ s/(^\{)|(\}$)//g?
			$stage->{$token}: $stage->$token;
	}
	return $stage;
}

# syntax coloring Perl -----------------------------------------------------

my $Font    = '';
my $Color   = '';
my $BgColor = '';

sub syntaxColor
{
	my ($lines, $showCourtesy) = @_;
	my $formatter = createFormatter();
	my $out = 
		qq{<pre style="
			background-color: #000000;
			color:            #888888;
			font:             10pt Terminal, Fixedsys, monospace;
			padding:          5px 5px 5px 5px;
		">}.
		($showCourtesy?
			$formatter->format_string
				("# coloring courtesy of Syntax::Hightlight::Perl\n\n"):
			''
		);
	foreach my $line (split /\n/, $lines) {
		$line =~ s/\t/   /g;
		$line = $formatter->format_string($line);
		$out .= "$line\n";
	}
	$out .= '</pre>';
	return $out;
}

sub createFormatter
{
	my %style   = (
		'Variable_Scalar'   => '<font color="#44ff22">,</font>',
		'Variable_Array'    => '<font color="#eeaa00">,</font>',
		'Variable_Hash'     => '<font color="#bb88ff">,</font>',
		'Variable_Typeglob' => '<font color="#ff8888">,</font>',
		'Subroutine'        => '<font color="#ffff44">,</font>',
		'Quote'             => '<font color="#ffffff">,</font>',
		'String'            => '<font color="#00ffff">,</font>',
		'Comment_Normal'    => '<font color="#22AAff">,</font>',
		'Comment_POD'       => '<font color="#0000ff">,</font>',
		'Bareword'          => '<font color="#cccccc">,</font>',
		'Package'           => '<font color="#22cc99">,</font>',
		'Number'            => '<font color="#ff00ff">,</font>',
		'Operator'          => '<font color="#eecccc">,</font>',
		'Symbol'            => '<font color="#cccccc">,</font>',
		'Keyword'           => '<font color="#ffffff">,</font>',
		'Builtin_Operator'  => '<font color="#ffffff">,</font>',
		'Builtin_Function'  => '<font color="#ffffaa">,</font>',
		'Character'         => '<font color="#ff0000">,</font>',
		'Directive'         => '<font color="#990099">,</font>',
		'Label'             => '<font color="#990099">,</font>',
		'Line'              => '<font color="#ffff00">,</font>',
	);

	my $formatter = Syntax::Highlight::Perl->new;
	$formatter->set_format
		('Line' => ['<SPAN CLASS="Line">', '</SPAN>']);
	$formatter->define_substitution
		('<' => '&lt;', '>' => '&gt;', '&' => '&amp;');

	foreach my $key (keys %style) {
		my ($start, $end) = split /,/, $style{$key};
		$formatter->set_format($key, [$start, $end]);
	}
	return $formatter;
}

# --------------------------------------------------------------------------

1;
