package Rui::View::DoubleList;

use Attribute::Protected;
use Rui::Model::DoubleList;

use base 'Rui::View::Base';

# creating -----------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	$self->{renderer} = $params{renderer} || 0;
	$self->SUPER::init(%params);
}

sub populate : Protected
{
	my $self = shift;
	$self->layoutManager(name => 'grid', numCols => 3);

	$self->{left} = $self->List(
		holder       => 'left',
		layoutHint   => {flex => 1, fill => 1},
		renderer     => $self->{renderer},
	);

	my $toolbar = $self->Panel->
		layoutManager(name => 'grid')->
		layoutHint(vAlign => 'center', hAlign => 'fill');
	my @buttonSpecs = (
		[add          => [qw( >  add       )]],
		[remove       => [qw( <  remove    )]],
		[up           => [qw( ^  moveUp    )]],
		[down         => [qw( v  moveDown  )]],
		['add all'    => [qw( >> addAll    )]],
		['remove all' => [qw( << removeAll )]],
	);
	foreach my $buttonSpec (@buttonSpecs) {
		my $buttonTooltip = $buttonSpec->[0];
		my $buttonValue   = $buttonSpec->[1]->[0];
		my $buttonCommand = $buttonSpec->[1]->[1];
		$toolbar->Button(
			value      => $buttonValue,
			tooltip    => $buttonTooltip,
			layoutHint => {hAlign => 'fill'},
			events     => {Click => [$buttonCommand => $self]},
		);
	};

	$self->{right} = $self->List(
		holder       => 'right',
		layoutHint   => {flex => 1, fill => 1},
		renderer     => $self->{renderer},
	);
}

sub destroy
{
	my ($self, $dontSendToClient) = @_;
	$self->SUPER::destroy($dontSendToClient);
	delete($self->{$_}) foreach qw(left right);
}

# commands -----------------------------------------------------------------

sub add       { shift->model->add }
sub remove    { shift->model->remove }
sub moveUp    { shift->model->moveUp }
sub moveDown  { shift->model->moveDown }
sub addAll    { shift->model->addAll }
sub removeAll { shift->model->removeAll }

# template methods ---------------------------------------------------------

sub getDefaultModel    : Protected { Rui::Model::DoubleList->new }
sub getModelEventNames : Protected { qw(Change) }

# --------------------------------------------------------------------------

1;
