package Rui::View::Table;

use Carp;
use Attribute::Protected;

use base 'Rui::View::List';

# creating -----------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	foreach (qw(numCols colWidths))
		{ $self->{$_} = $params{$_} if exists $params{$_} }
	$self->SUPER::init(%params);
}

# client handling ----------------------------------------------------------

sub sendItemAddedToClient : Protected
{
	my ($self, $index) = @_;
	$self->sendToClient(insertRow => $index);
	my $row = $self->model->list->get($index);
	my $numCols = $self->{numCols} || 1;
	for (my $i = 0; $i < $numCols; $i++) {

		my $rendering = ref $self->{renderer} eq 'CODE'?
			&{$self->{renderer}}($row, $i):
			$self->{renderer}->render($row, $i);
		my $pos = "${index}_$i";

		$self->sendToClient(setCellValue => $pos, $rendering->{value});
		$rendering->{image}->setOn($self, setCellImage => $pos) if
			exists $rendering->{image};

	}
}

sub sendItemRemovedToClient : Protected
{
	my ($self, $index) = @_;
	$self->sendToClient(removeRow => $index);
}

sub clear
{
	my $self = shift;
	$self->SUPER::clear;
	$self->sendToClient(numCols => $self->{numCols})
		if exists $self->{numCols};
	$self->sendToClient(colWidths => join '_', @{$self->{colWidths}})
		if exists $self->{colWidths};
	return $self;
}

# rendering ----------------------------------------------------------------

sub render
{
	my ($self, $item, $col) = @_;
	croak "cannot do default rendering on non-ref item: [$item]"
		unless ref $item;
	my $row =
		UNIVERSAL::can($item, 'value')?               $item->value:
		ref $item eq 'HASH' && exists $item->{value}? $item->{value}:
		ref $item eq 'ARRAY'?                         $item:
		croak "cannot render strange item: [$item]";
	return {value => scalar $row->[$col]};
}

# template methods ---------------------------------------------------------

sub getDefaultWidgetClass : Protected { 'Table' }

# --------------------------------------------------------------------------

1;


