package Rui::View::TextField;

use Carp;
use Rui::Model::Value;
use Attribute::Protected;

use base 'Rui::View::Base';

# creating -----------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	$self->SUPER::init(%params);
	$self->model->value($params{value}) if exists $params{value};
	$self->readOnly($params{readOnly}) if exists $params{readOnly};
}

# accessing ----------------------------------------------------------------

sub readOnly { shift->attribute(readOnly => @_) }

# client handling ----------------------------------------------------------

sub sendModelToClient : Protected
{
	my $self = shift;
	my $value          = $self->model->value;
	my $lastValue      = $self->{lastValue};
	$lastValue         = defined $lastValue? $lastValue: '';
	$value             = defined $value? $value: '';
	$self->{lastValue} = $value;
	return if $value eq $lastValue;
	$self->attribute(value => $self->renderValue($value));
}

sub handleClientEvent_Change
{
	my ($self, %eventData) = @_;
	my $model = $self->model;
	my $value = $eventData{value};
	$value = '' unless defined $value;
	$self->{lastValue} = $value;
	$model->value($value);
}

# template methods ---------------------------------------------------------

sub getDefaultModel       : Protected { Rui::Model::Value->new }
sub getDefaultWidgetClass : Protected { 'TextField' }
sub getModelEventNames    : Protected { qw(Change) }
sub renderValue           : Protected { pop }
sub onUndefModel          : Protected
{
	my $self = shift;
	$self->{lastValue} = '';
	$self->clear;
}   

# --------------------------------------------------------------------------

1;

=head1 NAME

Rui::View::TextField - the base textfield view

=head1 SYNOPSIS

  $textField = $parent->TextField(
     value    => 'Hello World',
     readOnly => 1,
  );
  $textField->readOnly(0);

=head1 SUPERCLASS

L<Rui::View::Base>

=head1 MODEL

L<Rui::Model::Value>

=head1 DESCRIPTION

A textfield displays a L<value model|Rui::Model::Value>, and modifies it
when the user changes the text in the textfield. You can use the
C<value> key in the constructor hash to set a value on the model. You
can set the textfield as read only with the C<readOnly> feature .

=cut
