package Rui::View::Tree;

use Carp;
use Attribute::Protected;

use Rui::Model::Value;
use Rui::Model::SelectionInTree;

use base 'Rui::View::Base';

# creating -----------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	$self->{renderer}  = $params{renderer} || $self;
	$self->SUPER::init(%params);
}

sub destroy
{
	my ($self, $dontSendToClient) = @_;
	delete $self->{renderer};
	$self->SUPER::destroy($dontSendToClient);
}

# event handling -----------------------------------------------------------

sub handleEvent_modelHolder_Selection_Change { shift->sendSelectionToClient }
sub handleEvent_modelHolder_Tree_Change      { shift->sync }

sub handleEvent_modelHolder_Tree_RangeAdded
{
	my ($self, $event) = @_;
	my @path  = eval { @{$event->path} };
	my $index = pop @path;
	for (my $i = $index; $i < $index + $event->size; $i++)
		{ $self->sendNodeAddedToClientRecursive(@path, $i) }
}

sub handleEvent_modelHolder_Tree_RangeRemoved
{
	my ($self, $event) = @_;
	my @path  = eval { @{$event->path} };
	my $index = pop @path;
	for (my $i = $index; $i < $index + $event->size; $i++)
		{ $self->sendNodeRemovedToClient(@path, $i) }
}

sub handleEvent_modelHolder_Tree_RangeChanged
{
	my ($self, $event) = @_;
	my @path           = eval { @{$event->path} };
	my $index          = pop @path;
#	for (my $i = $index; $i < $index + $event->size; $i++)
#		{ $self->sendNodeChangedToClient(@path, $i) }
}

# client handling ----------------------------------------------------------

sub renderPathFromServer
{
	my ($self, @path) = @_;
	unshift @path, 0;
	my $pathString = join '_', @path;
	return $pathString;

}

sub renderPathFromClient
{
	my ($self, $pathString) = @_;
	my @path  = split '_', $pathString;
	shift @path;
	return @path;
}

sub handleClientEvent_SelectionChanged
{
	my ($self, %eventData) = @_;
	my $selectedPathString = $eventData{selectedPath};
	$selectedPathString = "" unless defined $selectedPathString;
 	$self->model->selectedNode(
		$self->model->tree->nodeAt(
			$self->renderPathFromClient($selectedPathString)
		)
	);
}

sub sendModelToClient : Protected
{
	my $self = shift;
	$self->clear;
	my $tree = $self->model->tree;
	return unless $tree;
	$self->sendNodeAddedToClientRecursive;
	$self->sendSelectionToClient if defined $self->model->selectedNode;
}

sub sendSelectionToClient : Private
{
	my $self = shift;
 	$self->attribute(selectedPath => defined $self->model->selectedNode?
		$self->renderPathFromServer($self->model->selectedPath):-1
 	);
}

sub sendNodeAddedToClientRecursive : Private
{
	my ($self, @path) = @_;

	$self->sendNodeAddedToClient(@path);
	my $list = $self->model->tree->listAt(@path);
	return unless $list->getSize;

	for (my $i=0; $i < $list->getSize; $i++)
		{ $self->sendNodeAddedToClientRecursive(@path, $i) }
}

sub sendNodeAddedToClient : Private
{
	my ($self, @path) = @_;
	my $node = $self->model->tree->nodeAt(@path);

	my $rendering = ref $self->{renderer} eq 'CODE'?
		&{$self->{renderer}}($node):
		$self->{renderer}->render($node);
	my $pathString = $self->renderPathFromServer(@path);
	$self->sendToClient(insert => $pathString, $rendering->{value});
	$rendering->{image}->setOn($self, setNodeImage => $pathString) if
		exists $rendering->{image};
}

sub sendNodeRemovedToClient : Private
{
	my ($self, @path) = @_;
	$self->attribute(removeByPath => $self->renderPathFromServer(@path));
}

# rendering ----------------------------------------------------------------

sub render
{
	my ($self, $node) = @_;
	return { value =>
		UNIVERSAL::can($node, 'value')?
			$node->value:
		ref $node eq 'HASH' && exists $node->{value}?
			$node->{value}:
			scalar $node
	};
}

# template methods ---------------------------------------------------------

sub getDefaultWidgetClass : Protected { 'Tree' }
sub getDefaultModel       : Protected { Rui::Model::SelectionInTree->new }
sub onUndefModel          : Protected { shift->clear }   
sub getModelEventNames    : Protected
{qw(
	Tree_Change
	Tree_RangeAdded
	Tree_RangeRemoved
	Tree_RangeChanged
	Tree_RangeValueChanged
	Selection_Change
)}

# --------------------------------------------------------------------------

1;
