package Rui::View::tests::Table;

use Rui::Model::List;

use base qw(Rui::Widget::tests::TestCase);

sub getSubjectClass { 'Table' }

sub set_up
{
	my $self = shift;
	$self->SUPER::set_up;
	$self->{subject} = $self->createWidget;
	$self->flushBuffer;
}

sub prepareSubject
{
	
	$self->{subject}->model->list(Rui::Model::List->new(data => [
		[qw(a b c)],
		[qw(d e f)],
		[qw(g h i)],
		[qw(j k l)],
	]);
}

sub test_insertCol
{
	my $self = shift;
	
	$self->assert_serverBuffer(
		'W1', 'E1', (
			[qw(clear               )],
			[qw(insertRow      0    )],
			[qw(setCellValue   0_0 a)],
			[qw(insertRow      1    )],
			[qw(setCellValue   1_0 d)],
			[qw(insertRow      2    )],
			[qw(setCellValue   2_0 g)],
			[qw(insertRow      3    )],
			[qw(setCellValue   3_0 j)],
			[qw(selectedIndex  0    )],
		)
	);
}

sub test_3cols
{
	my $self = shift;
	my $subject = $self->createWidget(numCols => 3);
	$self->flushBuffer;
	$subject->model->list($self->getTableModel);
	$self->assert_serverBuffer(
		'W1', 'E1', (
			[qw(clear               )],
			[qw(numCols        3    )],
			[qw(insertRow      0    )],
			[qw(setCellValue   0_0 a)],
			[qw(setCellValue   0_1 b)],
			[qw(setCellValue   0_2 c)],
			[qw(insertRow      1    )],
			[qw(setCellValue   1_0 d)],
			[qw(setCellValue   1_1 e)],
			[qw(setCellValue   1_2 f)],
			[qw(insertRow      2    )],
			[qw(setCellValue   2_0 g)],
			[qw(setCellValue   2_1 h)],
			[qw(setCellValue   2_2 i)],
			[qw(insertRow      3    )],
			[qw(setCellValue   3_0 j)],
			[qw(setCellValue   3_1 k)],
			[qw(setCellValue   3_2 l)],
			[qw(selectedIndex  0    )],
		)
	);
}

sub test_rangeAdded
{
	my $self = shift;
	my $subject = $self->createWidget(numCols => 3);
	$subject->model->list($self->getTableModel);
	$self->flushBuffer;
	$subject->model->list->add([[qw(x y z)]], 1);
	$self->assert_serverBuffer(
		'W1', 'E1', (
			[qw(insertRow    1    )],
			[qw(setCellValue 1_0 x)],
			[qw(setCellValue 1_1 y)],
			[qw(setCellValue 1_2 z)],
		)
	);
}

sub test_rangeRemoved
{
	my $self = shift;
	my $subject = $self->createWidget(numCols => 3);
	$subject->model->list($self->getTableModel);
	$self->flushBuffer;
	$subject->model->list->remove(1);
	$self->assert_serverBuffer('W1', 'E1', ([qw(removeRow 1)]));
}

sub test_rangeChanged
{
	my $self = shift;
	my $subject = $self->createWidget(numCols => 3);
	$subject->model->list($self->getTableModel);
	$self->flushBuffer;
	$subject->model->list->set([[qw(x y z)]], 1);
	$self->assert_serverBuffer(
		'W1', 'E1', (
			[qw(removeRow    1    )],
			[qw(insertRow    1    )],
			[qw(setCellValue 1_0 x)],
			[qw(setCellValue 1_1 y)],
			[qw(setCellValue 1_2 z)],
		)
	);
}

sub test_colWidths
{
	my $self = shift;
	my $subject = $self->createWidget(
		numCols   => 3,
		colWidths => [qw(100 150 200)],
	);
	$self->assert_serverBuffer(
		'W1', 'E1', (
			[qw(create    E1         )],
			[qw(clear                )],
			[qw(numCols   3          )],
			[qw(colWidths 100_150_200)],
		)
	);
}

1;


