package Rui::Widget::Button;

use Attribute::Protected;
use Rui::Event;

use base 'Rui::Widget::Label';

# creating------------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	$self->SUPER::init(%params);
	foreach (qw(flat underlineHover confirm))
		{ $self->$_($params{$_}) if exists $params{$_} }
}

# accessing ----------------------------------------------------------------

sub flat           { shift->attribute(flat => @_) }
sub underlineHover { shift->attribute(underlineHover => @_) }
sub confirm        { shift->attribute(confirm => @_) }

# template methods ---------------------------------------------------------

sub getDefaultWidgetClass : Protected { 'Button' }

# event handling -----------------------------------------------------------

sub handleClientEvent_Click {
	my ($self, %params) = @_;
	$self->fireEvent(Rui::Event->new(
	   name   => 'Click',
	   source => $self,
	));
}

# --------------------------------------------------------------------------

1;

=head1 NAME

Rui::Widget::Button - the button widget

=head1 SYNOPSIS

  $button = $parent->Button(
     value          => 'click me',
     image          => Rui::Image->new(file => "pic.gif", size => [16, 16]),
     hAlign         => 'center', 
     flat           => 1, 
     underlineHover => 1, 
     confirm        => 'Are you sure?',
     events         => {
        Click => sub { shift->source->value('clicked!') },
     },
  );
  $button->flat(0);            # default
  $button->underlineHover(0);  # default

=head1 SUPERCLASS

L<Rui::Widget::Label>

=head1 EVENTS

=over 4

=item Click

L<Rui::Event>. The user has clicked the button.

=back

=head1 DESCRIPTION

A Button is a label that can be pressed. You can use the C<flat> and
C<underlineHover> features to change the button appearance. Use
C<confirm> to show the user an OK/Cancel dialog. The event will be fired
only if the user chooses OK.

=head1 METHODS

=head3 flat

=over4

=item parameters

Setting - Boolean. True if button is to be shown flat

=item description

Get/set button flatness.

=back

=head3 underlineHover

=over4

=item parameters

Setting - Boolean. True if button is to be underlined when user passes
over it with mouse

=item description

Get/set underline hover feature.

=back

=head3 confirm

=over4

=item parameters

Setting - String. The text of the question displayed when the
confirmation dialog pops up

=item description

Get/set confirmation message

=back

=cut