package Rui::Widget::Label;

use Attribute::Protected;

use base 'Rui::Widget::Base';

# creating------------------------------------------------------------------

sub init : Protected
{
	my ($self, %params) = @_;
	$self->SUPER::init(%params);
	foreach (qw(value image hAlign))
		{ $self->$_($params{$_}) if exists $params{$_} }
}

# accessing ----------------------------------------------------------------

sub value { shift->attribute(value => @_) }

sub image { 
	my ($self, $image) = @_;
	return $self->{image} if @_ == 1;
	$self->{image}->removeFrom($self)
		if defined $self->{image} && !defined $image;
	$self->{image} = $image;
	$image->setOn($self) if defined $image;
}

sub hAlign { shift->attribute(hAlign => @_) }

# template methods ---------------------------------------------------------

sub getDefaultWidgetClass : Protected { 'Label' }

# --------------------------------------------------------------------------

1;

=head1 NAME

Rui::Widget::Label - the label widget

=head1 SYNOPSIS

  $label = $parent->Label(
     value  => 'Hello World',
     image  => Rui::Image->new(file => "pic.gif", size => [16, 16]),
     hAlign => 'right', 
  );

=head1 SUPERCLASS

L<Rui::Widget::Base>

=head1 DESCRIPTION

A label shows some text. You can add an image and set horizontal text
alignment.

=head1 METHODS

=head3 value

=over4

=item parameters

Setting - string to set as text of label

=item description

Get/set label value.

=back

=head3 image

=over4

=item parameters

Setting - L<Rui::Image>

=item description

Get/set label image. Set image as undefined to remove.

=back

=head3 hAlign

=over4

=item parameters

Setting - one of 3 values: C<left>, C<center>, C<right>

=item description

Get/set the horizontal text alignment for the label text.

=back

=cut
