package Rui::Widget::NoteBook;

use Carp;
use Attribute::Protected;
use Rui::Event::Value;

use base 'Rui::Widget::Panel';

# creating -----------------------------------------------------------------

sub init
{
	my $self = shift;
	$self->SUPER::init(@_);
	$self->{header} = $self->SUPER::createWidget(
		type        => 'Panel',
		widgetClass => 'NoteBookHeader',
	);
	$self->{stack}  = $self->SUPER::createWidget(type => 'Panel');
}

# compositing --------------------------------------------------------------

sub getHeader { shift->{header} }
sub getStack  { shift->{stack} }
sub Page      { shift->{stack} ->createWidget(type => 'Panel', @_) }
sub Tab       { shift->{header}->createWidget(type => 'Tab', @_) }

# event handling -----------------------------------------------------------

sub handleClientEvent_SelectionChanged {
	my ($self, %params) = @_;
	$self->fireEvent(Rui::Event::Value->new(
	   name   => 'Selection_Change',
	   source => $self,
		value  => $params{selectedIndex},
	));
}

# template methods --------------------------------------------------------------

sub getDefaultWidgetClass : Protected { 'NoteBook' }

# --------------------------------------------------------------------------

1;

=head1 NAME

Rui::Widget::NoteBook - the notebook widget

=head1 SYNOPSIS

  $noteBook = $parent->NoteBook;
  $tab  = $noteBook->Tab(value => "a tab");
  $page = $noteBook->Page;
  $page->Label(value => foo');    # page is a panel
  $header = $noteBook->getHeader; # for styling
  $stack  = $noteBook->getStack;  # for styling
  $noteBook->addListener(Selection_Change => sub {
     $label->value(shift->value)
  });

=head1 SUPERCLASS

L<Rui::Widget::Panel>

=head1 SEE ALSO

Rui::Widget::Tab

=head1 EVENTS

=over 4

=item Selection_Change

L<Rui::Event::Value>. The user has switched pages. The value is the
selected index.

=back

=head1 DESCRIPTION

A notebook is a container that shows only one I<page> at a time.  Users
switch between pages using the I<tabs>. You can only create pages or
tabs in a notebook. The Nth tab will be associated with the Nth page,
where N is the order of creation. A C<Tab> is just a label, so you can
set styles and images on it.

You can get the header and the stack of the notebook, if you need to
give them specific styles. They are both objects of the class
L<Rui::Widget::Panel>.

=cut
