package Rui::Widget::tests::TestCase;

use Carp;
use Rui::Mock::Session;
use Rui::Mock::Application;

use base qw(Test::Unit::TestCase);

sub set_up
{
	my $self = shift;
	$self->{session}     = Rui::Mock::Session->new;
	$self->{application} = Rui::Mock::Application->new(session => $self->{session});
	$self->flushBuffer;
	$self->{session}->resetIds;
}

sub tear_down
{
	my $self = shift;
	$self->{application}->destroy;
	delete $self->{$_} foreach qw(session application);
}

sub assert_serverBuffer
{
	my ($self, $windowId, $widgetId, @lines) = @_;
	my $subjectClass = $self->getSubjectClass;
	my $oracle = join ("\n", map {
		my @line = @$_;
		my $methodName = shift @line;
		my $line = 
			"$subjectClass.$methodName.$windowId.$widgetId".
			(@line? (".". join('.', @line)): "");
		$line;
	} @lines). "\n";
	my $buffer = $self->flushBuffer;
	$self->assert_equals($oracle, $buffer);
}

sub createWidget
{
	my ($self, %params) = @_;
	return $self->{application}->createWidget(
		type => $self->getSubjectClass, %params
	);
}

sub session         { shift->{session} }
sub application     { shift->{application} }
sub flushBuffer     { shift->session->flushBuffer }
sub postClientEvent { shift->session->postClientEvent(@_) }
sub getSubjectClass { croak "abstract" }

1;

__END__
