
package PhoneBook::View::PhoneBook;

use Carp;
use Attribute::Protected;
use PhoneBook::Model::Contact;

use base 'Rui::View::Base';

sub populate : Protected
{
	my $self = shift;
	$self->layoutManager(name => 'grid');

	my $toolbar = $self->Panel;
	$toolbar->Button(
		value   => 'New',
		events  => {Click => [command_New => $self]},
	);
	$toolbar->Button(
		value   => 'Delete',
		events  => {Click => [command_Delete => $self]},
	);
	$toolbar->Button(
		value   => 'Exit',
		events  => {Click => sub { print $self->getWindow->destroy }},
		confirm => 'Are you sure you want to exit?',
	);

	my $mainPanel = $self->SplitPanel(layoutHint => {flex => 1, fill => 1});

	$self->{navigationModel} = $mainPanel->List(
		renderer   => $self,
		style      => {margin => '0px 0px 0px 0px'},
		layoutHint => {width => 200},
	)->model;
	$self->{navigationModel}->listHolder(
		$self->makeChildModelAdapter('contacts')
	);
	$self->{navigationModel}->selectFirst;

	$mainPanel->PhoneBook__View__Contact(
		adapt => [value => $self->{navigationModel}->selection],
	)->style(
		border   => '2px inset threedhighlight',
		padding  => '4px',
		overflow => 'auto',
	);
}

sub command_New : Private
{
	my $self = shift;
	$self->{navigationModel}->addBeforeSelection(
		PhoneBook::Model::Contact->create
	);
}

sub command_Delete : Private
{
	my $self = shift;
	my $selectedItem = $self->{navigationModel}->selectedItem;
	return unless defined $selectedItem;
	$self->{navigationModel}->removeSelection;
	$selectedItem->erase;
}

sub render : Private
{
	my ($self, $item) = @_;
	my $name = $item->name;
	my $hasName = defined $name && $name ne '';
	return {
		value => $hasName? $name: '*undefined*',
	};
}

1;

__END__



