
#use 5.006_1;

package Rui;

use Carp;
use FindBin;

use vars qw($PROJECTS_HOME $PROJECT_HOME $PROJECT_NAME $PROJECT_BASE_NAME);

my $defaultProjectsHome =
	$^O eq 'MSWin32'? "C:/Rui/Perl": "/home/eilara/Rui/Perl";

sub import
{
	$PROJECT_BASE_NAME = 'base';
	$PROJECTS_HOME = findProjectsHome() || $defaultProjectsHome;
	croak "Cannot find $PROJECTS_HOME [$PROJECTS_HOME]"
		unless -d $PROJECTS_HOME;
	croak "Cannot find $PROJECT_BASE_NAME [$PROJECTS_HOME/$PROJECT_BASE_NAME]"
		unless -d "$PROJECTS_HOME/$PROJECT_BASE_NAME";
	my $dir = $FindBin::Bin;
	croak
		"Cannot find \$PROJECT_HOME:\n   running script in [$dir]\n".
		"   not under projects home [$PROJECTS_HOME/\$PROJECT_NAME]\n"
			unless ($dir =~ s|^$PROJECTS_HOME/||);
	$PROJECT_NAME = (split /\//, $dir, 2)[0];
	$PROJECT_HOME = "$PROJECTS_HOME/$PROJECT_NAME";

	unshift @INC, "$PROJECTS_HOME/$PROJECT_BASE_NAME/perllib"
		if $PROJECT_NAME ne $PROJECT_BASE_NAME;
	unshift @INC, "$PROJECT_HOME/perllib";
}

sub findProjectsHome
{
	my $dir = $FindBin::Bin;
	while ($dir =~ s|/[^/]+$||) {
		return $dir if
			-f "$dir/.perlrui_projects_home" || 
			-f "$dir/perlrui_projects_home";
	}
	return undef;
}

# --------------------------------------------------------------------------

1;

=head1 NAME

Rui - Perl Remote User Interface

=head1 SYNOPSIS

  use Rui; # this script uses the Rui framework

=head1 DESCRIPTION

Perl Rui is a Perl UI framework for remote user interfaces. Rui
distributes the UI at the widget level. If you were sniffing the line
you might hear this boot message:

  Window.create(W1);
  Label.create(W1,E1,-1);
  Label.value(W1,E1,Hello World);

Perl Rui is part of a set of tools for presentation clients and servers:

  Server             Network      Client
  ------             -------      ------
  * Perl Rui     ]\             /[ * RuiIE6
  * SWTRui      | \___________/ | * RuiNS6
  * ...         | /    Rui    \ | * ...
                ]/   protocol  \[

This module must be used in all Perl Rui scripts. Scripts must be run
from somewhere within the directory of the Perl Rui project you are
using. This project directory must be under the Perl Rui projects home
directory. E.g. assuming we are now using the C<myproject> project:

  Rui/              Rui home
     Perl/          $Rui::PROJECTS_HOME Perl Rui projects home, set on install
        base/       Perl Rui base project
        myproject/  $Rui::PROJECT_HOME current project

This module defines C<$Rui::$PROJECT_BASE_NAME>, C<$Rui::PROJECT_NAME>,
C<$Rui::PROJECTS_HOME>, and C<$Rui::PROJECT_HOME>. It also adds the
C<perllib/> under the Perl Rui base project to C<@INC>. If the current
project is not C<base>, then the C<perllib/> under the current project
is added to C<@INC> as well.

C<$Rui::PROJECTS_HOME> is found by climbing up from the working
directory of the script, looking for a file called
C<.Perl Rui_projects_home> or C<Perl Rui_projects_home> (easier to create
on MSWin32). The directory of the file is set as the Perl Rui projects
home.

After using this module you can use any Perl Rui class and classes from
the current project.

=head1 REQUIRES

=over 4

=item *

Perl 5.6.1

=item *

HTTP::Daemon

=item *

XML::Parser

=item *

WeakRef

=item *

Syntax::Highlight::Perl

=item *

Attribute::Abstract

=item *

Attribute::Protected

=item *

Test::Unit

=back

=head1 INSTALLATION

To install Perl Rui follow these steps:

=over 4

=item *

Install Perl 5.6.1 and relevant L<modules|/REQUIRES>.

=item *


Put this file (C<Rui.pm>) under C<@INC>

=item *

Set C<$Rui::PROJECTS_HOME> by placing a file called
C<.Perl Rui_projects_home> or C<Perl Rui_projects_home>, in your Perl Rui
projects home directory

=back

=cut
