
use("Widget_WindowManager");
use("Widget_LayoutBuffer");

use("Widget_Window");
use("Widget_Label");
use("Widget_Panel");
use("Widget_SplitPanel");
use("Widget_Button");
use("Widget_HTMLArea");
use("Widget_CheckBox");
use("Widget_RadioBox");
use("Widget_TextField");
use("Widget_TextArea");
use("Widget_NoteBook");
use("Widget_List");
use("Widget_SelectBox");
use("Widget_Group");
use("Widget_PasswordField");
use("Widget_URLWindow");
use("Widget_HTMLEditor");
use("Widget_Tree");

Class("Client_Runner");
Class_Singleton();

_.init =  function () {
	this.layoutBuffer  = Widget_LayoutBuffer.get();
	this.monitorArea   = null;
	this.winManager    = Widget_WindowManager.get();
	this.wordSeperator = String.fromCharCode(1);
	this.lineSeperator = String.fromCharCode(2);
 	this.classes       = {
		"Window"        : Widget_Window,
		"URLWindow"     : Widget_URLWindow,
		"Label"         : Widget_Label,
		"Panel"         : Widget_Panel,
		"SplitPanel"    : Widget_SplitPanel,
		"Button"        : Widget_Button,
		"CheckBox"      : Widget_CheckBox,
		"RadioBox"      : Widget_RadioBox,
		"HTMLArea"      : Widget_HTMLArea,
		"TextArea"      : Widget_TextArea ,
		"TextField"     : Widget_TextField,
		"HTMLEditor"    : Widget_HTMLEditor,
		"PasswordField" : Widget_PasswordField,
		"NoteBook"      : Widget_NoteBook,
		"NoteBookHeader": Widget_NoteBook_Header,
		"Tab"           : Widget_NoteBook_Tab,
		"List"          : Widget_List,
		"SelectBox"     : Widget_SelectBox,
		"Tree"          : Widget_Tree,
		"Group"         : Widget_Group
	};
	this.isWindow = {
		"Window"          : true,
		"URLWindow": true
	}
}

_.run = function (serverMessage) {

dumpln("<textarea rows=25 cols=120>\n"+this.getNiceServerMessage(serverMessage)+"\n</textarea>");

	if (this.monitorArea) this.addMessageToMonitorArea(serverMessage);

	var
		line, widgetClassName, widgetClass, methodName, windowId, widgetId,
		params, widget;

	var lines = serverMessage.split(this.lineSeperator);

var totalLineCount = lines.length;
var isBigMessage = totalLineCount > 30? true: false;
var lineCount = 0;

	for (line in lines) { try {

if (isBigMessage) {
	lineCount++;
	window.status = "Running UI: " + Math.round((lineCount/totalLineCount)*100) + "%";
}
		line = lines[line];
		if (!line.match(/\w/)) continue;
		params          = line.split(this.wordSeperator);
		widgetClassName = params[0];
		methodName      = params[1];
		windowId        = params[2];
		if (!widgetClassName || !methodName || !windowId)
			Throw("Cannot run message: " + serverMessage + "\n" + line);
		params = params.slice(3);
		var isWindow = this.isWindow[widgetClassName];
		if (isWindow) {

			if (methodName == "create") {

				widgetClass = this.classes[widgetClassName];
				if (!widgetClass)
					Throw("No widget class: [" + widgetClassName +"]");
				new widgetClass(windowId,
					params[0], params[1], params[2], params[3], params[4], params[5],
					params[6], params[7], params[8], params[9], params[10], params[11]
				);

			} else {

				widget = this.winManager.getWindow(windowId);
				this.checkMethod(widget, methodName);
				widget[methodName].call(widget, 
					params[0], params[1], params[2], params[3], params[4], params[5],
					params[6], params[7], params[8], params[9], params[10], params[11]
				);

			}

		} else {

			widgetId = params[0];
			params   = params.slice(1);
			if (methodName == "create") {
			
				widgetClass = this.classes[widgetClassName];
				if (!widgetClass)
					Throw("No widget class: [" + widgetClassName +"]");
				var parent = this.winManager.getWidget(windowId, params[0]);
				params     = params.slice(1);
				new widgetClass(widgetId, parent,
					params[0], params[1], params[2], params[3], params[4], params[5],
					params[6], params[7], params[8], params[9], params[10], params[11]
				);

			} else {

				widget = this.winManager.getWidget(windowId, widgetId);
				this.checkMethod(widget, methodName);
				widget[methodName].call(widget, 
					params[0], params[1], params[2], params[3], params[4], params[5],
					params[6], params[7], params[8], params[9], params[10], params[11]
				);

			}
		}		

	} catch (e) {
		var niceLine = line.replace(/\x01/g, ".");
		Throw(e, "while running [" + niceLine + "]");
	}}

if (isBigMessage) window.status = "Laying out UI...";

	this.layoutBuffer.doLayout();

window.status = "Ready";

window.setTimeout("try { dumpln('<textarea rows=25 cols=120>'+document.body.lastChild.outerHTML+'</textarea>') } catch (e) {}",2000);

}

_.runNiceMessage = function (serverMessage)
	{ this.run(this.getRawServerMessage(serverMessage)) }

_.setMonitorArea = function (monitorArea) { this.monitorArea = monitorArea }

_.addMessageToMonitorArea = function (serverMessage) {
	if (!serverMessage) return;
	var monitorArea = this.monitorArea;
	if (!monitorArea.element) {
		this.monitorArea = null;
		return;
	}
	this.monitorArea.insert
		(this.getNiceServerMessage(serverMessage) + "\n");
}

_.getNiceServerMessage = function (serverMessage) {
	var out  = serverMessage;
	var date = new Date;
	if (!out) return "";
	out = out.replace(/\x01/g, ".").replace(/\x02/g, "\n");
	var dateString = "[" +
		date.getHours()+":"+date.getMinutes()+":"+
		date.getSeconds()+":"+date.getMilliseconds()+
	"] ";
	out =
		dateString + "Server message received from "+
		Util_Conduit.get().serverURL+
		"\n-------------------------------------------------------------------\n"+
		out;
	return out;
}

_.getRawServerMessage = function (serverMessage) {
	var out  = serverMessage;
	if (!out) return "";
	out = out.
		replace(/\./g, String.fromCharCode(1)).
		replace(/\n/g, String.fromCharCode(2));
	return out;
}

_.checkMethod = function (widget, methodName) {
	var method = widget[methodName];
	if (!method) Throw
		("No method: " + methodName + " on class: " + widget.$className);
	if (!(typeof method != "function"))
		("Not a method: " + methodName + " on class: " + widget.$className);
}