
/*

Essential stuff for IE6 client:

	* browser type sniffing
	* error handling
	* global events
	* global utility functions
	* javascript OO support
	* debug support
	* client server communications

*/

// sniffing -------------------------------------------------------------------

var VERSION = {
	"NS6up": (navigator.userAgent.indexOf("Gecko")  != -1)? true: false,
	"IE6up": (navigator.userAgent.indexOf("MSIE 6") != -1)? true: false
};

if (!VERSION.IE6up) {
	alert("Cannot start Rui client: this version will only work on Internet Explorer 6");
	throw new Error();
}

// global events --------------------------------------------------------------

window.onerror = function (message, url, line) {
	alert("Error: " + Class_LAST_ERROR.description);
	return true;
};

window.onbeforeunload = function () { Util_Debug.get().closeWindow() }

// error handling -------------------------------------------------------------

function Throw(a, b) {
	var message           = b? a.description + "\n" + b: a;
	var exception         = b? a: new Error;
	exception.description = message;
	Class_LAST_ERROR      = exception;
	throw exception;
}

// global utilities -----------------------------------------------------------

function isDefined (value) { return value != undefined }

function argsAsArray(args) {
	var results = [];
	for (var i in args) results.push(args[i]);
	return results;
}

function dumpln (message) {
	if (!Util_Debug.get().getEnabled()) return;
	var date    = new Date();
	var theTime = date.getTime().toString().substr(8);
	var out;
	out =
		"<span style='background-color:pink;'>[" + theTime +
		"]</span>" + message + "<br>";
	Util_Debug.get().dumpBare(out);
}

function use (className) { Class_Use(className) }

// oo support -----------------------------------------------------------------

$ForPrototype = false;

Class_Classes    = {};

function Class (className, parentClassName) {
	Class_CheckClassName(className);
	if (parentClassName) {
		Class_CheckClassName(parentClassName);
		use(parentClassName);
	}
	var classObj = eval(
		className + " = function () {" +
			"if (!this.init) Throw('Class not used: " + className + "');" +
			"if ($ForPrototype) return;" +
			"this.init.apply(this, arguments);" +
		"}"
	);
	classObj.className       = className;
	Class_Classes[className] = classObj;
	if (parentClassName) classObj.parentClassName = parentClassName;
	_ = classObj.prototype;
	_.$classObj = classObj;
}

function Class_Singleton () {
	var classObj = _.$classObj;
	var singleton;
	classObj.get = function () {
		if (singleton) return singleton;
		singleton = new classObj;
		return singleton;
	};
}

function Class_Use (className) { try {
	var classObj = Class_Classes[className];
	if (!classObj) Util_Conduit.get().use(className);
	classObj = Class_Classes[className];
	if (!classObj) Throw("Class not declared before use: " + className);
	if (classObj.isUsed) return;

	var proto = classObj.prototype;
	var methodNames = Class_getMethodNames(proto);
	var i; for (i in methodNames) {
		var methodName      = methodNames[i];
		var method          = proto[methodName];
		var wrapped         = Class_wrapMethod(className, methodName, method);
		wrapped.$methodName = methodName;
		proto[methodName]   = wrapped;
	}

	var parentClassName = classObj.parentClassName;
	if (parentClassName) {
		var parentClassObj  = Class_Classes[parentClassName];
		if (!parentClassObj) Throw(
			"Base class not declared before inheritance: "
			+ className + ":" + parentClassName
		);
		if (!parentClassObj.isUsed) Throw(
			"Base class not used before inheritance: "
			+ className + ":" + parentClassName
		);
		$ForPrototype = true;
		var parentInstance = new parentClassObj;
		$ForPrototype = false;
		var i; for (i in methodNames) {
			var methodName   = methodNames[i];
			var parentMethod = parentInstance[methodName];
			var childMethod  = proto[methodName];
			if (parentMethod) {
				parentInstance[parentClassName + "_" + methodName] =
					parentInstance[methodName];
			}
			parentInstance[methodName] = childMethod;
		}
		classObj.prototype = parentInstance;
	}
	proto = classObj.prototype;
	if (!proto.init) proto.init = function () {};
	proto.$className = className;
	proto.$classObj  = classObj;
	classObj.isUsed  = true;
} catch (e) { Throw(e, "Using: " + className) }}

function Class_wrapMethod (className, methodName, method) {
	return !Util_Debug.get().getEnabled()?
		method:	
		function () { try {

			Class_PreHandler(this, className, methodName, arguments);
			var returnValue = method.apply(this, arguments);
			Class_PostHandler(this, className, methodName, returnValue);
			return returnValue;

		} catch (e) { Throw(e,
			"* " + className + ":" + methodName + "(" + this.$className + ")"
		)}}
}

function Class_getMethodNames (o) {
	var methodNames = [];
	for (methodName in o)
		if (typeof o[methodName]  == "function" && !methodName.match(/^\$/))
			methodNames.push(methodName);
	return methodNames;
}

function Class_asString (o, verbose) {
	var result = "";
	var p; for (p in o) result += p + (verbose? ":" + o[p]: "");
	return result;
}

function Class_CheckClassName (className) {
	if (className.match(/\s/))
		Throw("Not a legal class name: [" + className + "]");
}

function Class_PreHandler (object, className, methodName, arguments) {
	return;
	if (methodName.match(/Min/))
		dumpln(
			"<font color=blue>pre: " + className + ":" + methodName +
			"(" + object.$className + ") : " +
			argsAsArray(arguments).join(",") + "</font>"
		);
}

function Class_PostHandler (object, className, methodName, returnValue) {
	return;
	if (methodName.match(/Min/))
		dumpln(
			"<font color=purple>post: " + className + ":" + methodName +
			"(" + object.$className + ") : " +
			returnValue + "</font>"
		);
}

// debug support --------------------------------------------------------------

Class("Util_Debug");
Class_Singleton();

_.init = function () {
	this.messageBuffer = "";
	this.window        = undefined;
	this.setEnabled(false);
}

_.getEnabled = function ()      { return this.isEnabled }
_.setEnabled = function (value) { this.isEnabled = value }

_.dumpBare = function (message) {
	var debugWindow;
	try { debugWindow = this.getWindow() }
		catch (e) { Throw(e, "Getting debug window") }
	var debugBody; try { debugBody = debugWindow.document.body } catch (e) {}
	if (debugBody) {
		debugBody.insertAdjacentHTML("afterBegin", message + this.messageBuffer);
		this.messageBuffer = "";
	} else {
		this.messageBuffer = message + this.messageBuffer;
		var self = this;
		window.setTimeout(function () {
			var buffer = self.messageBuffer;
			self.messageBuffer = "";
			self.dumpBare(buffer);
		}, 1);
	}
}

_.getWindow = function () {
	if (!this.getEnabled()) return false;
	if (!this.window) this.window = window.open(
		"blank.html",
		"",
			"height=400,width=1024,top=350,left=10px,resizable=1,toolbar=0," +
			"menubar=0,status=0,scrollbars=1"
	);
	if (!this.window) Throw("Cannot open debug window");
	return this.window;
}

_.closeWindow = function () {
	if (!this.window) return; 
	try { this.getWindow().close() } catch (e) {}
}

// client server communications -----------------------------------------------

Class("Util_Conduit");
Class_Singleton();

_.init = function () {
	var hash = location.hash;
	Util_Debug.get().setEnabled(
		hash.substr(1) == 1? true:
		hash.substr(1) == 0? false:
		Util_Debug.get().getEnabled()
	);
	var pathname = location.pathname.replace(/\/[^\/]+$/, ""); 
	this.serverURL =
		"http:/" + "/" + location.hostname + ":" +
		location.port + pathname + "/";
	this.xmlHTTP = new ActiveXObject("Microsoft.XMLHTTP");
}

_.request = function (request, payloadXML) { try {
	var xmlHTTP = this.xmlHTTP;
	xmlHTTP.open("GET", this.serverURL + request, false);

	try { xmlHTTP.send(payloadXML || null) }
		catch (e) { Throw(e, "No response from server") }

	var response = new String(xmlHTTP.responseText);
	if (xmlHTTP.status != "200") Throw(
		"Cannot send request: " + xmlHTTP.statusText + "\n" +
		xmlHTTP.responseText
	);
	return response;
} catch (e) { Throw(e,
	"While requesting url: " + this.serverURL + ", request: " + request
)}}

_.use = function (className) { try {
	window.status = "Loading " + className + "...";
	var fileName = className.replace(
		/^Widget_/,
		"Widget_" + (VERSION.NS6up? "NS6": "IE6") + "_"
	);
	var libDir = "jslib/";
	var classText;
	try { classText = this.request(
		libDir + fileName.replace(/_/g, "/") + ".js"
	) } catch (e) { Throw(e, "Requesting class from server") }
	try { eval(classText.toString()) }
		catch(e) { Throw(e, "Evaling class text") }
	window.status = "Loaded " + className;
} catch (e) { Throw(e, "Importing: " + className) }}
