
use("Widget_Label");

Class("Widget_Button", "Widget_Label");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.Widget_Label_populate();
	this.isFlat              = false;
	this.element.tabIndex    = 1;
	this.element.onmouseover = this.onMouseOver;
	this.element.onmouseout  = this.onMouseOut;
	this.element.onmousedown = this.onMouseDown;
	this.element.onmouseup   = this.onMouseUp;
	this.element.onkeyup     = this.onKeyUp;
	this.element.onkeydown   = this.onKeyDown;
	this.element.ondblclick  = this.onDblClick;
	this.element.onfocus     = this.onFocus;
}

_.destroy = function (dontPaint) {
	this.element.onmouseover = null;
	this.element.onmouseout  = null;
	this.element.onmousedown = null;
	this.element.onmouseup   = null;
	this.element.onkeyup     = null;
	this.element.onkeydown   = null;
	this.element.ondblclick  = null;
	this.element.onfocus     = null;
	this.Widget_Label_destroy(dontPaint);
}

// accessing ------------------------------------------------------------------

_.flat = function (value) {
	var cssClass = this.element.className;
	value = (value && value != "0")? true: false;
	cssClass = value?
		cssClass.replace(/\bButton\b/, "FlatButton"):
		cssClass.replace(/\bFlatButton\b/, "Button");
	this.element.tabIndex = value? -1: 1;
	this.element.className = cssClass;
	this.isFlat = value;
}

_.confirm = function (value)  { this.confirmValue = value };

_.underlineHover = function (value)
	{ this.underline = (value && value != "0")? true: false }

// events ---------------------------------------------------------------------

_.fireClickEvent = function () {
	if (this.confirmValue && !this.window.confirm(this.confirmValue))
		return;
	this.fireEvent({
		"name"    : "Click",
		"sourceId": this.id
	});
	return true;
}

_.onMouseOver = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	me.stateHover();
}

_.onMouseOut = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	me.stateNormal();
}

_.onMouseDown = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	me.statePressed();
	if (!(me.isFlat)) me.element.focus();
}

_.onMouseUp = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	me.fireClickEvent();
	// widget could have been removed
	try { me.stateHover() } catch (e) {}
}

_.onKeyDown = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	var keyCode = me.window.element.event.keyCode;
	if (keyCode != 32 && keyCode != 13) return;
	me.statePressed();
}

_.onKeyUp = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	var keyCode = me.window.element.event.keyCode;
	if (keyCode != 32 && keyCode != 13) return;
	me.fireClickEvent();
	// widget could have been removed
	try { me.stateHover() } catch (e) {}
	try { me.stateHover() } catch (e) {}
}

_.onDblClick = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	me.statePressed();
	me.window.element.setTimeout(function () {
		me.stateHover();
	}, 30);
}

_.onFocus = function () {
	var me = this.me;
	if (me.isFlat) {
		me.element.blur();
		return false;
	}
	return true;
}

// styling -----------------------------------------------------------------

_.stateNormal = function () {
	var cssClass = this.element.className;
	this.element.className = cssClass.
		replace(/\b(Flat)?Button(_Pressed|_Hover)?\b/, "$1Button");
	if (this.underline) this.element.runtimeStyle.textDecoration = "";
}

_.stateHover = function () {
	var cssClass = this.element.className;
	this.element.className = cssClass.
		replace(/\b(Flat)?Button(_Pressed)?\b/, "$1Button_Hover");
	if (this.underline) this.element.runtimeStyle.textDecoration = "underline";
}

_.statePressed = function () {
	var cssClass = this.element.className;
	this.element.className = cssClass.
		replace(/\b(Flat)?Button(_Hover)?\b/, "$1Button_Pressed");
	if (this.underline) this.element.runtimeStyle.textDecoration = "underline";
}

// template methods --------------------------------------------------------

_.getTag         = function () { return "div" }
_.getWidgetClass = function () { return "Button" }

