
use("Widget_Base");

Class("Widget_HTMLArea", "Widget_Base");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.element.onblur = this.onChange;
	this.lastValue      = "";
	this.refreshEditable();
}

_.destroy = function (dontPaint) {
	this.element.onblur = null;
	this.Widget_Base_destroy(dontPaint);
}

_.clear = function () {
	this.lastValue         = "";
	this.element.innerHTML = "";
	this.refreshEditable();
}

// accessing ------------------------------------------------------------------

_.value = function (value) {
	this.element.innerHTML = value;
	this.lastValue         = value;
}

_.disabled = function (value) {
	this.Widget_Base_disabled(value);
	this.isDisabled = value && value != "0";
	this.refreshEditable();
}

_.readOnly = function (value) {
	this.readOnlyValue = value;
	this.isDisabled = value && value != "0";
	this.refreshEditable();
}

_.refreshEditable = function () {
	this.element.contentEditable = !this.getIsDisabled() && !this.readOnlyValue
}

// layout ---------------------------------------------------------------------

_.computeMinWidth  = function () { return 80 }
_.computeMinHeight = function () { return 100 }

// events ---------------------------------------------------------------------


_.onChange = function () {
	me = this.me;
	if (me.getIsDisabled()) return;
	var value = me.element.innerHTML;
	if (me.lastValue == value) return;
	me.lastValue = value;
	me.fireEvent({
		"name"    : "Change",
		"sourceId": me.id,
		"value"   : value
	});
	return true;
}

// testing --------------------------------------------------------------------

_.getValue = function () { return this.element.innerHTML }

// template methods -----------------------------------------------------------

_.getTag         = function () { return "div" }
_.getWidgetClass = function () { return "HTMLArea" }
