
use("Widget_Label");
use("Widget_Button");
use("Widget_HTMLArea");

Class("Widget_HTMLEditor", "Widget_Panel");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.Widget_Panel_populate();
	var buttonNames = [
		"Bold", "Italic", "Underline", "|",
		"JustifyLeft", "JustifyCenter", "JustifyRight", "|",
		"InsertUnorderedList", "InsertOrderedList"
	];
	this.layoutManager("vSpacing", 0);

	var toolbar  = new Widget_Panel(this.id + "_Toolbar", this);
	toolbar.layoutManager("spacing", 0);
	toolbar.cssClass("ToolBar");

	var i; for (i in buttonNames) {
		var buttonName = buttonNames[i];
		if (buttonName == "|") {
			var seperator = new Widget_Label(this.id + "_Seperator_" + i, toolbar);
			seperator.cssClass("ToolBarSeperator");
			seperator.image("images/icons/Seperator.gif");
			seperator.imageWidth(6);
			seperator.imageHeight(20);
 			seperator.layoutHint("minWidth", 6);
			continue;
		}
		var button = new Widget_Button(this.id + "_" + buttonName, toolbar);
		button.image("images/icons/" + buttonName + ".gif");
		button.imageWidth(16);
		button.imageHeight(16);
 		button.layoutHint("minWidth", 24);
		button.flat(1);
		button.tooltip(buttonName);
		button.setListener("Click", this, this.execCommand, buttonName);
	}
 	var htmlArea = new Widget_HTMLArea(this.id + "_HTMLArea", this);
	htmlArea.cssClass("Contains_HTMLEditor_HTMLArea");
	htmlArea.layoutHint("hFlex", 1);
	htmlArea.layoutHint("vFlex", 1);
	htmlArea.layoutHint("hAlign", "fill");
	htmlArea.layoutHint("vAlign", "fill");
	htmlArea.setListener("Change", this, this.onChange);
}

// accessing ------------------------------------------------------------------

_.value = function (value) { this.lastChild().value(value) }

// commands -------------------------------------------------------------------

_.execCommand = function (event, commandName) {
	var range =  this.window.doc.selection.createRange();
	if (range.htmlText) {
		range.execCommand(commandName);
	} else {
		this.window.element.document.execCommand(commandName);
	}
	this.lastChild().element.focus();
}

// event handling--------------------------------------------------------------

_.onChange = function (event) {
	this.fireEvent({
		"name"     : "Change",
		"sourceId" : this.id,
		"value"    : event.value
	});
	return true;
}

// template methods -----------------------------------------------------------

_.getWidgetClass   = function () { return "HTMLEditor" }
_.getDefaultLayout = function () { return "grid" }
_.clear            = function () { this.value("") }
