
Class("Widget_Label", "Widget_Base");

// creating ----------------------------------------------------------------

_.populate = function () {
	this.hasImage  = false;
	var body       = this.createElement("span");
	body.className = this.getWidgetClass() + "_Body";
	this.element.insertBefore(body);
}

// accessing ---------------------------------------------------------------

_.value = function (value) {
	this.element.lastChild.innerText = value;
	this.layoutChanged();
}

_.hAlign = function (value) { this.element.style.textAlign = value }

_.disabled = function (value) {
	this.Widget_Base_disabled(value);
	var image = this.getImage();
	if (image) image.style.filter = parseInt(value)? "gray()" : "";
}

_.focus = function () {
	this.element.lastChild.tabIndex = 1;
	this.element.lastChild.focus();
}

_.blur = function () { this.element.lastChild.blur() }

_.getWidgetOfElement = function (element) {
	var re = new RegExp("^" + this.getWidgetClass() + "(_Selected)?$");
	if (element.className.match(re)) return element.me;
	var result = element.parentElement;
	if (result.className.match(re)) return result.me;
	return false;
}

// image management --------------------------------------------------------

_.imageWidth = function  (value)
	{ this.getImage().width  = value; this.layoutChanged() }
_.imageHeight = function (value)
	{ this.getImage().height = value; this.layoutChanged() }

_.image = function (value) {
	var image;
	if (value == "") {
		if (!this.hasImage) return;
		var image     = this.getImage();
		this.hasImage = false;
		this.layoutChanged();
		return this;
	} 
	if (this.hasImage) {
		var image = this.getImage();
		image.src = value;
		this.layoutChanged();
		return;
	}
	var image       = this.createElement("img");
	image.className = this.getWidgetClass() + "_Image";
	image.src       = value;
	image.align     = "absmiddle";
	this.hasImage   = true;
	this.element.insertBefore(image, this.element.lastChild);
	this.layoutChanged();
}

_.getImage = function () {
	if (!this.hasImage) return false;
	var images = this.element.getElementsByTagName("img");
	if (!images.length) return false;
	return images[images.length - 1];
}

// selection ---------------------------------------------------------------

_.select = function () {
	this.element.className    += "_Selected";
	var body                   = this.element.lastChild;
	body.className             = this.getWidgetClass() + "_Body_Selected";
	this.isSelected            = true;
	var image                  = this.getImage();
	if (image) image.className = this.getWidgetClass() + "_Image_Selected";
}

_.deselect = function () {
	this.element.className     = this.getWidgetClass();
	var body                   = this.element.lastChild;
	body.className             = this.getWidgetClass() + "_Body";
	this.isSelected            = false;
	var image                  = this.getImage();
	if (image) image.className = this.getWidgetClass() + "_Image";
}

// testing -----------------------------------------------------------------

_.getValue = function () { return this.element.lastChild.innerText }

// template methods --------------------------------------------------------

_.getTag         = function () { return "div"   }
_.getWidgetClass = function () { return "Label" }
_.isSelectable   = function () { return false   }

