
Class("Widget_Layout_Box");
Class_Singleton();

_.initLayout = function (widget) {
	widget.layout.manager.orient =
		widget.layout.manager.orient || "horizontal";
}

_.computeMinWidth = function (widget) {
	var children = widget.getChildren();
	var count    = children.length;
	if (!count) return 0;
	var space    = 0;
	var manager  = widget.layout.manager;
	var i;
	if (manager.orient == "horizontal") {
		for (i in children) { var child = children[i];
			var childHint = child.layout.hint.width;
			space += isDefined(childHint)? childHint: child.getMinWidth();
		}
		return space;
	}
	if (manager.orient == "vertical") {
		for (i in children) { var child = children[i];
			var childHint = child.layout.hint.width;
			space = Math.max(space, isDefined(childHint)?
				childHint: child.getMinWidth());
		}
		return space;
	}
	Throw("unknown orientation: [" + manager.orient + "]");
}

_.computeMinHeight = function (widget) {
	var children = widget.getChildren();
	var count    = children.length;
	if (!count) return 0;
	var space    = 0;
	var manager  = widget.layout.manager;
	var i;
	if (manager.orient == "vertical") {
		for (i in children) { var child = children[i];
			var childHint = child.layout.hint.height;
			space += isDefined(childHint)? childHint: child.getMinHeight();
		}
		return space;
	}
	if (manager.orient == "horizontal") {
		for (i in children) { var child = children[i];
			var childHint = child.layout.hint.height;
			space = Math.max(space, isDefined(childHint)? childHint: child.getMinHeight());
		}
		return space;
	}
	Throw("unknown orientation: [" + manager.orient + "]");
}

_.setChildrenBounds = function (widget) {
	var children        = widget.getChildren();
	var count           = children.length;
	if (!count) return;
	var totalWidth      = widget.getChildrenAreaWidth();
	var totalHeight     = widget.getChildrenAreaHeight();
	if (!totalWidth || !totalHeight) return;
	var totalFlex       = 0;
	var i;
	for (i in children) {
		var childHint = children[i].layout.hint;
		if (!isDefined(childHint.width) && !isDefined(childHint.height))
			totalFlex += (childHint.flex || 0);
	}
	var removeFlexFromLastChild = false;
	if (totalFlex == 0) {
		children[children.length - 1].layout.hint.flex = 1;
		totalFlex = 1;
		removeFlexFromLastChild = true;
	}
	if (widget.layout.manager.orient == "horizontal")
		this.setChildrenBoundsHorizontal(widget, totalFlex);
	else
		this.setChildrenBoundsVertical(widget, totalFlex);
	if (removeFlexFromLastChild)
		children[children.length - 1].layout.hint.flex = 0;
}

_.setChildrenBoundsHorizontal = function (widget, totalFlex) {
	var totalWidth  = widget.getChildrenAreaWidth();
	var totalHeight = widget.getChildrenAreaHeight();
	var flexSize    = (totalWidth + widget.getTrimWidth() - widget.getMinWidth()) / totalFlex;
	if (flexSize < 0) flexSize = 0;
	var x           = widget.getChildEdgeLeft();
	var y           = widget.getChildEdgeTop ();
	var children    = widget.getChildren();
	var lastIndex   = children.length - 1;
	var freeSpace   = totalWidth;
	var child, childHint, childSize;
	for (i in children) {
		child      = children[i];
		childHint  = child.layout.hint;
		childSize  = isDefined(childHint.width)?
			childHint.width:
			child.getMinWidth() + parseInt((childHint.flex || 0) * flexSize);
		freeSpace -= childSize;
		if (i == lastIndex && freeSpace > 0) childSize += freeSpace;
		child.setBounds(y, x, childSize, totalHeight);
		x += parseInt(childSize);
	}
}

_.setChildrenBoundsVertical = function (widget, totalFlex) {
	var totalWidth   = widget.getChildrenAreaWidth();
	var totalHeight  = widget.getChildrenAreaHeight();
	var flexSize     = (totalHeight + widget.getTrimHeight() - widget.getMinHeight()) / totalFlex;
	if (flexSize < 0) flexSize = 0;
	var x            = widget.getChildEdgeLeft();
	var y            = widget.getChildEdgeTop ();
	var children     = widget.getChildren();
	var lastIndex    = children.length - 1;
	var freeSpace    = totalHeight;
	var child, childHint, childSize;
	for (i in children) {
		child      = children[i];
		childHint  = child.layout.hint;
		childSize  = isDefined(childHint.height)?
			childHint.height:
			child.getMinHeight() + parseInt((childHint.flex || 0) * flexSize);
		freeSpace -= childSize;
		if (i == lastIndex && freeSpace > 0) childSize += freeSpace;
		child.setBounds(y, x, totalWidth, childSize);
		y += parseInt(childSize);
	}
}

/*

Children are layed out in a in a box, horizontal or vertical. You can
set size and flex on children for the axis of the container orientation.
Children are fully stretched on the axis perpendicular to the container
orientation.

Layout manager keys:

orient - horizontal/vertical.

Layout manager hints:

height, width - Integer. This size will be used on the child, overriding
the layout policy. You should only set the one parallel to the container
orientation.

flex - Integer. How greedy is this child for space?

*/



