
Class("Widget_Layout_Fill");
Class_Singleton();

_.initLayout = function (widget) {
	widget.layout.manager.orient =
		widget.layout.manager.orient || "horizontal";
}

_.computeMinWidth = function (widget) {
	var orient   = widget.layout.manager.orient;
	var children = widget.getChildren();
	var space    = 0;
	var i; for (i in children) {
		var child = children[i];
		space = Math.max(space, child.getMinWidth());
	}
	space = (orient == "horizontal")? (space * children.length): space;
	return space;
}

_.computeMinHeight = function (widget) {
	var orient   = widget.layout.manager.orient;
	var children = widget.getChildren();
	var space    = 0;
	var i; for (i in children) {
		var child = children[i];
		space = Math.max(space, child.getMinHeight());
	}
	space = (orient == "horizontal")? space: (space * children.length);
	return space;
}

_.setChildrenBounds = function (widget) {
	var totalWidth  = widget.getChildrenAreaWidth ();
	var totalHeight = widget.getChildrenAreaHeight();
	if (!totalWidth || !totalHeight) return;

	var orient   = widget.layout.manager.orient;
	var children = widget.getChildren();
	var count    = children.length;
	if (!count) return;

	var childWidth, childHeight;
	var x = widget.getChildEdgeLeft();
	var y = widget.getChildEdgeTop ();

	if (orient == "horizontal") {
		childWidth = parseInt(totalWidth / count);
		if (!childWidth) return;
		childHeight = totalHeight;
		x += parseInt((totalWidth % count) / 2);
		var i; for (i in children) {
			var child = children[i];
			child.setBounds(y, x, childWidth, childHeight);
			x += childWidth;
		}
		return;
	}
	childWidth  = totalWidth;
	childHeight = parseInt(totalHeight / count);
	if (!childHeight) return;
	y = y + parseInt((totalHeight % count) / 2);
	var i; for (i in children) {
		var child = children[i];
		child.setBounds(y, x , childWidth, childHeight);
		y += childHeight;
	}
}
