
Class("Widget_Layout_Row");
Class_Singleton();

_.initLayout = function (widget) {
	var manager = widget.layout.manager;
	manager.spacing = parseInt(manager.spacing) || 3;
	manager.pack    = parseInt(manager.pack)    || 1;
	manager.justify = parseInt(manager.justify) || 0;
}

_.computeMinWidth = function (widget) {
	var children = widget.getChildren();
	var count    = children.length;
	if (!count) return 0;
	this.prepare(widget);
	var space    = 0;
	var layout   = widget.layout;
	var manager  = layout.manager;
	var spacing  = (manager.spacing * (count -1));
	if (!manager.pack) {
		var i; for (i in children) { var child = children[i];
			space = Math.max(space,
				child.layout.hint.width || child.getMinWidth()
			);
		}
		space = 0 + spacing + (space * count);
		return space;
	}

	var i; for (i in children) { var child = children[i];
		space += child.layout.hint.width || child.getMinWidth();
	}
	space += 0 + spacing;
	return space;
}

_.computeMinHeight = function (widget) {
	var children = widget.getChildren();
	var count    = children.length;
	if (!count) return 0;
	this.prepare(widget);
	var space    = 0;
	var layout   = widget.layout;
	var manager  = layout.manager;
	var i; for (i in children) { var child = children[i];
		space = Math.max(space,
			child.layout.hint.height || child.getMinHeight()
		);
	}
	return space;
}

_.setChildrenBounds = function (widget) {
	var children    = widget.getChildren();
	var count       = children.length;
	if (!count) return;
	this.prepare(widget);
	var layout          = widget.layout;
	var manager         = layout.manager;
	var totalWidth      = widget.getChildrenAreaWidth();
	var totalHeight     = widget.getChildrenAreaHeight();
	if (!totalWidth || !totalHeight) return;
	var spacing         = manager.spacing || 0;
	var childWidth      = 0;
	var childHeight     = 0;
	var x               = widget.getChildEdgeLeft();
	var y               = widget.getChildEdgeTop ();
	var childOuterWidth = 0;

	if (manager.justify) {
		childOuterWidth = parseInt(totalWidth / count);
		x = x + parseInt((totalWidth % count) / 2);
	}
	if (!manager.pack) {
		childHeight = widget.getMinHeight() - widget.getTrimHeight();
		var i; for (i in children) { var child = children[i];
			childWidth = Math.max(childWidth,
				child.layout.hint.width || child.getMinWidth()
			);
		}
	}
	var i; for (i in children) { var child = children[i];
		if (manager.pack) {
			childWidth  = child.layout.hint.width  || child.getMinWidth();
			childHeight = child.layout.hint.height || child.getMinHeight();
		}
		var childX = manager.justify?
			x + parseInt((childOuterWidth - childWidth) / 2): x;
		childHeight += parseInt(child.layout.hint.extraHeight) || 0;
		child.setBounds(y, childX, childWidth, childHeight);
		x += (childOuterWidth? childOuterWidth: childWidth) + spacing;
	}
}

_.prepare = function (widget) {
	var cache = widget.layout.cache;
	if (cache.manager && cache.manager.prepared) return;
	cache.manager = {};
	var manager = widget.layout.manager;
	manager.spacing = parseInt(manager.spacing);
	manager.pack    = parseInt(manager.pack);
	manager.justify = parseInt(manager.justify);
	cache.manager.prepared = true;
}

/*

Children are layed out in a horizontal row.

Layout manager keys:

pack - Boolean. If true, children are sized at their minimum sizes. If
false, children are equally sized at the maximum of their minimum sizes.

justify - Boolean. If true, children are spread out to fill available
space in the composite. If false, they remain in their initial
positions, as close to each other as possible, left aligned.

spacing - Integer. Number of pixels to put between children.

Default is pack=true, justify=false, spacing=3

Layout manager hints:

height, width - Integer. This size will be used on the child, instead of
the layout policy.

*/
