
Class("Widget_Layout_Stack");
Class_Singleton();

// creating -------------------------------------------------------------------

_.initLayout = function (widget) {
	widget.layout.manager.oldSelected = -1;
	widget.layout.manager.selected    = -1;
}

// accessing ------------------------------------------------------------------

_.selectedIndex = function (widget, index)
	{ widget.layout.manager.selected = widget.getChild(index) }

// layout ---------------------------------------------------------------------

_.computeMinWidth = function (widget) {
	var children = widget.getChildren();
	var space = 0;
	var i; for (i in children) {
		var child = children[i];
		space = Math.max(space, child.getMinWidth());
	}
	return space;
}

_.computeMinHeight = function (widget) {
	var children = widget.getChildren();
	var space    = 0;
	var i; for (i in children) {
		var child = children[i];
		space = Math.max(space, child.getMinHeight());
	}
	return space;
}

_.setChildrenBounds = function (widget) {
	var totalWidth  = widget.getChildrenAreaWidth();
	var totalHeight = widget.getChildrenAreaHeight();
	if (!totalWidth || !totalHeight) return;

	var children = widget.getChildren();
	var count    = children.length;
	if (!count) return;

	var manager     = widget.layout.manager;
	var oldSelected = manager.oldSelected;
	var selected    = manager.selected;
	var i; for (i in children) {
		var child = children[i];
		if (child != selected)
			child.element.runtimeStyle.display = "none";
	}

	if (selected == -1 && oldSelected == -1)
		selected = widget.getChild(0);
	if (oldSelected != selected)
		selected.element.runtimeStyle.display = "";

	manager.selected    = selected;
	manager.oldSelected = selected;

	selected.setBounds(
		widget.getChildEdgeTop(), widget.getChildEdgeLeft (),
		totalWidth, totalHeight
	);
}

