

Class("Widget_Layout_tests_Box", "Widget_Layout_tests_TestCase");

_.getLayoutManagerName = function () { return "box" }

_.testMinSizesHorizontal = function () {
	this.create3Children("horizontal");
	this.assertEquals
		("width", this.getHelloWorldWidth() * 6, this.getMinWidth(this.subject));
	this.assertEquals
		("height", this.getCharHeight(), this.getMinHeight(this.subject));
}

_.testMinSizesVertical = function () {
	this.create3Children("vertical");
	this.assertEquals
		("width", this.getHelloWorldWidth() * 3, this.getMinWidth(this.subject));
	this.assertEquals
		("height", this.getCharHeight() * 3, this.getMinHeight(this.subject));
}

_.testMinSizesHorizontalWithHint = function () {
	var children = this.create3Children("horizontal");
	var child2   = children[1];
	child2.layoutHint("width", 44);
	child2.layoutHint("height", 55);
	this.assertEquals(
		"min width",
		this.getHelloWorldWidth() * 4 + 44,
		this.getMinWidth(this.subject)
	);
	this.assertEquals("min height", 55, this.getMinHeight(this.subject));
}

_.testMinSizesVerticalWithHint = function () {
	var children = this.create3Children("vertical");
	var child2   = children[1];
	child2.layoutHint("width", 44);
	child2.layoutHint("height", 55);
	this.assertEquals
		("width", this.getHelloWorldWidth() * 3, this.getMinWidth(this.subject));
	this.assertEquals
		("height", this.getCharHeight() * 2 + 55, this.getMinHeight(this.subject));
}

_.testHorizontalLayout1 = function () {
	var children = this.create3Children("horizontal");
	children[0].layoutHint("flex", 1);
	children[1].layoutHint("width", 7);
	this.assertChildrenWidth(children, [
		this.getWindowWidth() - 7 - (this.getHelloWorldWidth() * 3),
		7,
		this.getHelloWorldWidth() * 3
	]);
	this.assertChildrenHeight(children, [
		this.getWindowHeight(),
		this.getWindowHeight(),
		this.getWindowHeight()
	]);
}

_.testHorizontalLayout2 = function () {
	var children = this.create3Children("horizontal");
	children[1].layoutHint("width", 17);
	this.assertChildrenWidth(children, [
		this.getHelloWorldWidth(),
		17,
		this.getWindowWidth() - 17 - this.getHelloWorldWidth()
	]);
	this.assertChildrenHeight(children, [
		this.getWindowHeight(),
		this.getWindowHeight(),
		this.getWindowHeight()
	]);
}

_.testHorizontalLayout3 = function () {
	var children = this.create3Children("horizontal");
	children[1].layoutHint("width", 17);
	children[1].layoutHint("flex", 1);
	this.assertChildrenWidth(children, [
		this.getHelloWorldWidth(),
		17,
		this.getWindowWidth() - 17 - this.getHelloWorldWidth()
	]);
}

_.testHorizontalLayout4 = function () {
	var child1 = new Widget_Label("E2", this.subject);
	var child2 = new Widget_Label("E3", this.subject);
	child1.value("Hello World");
	child2.value("Hello WorldHello World");
	child1.layoutHint("width", 100);
	this.doLayout();
	this.assertEquals("width 1", 100, this.getWidgetWidth(child1));
	this.assertEquals
		("width 2", this.getWindowWidth() - 100, this.getWidgetWidth(child2));
}

_.testVerticalLayout1 = function () {
	var children = this.create3Children("vertical");
	children[0].layoutHint("flex", 1);
	children[1].layoutHint("height", 7);
	this.assertChildrenHeight(children, [
		this.getWindowHeight() - 7 - this.getCharHeight(),
		7,
		this.getCharHeight()
	]);
	this.assertChildrenWidth(children, [
		this.getWindowWidth(),
		this.getWindowWidth(),
		this.getWindowWidth()
	]);
}

_.testVerticalLayout2 = function () {
	var children = this.create3Children("vertical");
	children[1].layoutHint("height", 17);
	this.assertChildrenHeight(children, [
		this.getCharHeight(),
		17,
		this.getWindowHeight() - 17 - this.getCharHeight()
	]);
	this.assertChildrenWidth(children, [
		this.getWindowWidth(),
		this.getWindowWidth(),
		this.getWindowWidth()
	]);
}

_.testHorizontalLayoutWithTrim1 = function () {
	var children = this.create3Children("horizontal");
	var child1   = children[0];
	child1.style("margin", "1px 2px 3px 4px");
	this.assertChildrenWidth(children, [
		this.getHelloWorldWidth(),
		this.getHelloWorldWidth() * 2,
		this.getWindowWidth() - this.getHelloWorldWidth() * 3 - 2 - 4,
	]);
	this.assertChildrenHeight(children, [
		this.getWindowHeight() - 1 - 3,
		this.getWindowHeight(),
		this.getWindowHeight()
	]);
}

_.testHorizontalLayoutWithTrim2 = function () {
	var children = this.create3Children("horizontal");
	var child1   = children[0];
	child1.style("padding", "1px 2px 3px 4px");
	this.assertChildrenWidth(children, [
		this.getHelloWorldWidth(),
		this.getHelloWorldWidth() * 2,
		this.getWindowWidth() - this.getHelloWorldWidth() * 3 - 2 - 4,
	]);
	this.assertChildrenHeight(children, [
		this.getWindowHeight() - 1 - 3,
		this.getWindowHeight(),
		this.getWindowHeight()
	]);
}

_.testHorizontalLayoutWithTrim3 = function () {
	var children = this.create3Children("horizontal");
	var child1   = children[0];
	child1.style("border", "5px solid #00FF00");
	this.assertChildrenWidth(children, [
		this.getHelloWorldWidth(),
		this.getHelloWorldWidth() * 2,
		this.getWindowWidth() - this.getHelloWorldWidth() * 3 - 5 - 5,
	]);
	this.assertChildrenHeight(children, [
		this.getWindowHeight() - 5 - 5,
		this.getWindowHeight(),
		this.getWindowHeight()
	]);
}

_.testHorizontalLayoutWithParentTrim = function () {
	var children = this.create3Children("horizontal");
	this.subject.style("border", "3px solid #FF0000");
	this.subject.style("padding", "5px 6px 7px 8px");
	this.subject.style("margin", "1px 2px 3px 4px");
	var subjectWidth  = this.getWindowWidth()  - 3 - 3 - 2 - 4 - 6 - 8;
	var subjectHeight = this.getWindowHeight() - 3 - 3 - 5 - 7 - 1 - 3;
	this.assertChildrenWidth(children, [
		this.getHelloWorldWidth(),
		this.getHelloWorldWidth() * 2,
		subjectWidth - this.getHelloWorldWidth() * 3
	]);
	this.assertChildrenHeight(children, [
		subjectHeight,
		subjectHeight,
		subjectHeight
	]);
}

_.testVerticalLayoutWithTrim = function () {
	var children = this.create3Children("vertical");
	this.subject.style("border", "3px solid #FF0000");
	var child1   = children[0];
	child1.style("margin", "1px 2px 3px 4px");
	this.assertChildrenHeight(children, [
		this.getCharHeight(),
		this.getCharHeight(),
		this.getWindowHeight() - this.getCharHeight() * 2 - 1 - 3 - 6
	]);
	this.assertChildrenWidth(children, [
		this.getWindowWidth() - 2 - 4 - 6,
		this.getWindowWidth() - 6,
		this.getWindowWidth() - 6
	]);
}

_.testHorizontalLayoutWithEverything = function () {
	var children = this.create3Children("horizontal");
	this.subject.style("border", "3px solid #FF0000");
	this.subject.style("padding", "5px 6px 7px 8px");
	this.subject.style("margin", "1px 2px 3px 4px");
	var child1   = children[0];
	child1.style("margin", "9px 10px 11px 12px");
	child1.layoutHint("width", 47);
	var subjectWidth  = this.getWindowWidth()  - 3 - 3 - 2 - 4 - 6 - 8;
	var subjectHeight = this.getWindowHeight() - 3 - 3 - 5 - 7 - 1 - 3;
	this.assertChildrenWidth(children, [
		47 - 10 - 12,
		this.getHelloWorldWidth() * 2,
		subjectWidth - this.getHelloWorldWidth() * 2 - 47
	]);
	this.assertChildrenHeight(children, [
		subjectHeight - 9 - 11,
		subjectHeight,
		subjectHeight
	]);
}
