
Class("Widget_LayoutBuffer");
Class_Singleton();

_.init = function () { this.dirtyWindows = {} }

_.addDirtyWindow = function (win) { this.dirtyWindows[win.getId()] = win }

_.doLayout = function () {
	var key;
	for (key in this.dirtyWindows)
		this.dirtyWindows[key].setChildrenBounds();
	this.dirtyWindows = {};
}

/*

IE6 Client Layout
=================

You do not set size or location explicitly for widgets, instead you set
a layout manager on the parent composite, configure it, and configure
any layout hints on the child widget. The layout manager will layout the
widget according to its configuration and the hints of the children.

Configuring Layouts
-------------------

You call layoutManager("name", layoutManagerName) on a composite to set
its layout manager. Valid values are 'row', 'fill', 'stack', and 'grid'.
Panel has a default 'row' layout.

Window does its own layout management on its only permissable child- it
expands it to fill the entire window.

You call layoutManager(key, value) on a composite to set properties of
the layout manager it uses. Valid keys depend on the layout manager.
Fill has a key 'orient' for example, with possible values 'vertical' and
'horizontal'. Then 'name' key determines the layout manager that will be
used.

You call layoutHint(key, value) on a widget to give the layout manager
of the composite hints that it can use to layout the widget. Valid keys
depend on the layout manager of the containing composite. Row has a key
'width' for example, that indicates the child will have a specific
width, despite the layout manager policy.

Widgets allow you to set margin, border, and padding width. You can set
each of these for the top, left, bottom, and right edges, using the
style(key, value) method. Valid keys are 'margin', 'padding', and
'borderWidth'. If you want to set only one side, use 'marginLeft',
'paddingTop', 'borderBottom', etc. Value is string made up of integer
width in pixels, and a 'px' suffix. Nothing will work if you forget the
'px'. The layout mechanism will take padding, margins, and borders into
account.

The Framework
-------------

Layout happens automatically when you create, resize, or modify widgets.
For performance reasons layout() is called on the LayoutBuffer
(explained below) right before you want to present the UI to the user:
when done handling data from the server, when done handling some event,
or anything that may have changed the layout. No actual layout will
occur until that time. Layout is expensive, so you want to minimize
calls to layout(). The framework calls the method after a server message
has been processed.

The sum of the margin, padding, and border width for each side is called
the trim. The rectangle constraining the widget is called the bounds.
The rectangle that is left after you remove the trim from the bounds is
called the client area.

Widget size and location are set by their parent layout manager, through
the method setBounds(top, left, width, height). Only layout managers and
windows should call this method. All parameters are in pixels, top and
left are from top left corner of the parent.

Base implements setBounds() and an empty setChildrenBounds(), as it has
no children. setBounds() checks if the widget is dirty (dirtiness is
explained below) or the new bounds different from old bounds, if so sets
new top/leftwidth/height on element according to the parameters, sets
widget as clean (not dirty), and calls setChildrenBounds().

Composites must implement setChildrenBounds(), where they setBounds() on
each of their children, by calling layout() on their layout manager.

Window has an event handler for window resize, that calls
setChildrenBounds(). setChildrenBounds() calls setBounds() on the child
of the window. This goes down the containement hierarchy until reaching
non-composites, or composites whose size has not changed and are not
dirty .

Layout side effect methods are methods that require some layout of
widgets after they are done.

   Widget_Label value
   Widget_Base style, clear, flex, minWidth, layoutHint, ...
   Widget_Panel add, layoutManager, ...

These methods must end in a call to this.layoutChanged(). These are methods
that make the widget dirty. When you call layout you are saying the
widget needs layout, and perhaps its composite and children require a
layout as well.

To start laying out windows after some widgets have become dirty, call
layout() on the LayoutBuffer. This will make them clean again. The
Client_Runner does this, after a server message has been run. When
testing you must call layout() on the buffer or no widgets will be layed
out after becoming dirty. The job of the layout buffer is to reduce the
number of layouts, leading to much better performance.

Widget_Base implements layout() so: if the isLayoutDirty flag is true,
we return. If not, we set it to true, reset all size caches and call
layout() on the parent.

Unlike Widget_Panel or Widget_Label, some widgets do not require any
layout upon state changes. Widget_List and Widget_ListItem need no
laying out when the value is changed, or an item added. These implement
empty layout().

Widget_Window implements layout() by calling the LayoutBuffer
addDirtyWindow() method. So after a bubbling frenzy, the window is
dirty.

Layout_Buffer layout() calls setChildrenBounds() on each registered
dirty window. This goes down the containement hierarchy, but only on
dirty paths, since the window size has not changed.

Above all this there is a mechanism to cache sizes important for layout.

*/
