
use("Widget_List_Item");
use("Widget_List_Controller_Single");

Class("Widget_List", "Widget_Panel");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.Widget_Panel_populate();
	this.element["on" + this.getDOMSelectEventName()] = this.onClick;
	this.controller = this.getDefaultController();
}

_.destroy = function (dontPaint) {
	this.element["on" + this.getDOMSelectEventName()] = null;
	this.Widget_Base_destroy(dontPaint);
}

_.clear = function () {
	this.clearElementChildren(this.element);
	this.selectedId = -1;
}

// accessing  -----------------------------------------------------------------

_.insert = function (index, value) {
	var itemId    = this.id + "_Item_" + this.window.doc.uniqueID;
	var itemClass = this.getItemClass();
	var item      = new itemClass(itemId, this, index);
	item.value(value);
	return item;
}

_.setItemImage = function (index, url, width, height) {
	var item = this.getChild(index);
	item.image(url);
	item.imageWidth(width);
	item.imageHeight(height);
}

_.removeByIndex = function (index) {
	this.getController().removeItem(this, this.getChild(index));
	this.Widget_Panel_removeByIndex(index);
}

// selection  -----------------------------------------------------------------

_.selectedIndex = function (index) {
	var controller = this.getController();
	var item = index == -1? -1: this.getChild(index);
	controller.selectItem(this, item);
}

// event handling -------------------------------------------------------------

_.onClick = function () {
	var me = this.me;
	if (!me.hasChildren()) return;
	if (me.getIsDisabled()) return;

	var source = this.document.parentWindow.event.srcElement;
	if (source == me.element) return;
	var item = me.getChild(0).getWidgetOfElement(source);
	if (item.isSelected) {
		item.focus();
		return;
	}

	var index       = me.getIndexOfChild(item);
	var controller  = me.getController();
	var shiftOn     = 0;
	var ctrlOn      = 0;
	var serverEvent = controller.onClick(me, index, shiftOn, ctrlOn);
	if (!serverEvent) return;
	me.fireEvent(serverEvent);
	return true;
}

_.getController = function ()           { return this.controller }
_.setController = function (controller) { this.controller = controller }

// template methods -----------------------------------------------------------

_.getTag                = function () { return "div"   }
_.getWidgetClass        = function () { return "List"  }
_.propagateDisable      = function () { return false   }
_.layoutOnChildChange   = function () { return false   }
_.getDefaultLayout      = function () { return "null"  }
_.getDOMSelectEventName = function () { return "click" }
_.getItemClass          = function () { return Widget_List_Item }
_.getDefaultController  = function () { return Widget_List_Controller_Single.get() }
