
use("Widget_NoteBook_Header");

Class("Widget_NoteBook", "Widget_Panel");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.Widget_Panel_populate();
	this.layout.manager.hSpacing = 0;
	this.layout.manager.vSpacing = 0;
}

_.destroy = function (dontPaint) {
	if (this.childCount())
		this.getChild(0).element.onclick = null;
	this.Widget_Base_destroy(dontPaint);
}

// accessing ------------------------------------------------------------------

_.selectedIndex = function (index) {
	var header = this.getChild(0);
	var stack  = this.getChild(1);
	header.selectedIndex(index);
	stack.layout.manager.fly.selectedIndex(stack, index);
	stack.layoutChanged();
}

// compositing ----------------------------------------------------------------

_.add = function (child, index) {
	var count = this.childCount();
	if (count > 1) Throw("Notebook can only host Header and Stack");
	var hint    = child.layout.hint;
	hint.hFlex  = 1;
	hint.hAlign = "fill";
	if (count) {
		hint.vFlex                 = 1;
		hint.vAlign                = "fill";
		child.element.className    = "Stack";
		child.element.style.zIndex = 1;
		child.layoutManager("name", "stack");
	} else {
		hint.indent                = 1;
		hint.extraHeight           = 4;
		child.element.style.zIndex = 2;
		child.element.onclick      = this.onClick;
	}
	this.Widget_Panel_add(child, index);
}

// event handling -------------------------------------------------------------

_.onClick = function () {
	var header = this.me;
	if (!header.hasChildren()) return;
	var me = header.parent;
	if (me.getIsDisabled()) return;

	var source = this.document.parentWindow.event.srcElement;
	if (source == me.element || source == header.element) return;
	if (source.className.match(/_Selected/)) return;

	var tab = header.getChild(0).getWidgetOfElement(source);
	if (tab.isSelected) return;
	var index = header.getIndexOfChild(tab);
	me.selectedIndex(index);
	me.fireEvent({
		"name"         : "SelectionChanged",
		"sourceId"     : me.id,
		"selectedIndex": index
	});
	return true;
}

// template methods -----------------------------------------------------------

_.getWidgetClass   = function () { return "NoteBook" }
_.getDefaultLayout = function () { return "grid" }
