
use("Widget_Layout_Null");
use("Widget_Layout_Fill");
use("Widget_Layout_Row");
use("Widget_Layout_Grid");
use("Widget_Layout_Stack");
use("Widget_Layout_Box");

Class("Widget_Panel", "Widget_Base");

Widget_Panel.layoutMap = {
	"null" : Widget_Layout_Null.get(),
	"fill" : Widget_Layout_Fill.get(),
	"row"  : Widget_Layout_Row.get(),
	"grid" : Widget_Layout_Grid.get(),
	"box"  : Widget_Layout_Box.get(),
	"stack": Widget_Layout_Stack.get()
};

// creating -------------------------------------------------------------------

_.populate = function () {
	var defaultLayout = Widget_Panel.layoutMap[this.getDefaultLayout()];
	this.layout.manager = {"fly" : defaultLayout};
	defaultLayout.initLayout(this);
}

_.destroy = function (dontPaint) {
	var children = this.getChildren();
	var i; for (i in children) {
		var child = children[i];
		child.destroy(child, 1);
	}
	this.layout.manager = undefined;
	this.Widget_Base_destroy(dontPaint);
}

// compositing ----------------------------------------------------------------

_.add = function (child, index) {
	var parentElement = this.getMainElement(parent);
	var childElement  = child.element;
	if (index == undefined) {
		parentElement.appendChild(childElement);
	} else {
		index        = parseInt(index);
		var children = parentElement.childNodes;
		var count    = children.length;
		if (count == 0) {
			parentElement.appendChild(childElement);
		} else {
			parentElement.insertBefore(
				childElement,
				(index == count)? null: this.getChild(index).element
			);
		}
	}
	this.childCountChanged();
}

_.removeByIndex = function (index) { 
	this.getChild(index).destroy();
	this.childCountChanged();
}

_.getIndexOfChild = function (child) {
	var childId       = child.id;
	var index         = 0;
	var parentElement = this.getMainElement();
	var childElement  = parentElement.firstChild;
	while (childElement.id != childId) {
		childElement = childElement.nextSibling;   
		index++; 
	}
	return index;
}

_.getChildren = function () {
	var cachedChildren = this.layout.cache.children;
	if (cachedChildren) return cachedChildren;

	var parentElement = this.getMainElement();
	var childElement  = parentElement.firstChild;
	var children      = new Array();
	while (childElement) {
		children.push(childElement.me);
		childElement = childElement.nextSibling;    
	}
	this.layout.cache.children = children;
	return children;
}

_.getChildrenRecursive = function () {
	var myChildren  = this.getChildren();
	var allChildren = myChildren;
	var i; for (i in myChildren) {
		var child = myChildren[i];
		allChildren.push( child.getChildrenRecursive() );
	}
	return allChildren;
}

_.getChild = function (index)
	{ return this.getMainElement().children.item(parseInt(index)).me }

_.childCount    = function () { return this.getMainElement().children.length }
_.hasChildren   = function () { return this.getMainElement().hasChildNodes() }
_.firstChild    = function () { return this.getChild(0) }
_.lastChild     = function () { return this.getChild(this.childCount() - 1) }

// layout ---------------------------------------------------------------------

_.layoutManager = function (key, value) {
	var config = this.layout.manager;
	if (key == "name") {
		config.fly = Widget_Panel.layoutMap[value];
		if (!config.fly) Throw("no such layout manager [" + value + "]");
		config.fly.initLayout(this);
	} else {
		config[key] = value == "0"? 0: value;
	}
	this.layoutChanged();
}

_.setChildrenBounds = function () { try {
	this.layout.manager.fly.setChildrenBounds(this);
} catch (e) { Throw(e, "Setting: " +
	this.getChildrenAreaWidth()  + ":" + this.getChildrenAreaHeight() + ":" +
	this.getChildEdgeLeft()      + ":" + this.getChildEdgeTop ()
)}}

_.computeMinWidth = function () {
	return this.layout.manager.fly.computeMinWidth(this) + this.getTrimWidth();
}

_.computeMinHeight = function () {
	return this.layout.manager.fly.computeMinHeight(this) + this.getTrimHeight();
}

_.getChildrenAreaWidth  = function () { return this.layout.cache.innerWidth }
_.getChildrenAreaHeight = function () { return this.layout.cache.innerHeight }

_.childCountChanged = function ()
	{ if (this.layoutOnChildChange()) this.layoutChanged() }

// template methods -----------------------------------------------------------

_.disableChildren = function (value) {
	if (!this.propagateDisable()) return;
	var children = this.getChildren();
	var i; for (i in children) {
		var child = children[i];
		child.disabled(value);
	}
}

_.getTag              = function () { return "div"        }
_.getWidgetClass      = function () { return "Panel"      }
_.getDefaultLayout    = function () { return "row"        }
_.propagateDisable    = function () { return true         }
_.getMainElement      = function () { return this.element }
_.layoutOnChildChange = function () { true                }
