
use("Widget_SelectBox_Item");
use("Widget_SelectBox_Controller_Single");

Class("Widget_SelectBox", "Widget_List");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.Widget_List_populate();
	this.createUnselected();
}

_.clear = function () {
	this.Widget_List_clear();
	this.createUnselected();
}

// accessing  -----------------------------------------------------------------

_.insert = function (index, value) {
	index    = this.checkIndex(index);
	var item = this.Widget_List_insert(index, value);
	this.checkVisibility();
	return item;
}

_.removeByIndex = function (index) {
	var i          = this.checkIndex(index);
	var controller = this.getController();
	var item       = this.getChild(i);
	controller.removeItem(this, item);
	item.destroy(item);
	this.checkVisibility();
}

// selection  -----------------------------------------------------------------

_.selectedIndex = function (index) {
	this.Widget_List_selectedIndex(this.checkIndex(index));
	if (index != -1) this.element.selectedIndex = index;
	this.checkVisibility();
}

// no selection  methods  -----------------------------------------------------

_.createUnselected = function () {
	var nonItem                = this.insert(0, "");
	this.element.selectedIndex = 0;
	this.nonItem               = nonItem;
	this.selectedId            = -1;
	this.checkVisibility();
}

_.removeUnselected = function () {
	this.nonItem.element.removeNode(true);
	this.selectedId = 0;
	this.checkVisibility();
}

_.checkIndex = function (index){
	var i = index;
	if (this.selectedId == -1 && index != -1) i++;
	return i;
}

// flash bug methods  ---------------------------------------------------------

_.checkVisibility = function () {
	if (this.element.offsetWidth == 0){
		this.element.style.display = "inline";
		this.element.style.display = "none";
		this.layoutChanged();
	}
}

_.setChildrenBounds = function () { this.element.style.display = "inline" }

// layout  --------------------------------------------------------------------

_.computeMinHeight = function ()
	{ return this.window.computeMinHeight(this.element) }

// event handling -------------------------------------------------------------

_.onClick = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;

	var widgetId   = me.id;
	var windowId   = me.window.id;
	var index      = me.element.selectedIndex;
	var controller = me.getController();
	var shiftOn    = 0;
	var ctrlOn     = 0;
	var event      = controller.onClick(me, index++, shiftOn, ctrlOn);
	if (!event) return;
	me.fireEvent(event);
	return true;
}

// template methods -----------------------------------------------------------

_.getTag                = function () { return "select" }
_.getWidgetClass        = function () { return "SelectBox" }
_.getDOMSelectEventName = function () { return "change" }
_.getItemClass          = function () { return Widget_SelectBox_Item }
_.getDefaultController  = function () { return Widget_SelectBox_Controller_Single.get() }
