
use("Widget_Splitter");
use("Widget_LayoutBuffer");

Class("Widget_SplitPanel", "Widget_Panel");

Widget_SplitPanel.splitterSize = 6;

// compositing ----------------------------------------------------------------

_.add = function (child) {
	if (!this.isSplitter(child) && this.hasChildren()) {
		var splitter     = new Widget_Splitter(parent.id + "_splitter", this);
		var isHorizontal = (this.layout.manager.orient == "horizontal");
		splitter.layoutHint
			(isHorizontal? "width": "height", Widget_SplitPanel.splitterSize);
		splitter.cssClass("Splitter_" + (isHorizontal? "Horizontal": "Vertical"));
		splitter.element.onmousedown      = this.onMouseDown;
		splitter.element.onmousemove      = this.onMouseMove;
		splitter.element.onmouseup        = this.onMouseUp;
		splitter.element.onreleasecapture = this.onReleaseCapture;
	}
	this.Widget_Panel_add(child);
}

_.isSplitter = function (widget)
	{ return widget.element.className.match(/^Splitter/)? true: false }

// event handling -------------------------------------------------------------

_.onMouseDown = function() {
	var splitter           = this.me;
	var me                 = splitter.parent;
	var isHorizontal       = me.layout.manager.orient == "horizontal";
	var index              = me.getIndexOfChild(splitter);
	var child              = me.getChild(index - 1);
	var childOuterSize     = child.layout.cache[isHorizontal? "outerWidth": "outerHeight"];
	var childStart         = isHorizontal? child.getOffsetLeft(child): child.getOffsetTop(child);
	var splitterStart      = childStart + childOuterSize;
	var event              = me.element.document.parentWindow.event;
	var mousePos           = isHorizontal? event.clientX: event.clientY;
	me.mouseSplitterOffset = mousePos - splitterStart;
	me.isCapturing         = true;
	splitter.element.setCapture(true);
}

_.onMouseMove = function () {
	var splitter           = this.me;
	var me                 = splitter.parent;
	if (!me.isCapturing) return;
	var isHorizontal       = me.layout.manager.orient == "horizontal";
	var axis               = isHorizontal? "width": "height";
	var parentStart        = isHorizontal? me.getOffsetLeft(): me.getOffsetTop();
	var index              = me.getIndexOfChild(splitter);
	var child              = me.getChild(index - 1);
	var childStart         = parentStart + child.layout.cache[isHorizontal? "left": "top"];
	var parentEnd          = parentStart + me.layout.cache[isHorizontal? "innerWidth": "innerHeight"];
	parentEnd              += isHorizontal? me.getTrimLeft(): me.getTrimTop();
	var maxPos             = parentEnd + me.mouseSplitterOffset - me.splitterSize - 1;
	var event              = me.element.document.parentWindow.event;
	var mousePos           = isHorizontal? event.clientX: event.clientY;
	if (mousePos > maxPos) mousePos = maxPos;
	var size               = mousePos - me.mouseSplitterOffset - childStart;
	if (size < 1) size = 1;
	child.layoutHint(axis, size);
	Widget_LayoutBuffer.get().doLayout();
}

_.onMouseUp = function () {
	var splitter   = this.me;
	var me         = splitter.parent;
	me.isCapturing = false;
	splitter.element.releaseCapture();
}

_.onReleaseCapture = function () {
	var splitter   = this.me;
	var me         = splitter.parent;
	me.isCapturing = false;
}

// template methods -----------------------------------------------------------

_.getWidgetClass   = function () { return "SplitPanel" }
_.getDefaultLayout = function () { return "box"   }

