
use("Widget_Tree_Node");
use("Widget_Tree_Controller_Single");

Class("Widget_Tree", "Widget_Panel");

// creating -------------------------------------------------------------------

_.populate = function () {
	this.Widget_Panel_populate();
	this.element.onmousedown = this.onClick;
	this.element.ondblclick  = this.onClick;
	this.controller          = this.getDefaultController();
}

_.destroy = function (dontPaint) {
	this.element.onmousedown = null;
	this.element.ondblclick  = null;
	this.Widget_Base_destroy(dontPaint);
}

// accessing  -----------------------------------------------------------------

_.insert = function (pathString, value) {
	var parent, parentNode;
	var path  = this.getPathFromString(pathString);
	var index = path.pop();
	if (!path.length) {
		parent = this;
	} else {
		parentNode = this.getNodeAt(path);
		parent     = parentNode.getBody();
	}
	var nodeId = this.id + "_Node_" + this.window.doc.uniqueID;
	var node   = new Widget_Tree_Node(nodeId, parent, index);
	node.value(value);
	if (parentNode) parentNode.nodeCountUp();
	return node;
}

_.removeByPath = function (path) {
	var node = this.getNodeAt(this.getPathFromString(path));
	this.getController().removeNode(this, node);
	node.destroy();
}

_.setNodeImage = function (pathString, url, width, height) {
	var path    = this.getPathFromString(pathString);
	var node    = this.getNodeAt(path);
	var head    = node.getHead();
	head.image(url);
	head.imageWidth(width);
	head.imageHeight(height);
}

_.setNodeValue = function (pathString, value)
	{ this.getNodeAt(this.getPathFromString(pathString)).value(value) }

_.getNodeAt = function (path) {
	if (!path.length) return this;
	var node;
	var i; for (i in path) {
		if (i==0){
			node = this.getChild(path[i])
		} else {
			var body = node.getBody();
			node = body.getChild(path[i]);
		}
	}
	return node;
}

_.getNodePath = function (node) {
	var rootId      = this.id;
	var path        = new Array();
	var currentNode = node;
	do {
		var parentBody = currentNode.element.parentElement.me;
		var parentNode = parentBody.element.parentElement.me;
		path.push(parentBody.getIndexOfChild(currentNode));
		currentNode = parentNode;
	} while (parentBody.id != rootId)
	path = path.reverse();
	return path;
}

_.getPathFromString = function (pathString) {
	if (typeof pathString != "string") return pathString;
	var path = pathString == ""?
		new Array(): pathString.split("_");
	return path;
}

// selection  -----------------------------------------------------------------

_.selectedPath = function (pathString) {
	var path =  this.getPathFromString(pathString);
	var node = path == -1? -1: this.getNodeAt(path);
	this.getController().selectNode(this, node);
}

// event handling -------------------------------------------------------------

_.onClick = function () {
	var me = this.me;
	if (me.getIsDisabled()) return;
	if (!me.hasChildren()) return;

	var event  = this.document.parentWindow.event;
	var source = event.srcElement;
	if (source == this.element) return;

	var head = me.getChild(0).getHead().getWidgetOfElement(source);
	if (!head) return;
	var node = head.element.parentElement.me;

	var controller  = me.getController();
	var sourceClass = source.className;
	var serverEvent;
	if (sourceClass.match(/^Tree_NodeHead_Button/)) {
		serverEvent = controller.onFlip(me, node);
	} else {
		if (
			!sourceClass.match(/^Tree_NodeHead_Body/) &&
			!sourceClass.match(/^Tree_NodeHead_Image/)
		) return;
		serverEvent = event.type ==  "dblclick"?
			controller.onFlip(me, node):
			controller.onClick(me, node, 0, 0);
	}
	if (!serverEvent) return;
	me.fireEvent(serverEvent);
	return true;
}

_.getController = function ()           { return this.controller }
_.setController = function (controller) { this.controller = controller }

// template methods -----------------------------------------------------------

_.getWidgetClass       = function () { return "Tree" }
_.propagateDisable     = function () { return false }
_.defaultBubbleLayout  = function () { return false }
_.getDefaultLayout     = function () { return "null"   }
_.getDefaultController = function () { return Widget_Tree_Controller_Single.get() }
