
use("Widget_WindowManager");

Class("Widget_Tree_Controller_Single");
Class_Singleton();

// accessing  -----------------------------------------------------------------

_.removeNode = function (tree, node)
	{ if (node.id == tree.selectedId) tree.selectedId = -1 }

_.focusNode = function (tree, node) {
	if (node == -1) return;
	node.focus();
}

_.selectNode = function (tree, node) {
	var oldId = tree.selectedId;
	if (oldId && oldId != -1)
		Widget_WindowManager.get().
			getWidget(tree.window.getId(), oldId).deselect();
	if (node == -1 ) {
		tree.selectedId = -1;
		return;
	}
	tree.selectedId = node.id;
	node.select();
}

// event handling -------------------------------------------------------------

_.onClick = function (tree, node, shiftOn, ctrlOn) {
	var oldId = tree.selectedId;
	if (oldId == node.id) return false;
	this.selectNode(tree, node);
	this.focusNode(tree, node);
	var path = tree.getNodePath(node);
	var event = {
		"name"         : "SelectionChanged",
		"sourceId"     : tree.id,
		"selectedPath" : path.length? path.join("_"): ""
	};
	return event;
}

_.onFlip = function (tree, node) {
	node.flip();

	if (node.getButtonState(node) == "O") {
		var treeElement = tree.element;
		var nodeElement = node.element;
		var treeBottom  = treeElement.scrollTop + treeElement.offsetHeight;
		var nodeBottom  = nodeElement.offsetTop + nodeElement.offsetHeight;
		var scrollBy    = nodeBottom - treeBottom;
		if (scrollBy <= 0) return;
		var nodeTop     = nodeElement.offsetTop - treeElement.scrollTop - 1;
		treeElement.scrollTop += Math.min(scrollBy + 10, nodeTop);
	}

	var selectedId = tree.selectedId;
	if (
		selectedId && selectedId != -1 &&
		node.isChildOf
			(Widget_WindowManager.get().getWidget(tree.window.id, selectedId))
	) {
		this.selectNode(tree, node);
		this.focusNode(tree, node);
		var path = tree.getNodePath(node);
		var event = {
			"name"         : "SelectionChanged",
			"sourceId"     : tree.id,
			"selectedPath" : path.length? path.join("_"): ""
		};
		return event;
	}
	return false;
}