
use("Widget_WindowManager");

Class("Widget_Window");

// creating -------------------------------------------------------------------

_.init = function (windowId, width, height, isResizable) {
	var realWindow;
	if (Widget_WindowManager.get().hasWindows()) {
		if (!parseInt(width))  width  = "200";
		if (!parseInt(height)) height = "200";
		var resizable = isResizable == 1? ",resizable": "";
		realWindow = window.open(
			this.getWindowStartURL(),
			(new Date()).getTime(),
			"toolbar=0,menubar=0,width="+width+",height="+height+resizable
		);
		while (realWindow.document.readyState != "complete") {}
	} else { realWindow = window }
	realWindow.id      = windowId;
	realWindow.me      = this;
	this.id            = windowId;
	this.element       = realWindow;
	this.doc           = realWindow.document;
	this.body          = realWindow.document.body;
	this.isDirty       = true;
	this.isDestroyed   = false;
	var this2          = this;
	this.body.onunload = function () { this2.onUnload() };
	Widget_WindowManager.get().registerWindow(windowId, this);
	this.populate();
	realWindow.focus();
}

_.destroy = function () {
	if (this.isDestroyed) Throw("Window destroyed twice: " + this.getId());
	this.isDestroyed = true;
	var realWindow = this.element;
	if (realWindow && !realWindow.closed) {
		realWindow.onresize = function () {};
		var topPanel = realWindow.document.body.lastChild;
		if (topPanel && topPanel.me) topPanel.me.destroy(1);
		realWindow.document.body.innerHTML = "";
	}
	if (Widget_WindowManager.get().isMainWindow(this.id))
		Widget_WindowManager.get().closeAllButMain();
	else
		if (realWindow && !realWindow.closed) realWindow.close();
	Widget_WindowManager.get().unregisterWindow(this.id);

	this.element   = undefined;
	this.doc       = undefined;
	this.body      = undefined;
	this.scrapArea = undefined;

	if (realWindow) realWindow.me = undefined;
}

_.populate = function () {
	var scrapArea          = this.doc.createElement("div");
	scrapArea.id           = this.id + "_ScrapArea";
	scrapArea.className    = "ScrapArea";
	scrapArea.style.width  = "1000px";
	scrapArea.style.height = "1000px";
	this.body.insertBefore(scrapArea, null);
	this.scrapArea = scrapArea;
	var this2      = this;
	this.body.onresize = function () { this2.setChildrenBounds() };
}

// accessing ------------------------------------------------------------------

_.getId     = function () { return this.id }
_.getWindow = function () { return this }

_.title = function (value) { this.doc.title = value }

_.importStyle = function (value)
	{ this.doc.styleSheets[0].addImport("" + value) }

_.confirm = function (message)
	{ return this.element.confirm(message) }

// compositing ----------------------------------------------------------------

_.add = function (child) {
	this.isDirty = true;
	if (child.element.className.match(/^Panel/))
		child.element.className += " Panel_TopLevel";
	this.body.insertBefore(child.element, null);
}

// event handling -------------------------------------------------------------


_.fireEvent = function (event)
	{ Client_Application.get().fireEvent(event) }

_.onUnload = function () {
	if (this.isDestroyed) return; // in case window was destoyed from server
	this.destroy();
	// hack! should only ignore session-dont-exist and no-server errors
	try { this.fireEvent({
		"name"    : "Close",
		"sourceId": this.id
	}) } catch (e) {}
}

// layout ---------------------------------------------------------------------

_.layoutChanged = function (child, layoutBuffer) {
	this.isDirty = true;
	layoutBuffer.addDirtyWindow(this);
}

_.setChildrenBounds = function () {
	var winBody      = this.body;
	var childElement = winBody.lastChild;
	if (!childElement.id || childElement.className == "ScrapArea") return;

	var currentWidth  = winBody.clientWidth;
	var currentHeight = winBody.clientHeight;
	if (
		!this.isDirty &&
		this.width == currentWidth && this.height == currentHeight
	) return;
	var child    = childElement.me;
	this.width   = currentWidth;
	this.height  = currentHeight;
	child.setBounds(0, 0, currentWidth, currentHeight);
	this.isDirty = false;
}

_.computeMinWidth = function (element) {
	var scrap = this.scrapArea;
	var clone = element.cloneNode(true);
	scrap.insertBefore(clone, null);

	var cloneStyle   = clone.style;
	var elementStyle = element.currentStyle;
	this.prepareCloneForSizing(cloneStyle);
	var total = parseInt(clone.offsetWidth);
	total +=  
		(parseInt(elementStyle.marginLeft) ||0) +
		(parseInt(elementStyle.marginRight)||0);
	scrap.removeChild(clone);
	return total;
}

_.computeMinHeight = function (element) {
	var scrap = this.scrapArea;
	var clone = element.cloneNode(true);
	scrap.insertBefore(clone, null);

	var cloneStyle   = clone.style;
	var elementStyle = element.currentStyle;
	this.prepareCloneForSizing(cloneStyle);
	var total = parseInt(clone.offsetHeight);
	total +=  
		(parseInt(elementStyle.marginTop)   ||0) +
		(parseInt(elementStyle.marginBottom)||0);
	scrap.removeChild(clone);
	return total;
}

_.prepareCloneForSizing = function (cloneStyle) {
	cloneStyle.display  = "block";
	cloneStyle.overflow = "visible";
	cloneStyle.position = "absolute";
	cloneStyle.width    = "1px";
	cloneStyle.height   = "1px";
}

// utilities ------------------------------------------------------------------

_.getWidget = function (a, b) {
	var widgetId, win;
	if (b) {
		win = Widget_WindowManager.get().getWindow(a);
		widgetId = b;
	} else {
		win = this;
		widgetId = a;
	}

	// special widgetId for windows
	if (widgetId == -1) return this;

	var widgetElement = win.doc.getElementById(widgetId);
	if (!widgetElement) Throw("No such widget: " + win.id + ":" + widgetId);
	return widgetElement.me;
}

// testing --------------------------------------------------------------------

_.getTitle = function () { return this.doc.title }

// template methods -----------------------------------------------------------

_.getWindowStartURL = function () { return "window.html" }