
Class("Widget_WindowManager");
Class_Singleton();

_.init = function () {
	this.windows      = {};
	this.mainWindowId = null;
}

_.registerWindow = function (windowId, win) {
	if (!this.mainWindowId) this.mainWindowId = windowId;
	this.windows[windowId] = win;
}

_.unregisterWindow = function (windowId) {
	if (windowId == this.mainWindowId) this.mainWindowId = null;
	delete this.windows[windowId];
}

_.getWindow = function (windowId) {
	var win = this.windows[windowId];
	if (!win) Throw("WindowId not found: " + windowId);
	return win;
}

_.getWidget = function (windowId, widgetId) {
	if (!widgetId || widgetId == -1) return this.getWindow(windowId);
	var widget = this.getWindow(windowId).getWidget(widgetId);
	if (!widget) Throw("widgetId not found: " + widgetId);
	return widget;
}

_.hasWindows = function () { return this.mainWindowId? true: false }

_.isMainWindow = function (windowId)
	{ return (windowId == this.mainWindowId)? true: false }

_.getMainWindow = function (windowId) {
	var win = this.windows[this.mainWindowId];
	if (!win) Throw("main window not found: " + windowId);
	return win;
}

_.closeAllButMain = function () {
	var windows = this.windows;
	var windowId;
	for (windowId in windows)
		if (!this.isMainWindow(windowId))
			windows[windowId].destroy();
}

