
use ("Widget_Panel");
use ("Widget_Label");

Class("Widget_tests_Panel", "Widget_tests_TestCase");

// template methods -----------------------------------------------------------

_.getSubjectClass = function () { return Widget_Panel }

// composition tests ----------------------------------------------------------

_.testAddNoIndex= function () {
	var child1  = this.createSubject("E2", this.subject);
	var child2  = this.createSubject("E3", this.subject);
	var child3  = this.createSubject("E4", this.subject);
	this.assertEquals("1st child", "E2", this.subject.getChild(0).id);
	this.assertEquals("2nd child", "E3", this.subject.getChild(1).id);
	this.assertEquals("3rd child", "E4", this.subject.getChild(2).id);
}

_.testAddWithIndex = function () {
	var child1  = this.createSubject("E2", this.subject, 0);
	var child2  = this.createSubject("E3", this.subject, 1);
	var child3  = this.createSubject("E4", this.subject, 1);
	this.assertEquals("1st child", "E2", this.subject.getChild(0).id);
	this.assertEquals("2nd child", "E4", this.subject.getChild(1).id);
	this.assertEquals("3rd child", "E3", this.subject.getChild(2).id);
}

_.testGetIndexOfChild = function () {
	var child1  = this.createSubject("E2", this.subject);
	var child2  = this.createSubject("E3", this.subject);
	var child3  = this.createSubject("E4", this.subject);
	this.assertEquals("1st child", 0, this.subject.getIndexOfChild(child1));
	this.assertEquals("2nd child", 1, this.subject.getIndexOfChild(child2));
	this.assertEquals("3rd child", 2, this.subject.getIndexOfChild(child3));
}

_.testRemoveByIndex = function () {
	var child1  = this.createSubject("E2", this.subject);
	var child2  = this.createSubject("E3", this.subject);
	var child3  = this.createSubject("E4", this.subject);
	this.subject.removeByIndex(1);
	this.assertEquals("1st child", "E2", this.subject.getChild(0).id);
	this.assertEquals("2nd child", "E4", this.subject.getChild(1).id);
}

_.testHasChildren = function () {
	this.assertEquals("no children", false, this.subject.hasChildren());
	var child  = this.createSubject("E1", this.subject);
	this.assertEquals("yes children", true, this.subject.hasChildren());
}

// layout info tests ----------------------------------------------------------

_.testOffsetLeft = function () {
	var panel1 = this.createSubject("E2", this.subject);
	var panel2 = this.createSubject("E3", this.subject);
	this.subject.layoutManager("name", "box");
	panel1.layoutManager("name", "box");
	panel2.layoutManager("name", "box");
	var child1 = new Widget_Label("E4", panel1);
	var child2 = new Widget_Label("E5", panel2);
	var child3 = new Widget_Label("E6", panel2);
	child1.value("Hello World");
	child2.value("Hello WorldHello World");
	child3.value("Hello WorldHello WorldHello World");
	this.doLayout();
	var extra = this.subject.getBorderLeftWidth();
	this.assertEquals("parent", 0, this.subject.getOffsetLeft());
	this.assertEquals("panel 1", extra, panel1.getOffsetLeft());
	this.assertEquals
		("panel 2", extra + this.getHelloWorldWidth(), panel2.getOffsetLeft());
	this.assertEquals("child 1", extra, child1.getOffsetLeft());
	this.assertEquals
		("child 2", extra + this.getHelloWorldWidth(), child2.getOffsetLeft());
	this.assertEquals
		("child 3", extra + this.getHelloWorldWidth() * 3, child3.getOffsetLeft());
}

_.testOffsetLeftWithTrim1 = function () {
	this.subject.style("margin", "0px 0px 0px 1px");
	this.subject.style("border", "2px solid red");
	this.subject.style("padding", "0px 0px 0px 4px");
	this.subject.layoutManager("name", "box");
	var child1 = new Widget_Label("E2", this.subject);
	var child2 = new Widget_Label("E3", this.subject);
	child1.value("Hello World");
	child2.value("Hello WorldHello World");
	this.doLayout();
	this.assertEquals("parent", 0, this.subject.getOffsetLeft());
	this.assertEquals("child 1", 1 + 2 + 4, child1.getOffsetLeft());
	this.assertEquals(
		"child 2", 1 + 2 + 4 + this.getHelloWorldWidth(), child2.getOffsetLeft()
	);
}

_.testOffsetLeftWithTrim2 = function () {
	this.subject.style("margin", "0px 0px 0px 1px");
	this.subject.style("border", "2px solid red");
	this.subject.style("padding", "0px 0px 0px 4px");
	this.subject.layoutManager("name", "box");
	var child1 = new Widget_Label("E2", this.subject);
	var child2 = new Widget_Label("E3", this.subject);
	child1.value("Hello World");
	child2.value("Hello WorldHello World");
	child1.style("margin", "0px 0px 0px 8px");
	child1.style("border", "16px solid red");
	child1.style("padding", "0px 0px 0px 32px");
	this.doLayout();
	this.assertEquals("child 1", 1 + 2 + 4, child1.getOffsetLeft());
	this.assertEquals(
		"child 2",
		1 + 2 + 4 + 8 + 16*2 + 32 + this.getHelloWorldWidth(),
		child2.getOffsetLeft()
	);
}

// small size tests -----------------------------------------------------------

_.testSmallWidth1 = function () {
	this.subject.style("marginLeft",  "1px");
	this.subject.style("borderLeft",  "2px solid red");
	this.subject.style("paddingLeft", "4px");
	this.subject.setBounds(0, 0, 7, 10);
	this.assertEquals("innerWidth", 0, this.getWidgetWidth(this.subject));
}

_.testSmallWidth2 = function () {
	this.subject.style("paddingLeft", "1px");
	this.subject.style("border", "2px solid red");
	this.subject.style("marginLeft", "4px");

	this.subject.setBounds(0, 0, 6, 10);
	this.assertEquals("innerWidth 1"     , 0, this.getWidgetWidth(this.subject));
	this.assertEquals("paddingLeft 1"    , 0, this.subject.getPaddingLeft());
	this.assertEquals("borderLeftWidth 1", 2, this.subject.getBorderLeftWidth());
	this.assertEquals("marginLeft 1"     , 4, this.subject.getMarginLeft());

	this.subject.setBounds(0, 0, 5, 10);
	this.assertEquals("paddingLeft 2"    , 0, this.subject.getPaddingLeft());
	this.assertEquals("borderLeftWidth 2", 1, this.subject.getBorderLeftWidth());
	this.assertEquals("marginLeft 2"     , 4, this.subject.getMarginLeft());

	this.subject.setBounds(0, 0, 3, 10);
	this.assertEquals("paddingLeft 3"    , 0, this.subject.getPaddingLeft());
	this.assertEquals("borderLeftWidth 3", 0, this.subject.getBorderLeftWidth());
	this.assertEquals("marginLeft 3"     , 3, this.subject.getMarginLeft());

	this.subject.setBounds(0, 0, 0, 10);
	this.assertEquals("paddingLeft 4"    , 0, this.subject.getPaddingLeft());
	this.assertEquals("borderLeftWidth 4", 0, this.subject.getBorderLeftWidth());
	this.assertEquals("marginLeft 4"     , 0, this.subject.getMarginLeft());

	this.subject.setBounds(0, 0, 10, 10);
	this.assertEquals("innerWidth 2"     , 1, this.getWidgetWidth(this.subject));
}